@@ -8,7 +8,7 @@
 #![allow(clippy::print_stdout)]
 #![allow(clippy::single_char_pattern)]
 #![allow(clippy::unwrap_used)]
-#![allow(clippy::unchecked_duration_subtraction)]
+#![allow(clippy::unchecked_time_subtraction)]
 #![allow(clippy::useless_vec)]
 #![allow(clippy::needless_pass_by_value)]
 //! <!-- @@ end example lint list maintained by maint/add_warning @@ -->
@@ -284,6 +284,10 @@ where
     fn set_tcp_notsent_lowat(&self, notsent_lowat: u32) -> IoResult<()> {
         self.inner.set_tcp_notsent_lowat(notsent_lowat)
     }
+
+    fn new_handle(&self) -> Box<dyn StreamOps + Send + Unpin> {
+        self.inner.new_handle()
+    }
 }
 
 impl<T> Drop for CustomTcpStream<T> {
