@@ -13,27 +13,28 @@ use crate::address::{IntoTorAddr, ResolveInstructions, StreamInstructions};
 use crate::config::{
     ClientAddrConfig, SoftwareStatusOverrideConfig, StreamTimeoutConfig, TorClientConfig,
 };
-use safelog::{sensitive, Sensitive};
+use safelog::{Sensitive, sensitive};
 use tor_async_utils::{DropNotifyWatchSender, PostageWatchSenderExt};
+use tor_chanmgr::ChanMgrConfig;
+use tor_circmgr::ClientDataTunnel;
 use tor_circmgr::isolation::{Isolation, StreamIsolation};
-use tor_circmgr::{isolation::StreamIsolationBuilder, IsolationToken, TargetPort};
+use tor_circmgr::{IsolationToken, TargetPort, isolation::StreamIsolationBuilder};
 use tor_config::MutCfg;
 #[cfg(feature = "bridge-client")]
 use tor_dirmgr::bridgedesc::BridgeDescMgr;
 use tor_dirmgr::{DirMgrStore, Timeliness};
-use tor_error::{error_report, internal, Bug};
+use tor_error::{Bug, error_report, internal};
 use tor_guardmgr::{GuardMgr, RetireCircuits};
 use tor_keymgr::Keystore;
 use tor_memquota::MemoryQuotaTracker;
-use tor_netdir::{params::NetParameters, NetDirProvider};
+use tor_netdir::{NetDirProvider, params::NetParameters};
 #[cfg(feature = "onion-service-service")]
 use tor_persist::state_dir::StateDirectory;
 use tor_persist::{FsStateMgr, StateMgr};
-use tor_proto::circuit::ClientCirc;
-use tor_proto::stream::{DataStream, IpVersionPreference, StreamParameters};
+use tor_proto::client::stream::{DataStream, IpVersionPreference, StreamParameters};
 #[cfg(all(
     any(feature = "native-tls", feature = "rustls"),
-    any(feature = "async-std", feature = "tokio")
+    any(feature = "async-std", feature = "tokio"),
 ))]
 use tor_rtcompat::PreferredRuntime;
 use tor_rtcompat::{Runtime, SleepProviderExt};
@@ -50,7 +51,7 @@ use tor_hsservice::HsIdKeypairSpecifier;
 #[cfg(all(feature = "onion-service-client", feature = "experimental-api"))]
 use {tor_hscrypto::pk::HsId, tor_hscrypto::pk::HsIdKeypair, tor_keymgr::KeystoreSelector};
 
-use tor_keymgr::{config::ArtiKeystoreKind, ArtiNativeKeystore, KeyMgr, KeyMgrBuilder};
+use tor_keymgr::{ArtiNativeKeystore, KeyMgr, KeyMgrBuilder, config::ArtiKeystoreKind};
 
 #[cfg(feature = "ephemeral-keystore")]
 use tor_keymgr::ArtiEphemeralKeystore;
@@ -58,19 +59,19 @@ use tor_keymgr::ArtiEphemeralKeystore;
 #[cfg(feature = "ctor-keystore")]
 use tor_keymgr::{CTorClientKeystore, CTorServiceKeystore};
 
-use futures::lock::Mutex as AsyncMutex;
-use futures::task::SpawnExt;
 use futures::StreamExt as _;
+use futures::lock::Mutex as AsyncMutex;
 use std::net::IpAddr;
 use std::result::Result as StdResult;
 use std::sync::{Arc, Mutex};
+use tor_rtcompat::SpawnExt;
 
 use crate::err::ErrorDetail;
-use crate::{status, util, TorClientBuilder};
+use crate::{TorClientBuilder, status, util};
 #[cfg(feature = "geoip")]
 use tor_geoip::CountryCode;
 use tor_rtcompat::scheduler::TaskHandle;
-use tracing::{debug, info};
+use tracing::{debug, info, instrument};
 
 /// An active client session on the Tor network.
 ///
@@ -251,7 +252,9 @@ impl InertTorClient {
 
                 let native_store =
                     ArtiNativeKeystore::from_path_and_mistrust(&key_store_dir, permissions)?;
-                info!("Using keystore from {key_store_dir:?}");
+                // Should only log fs paths at debug level or lower,
+                // unless they're part of a diagnostic message.
+                debug!("Using keystore from {key_store_dir:?}");
 
                 Box::new(native_store)
             }
@@ -351,14 +354,6 @@ impl InertTorClient {
         feature = "experimental-api",
         feature = "keymgr"
     ))]
-    #[cfg_attr(
-        docsrs,
-        doc(cfg(all(
-            feature = "onion-service-client",
-            feature = "experimental-api",
-            feature = "keymgr"
-        )))
-    )]
     pub fn rotate_service_discovery_key(
         &self,
         selector: KeystoreSelector,
@@ -483,6 +478,34 @@ impl InertTorClient {
             action: "get key manager handle",
         })?)
     }
+
+    /// Create (but do not launch) a new
+    /// [`OnionService`](tor_hsservice::OnionService)
+    /// using the given configuration.
+    ///
+    /// See [`TorClient::create_onion_service`].
+    #[cfg(feature = "onion-service-service")]
+    #[instrument(skip_all, level = "trace")]
+    pub fn create_onion_service(
+        &self,
+        config: &TorClientConfig,
+        svc_config: tor_hsservice::OnionServiceConfig,
+    ) -> crate::Result<tor_hsservice::OnionService> {
+        let keymgr = self.keymgr.as_ref().ok_or(ErrorDetail::KeystoreRequired {
+            action: "create onion service",
+        })?;
+
+        let (state_dir, mistrust) = config.state_dir()?;
+        let state_dir =
+            self::StateDirectory::new(state_dir, mistrust).map_err(ErrorDetail::StateAccess)?;
+
+        Ok(tor_hsservice::OnionService::builder()
+            .config(svc_config)
+            .keymgr(keymgr.clone())
+            .state_dir(state_dir)
+            .build()
+            .map_err(ErrorDetail::OnionServiceSetup)?)
+    }
 }
 
 /// Preferences for whether a [`TorClient`] should bootstrap on its own or not.
@@ -615,7 +638,6 @@ impl StreamPrefs {
     /// When this option is set, we will only pick exit relays that
     /// have an IP address that matches the country in our GeoIP database.
     #[cfg(feature = "geoip")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "geoip")))]
     pub fn exit_country(&mut self, country_code: CountryCode) -> &mut Self {
         self.country_code = Some(country_code);
         self
@@ -668,14 +690,7 @@ impl StreamPrefs {
     /// If `Explicit(true)`, Onion Service connections are enabled.
     ///
     /// If `Auto`, the behaviour depends on the `address_filter.allow_onion_addrs`
-    /// configuration option, which is in turn **disabled** by default.
-    ///
-    /// **Note**: Arti currently lacks the
-    /// "vanguards" feature that Tor uses to prevent guard discovery attacks over time.
-    /// As such, you should probably stick with C Tor if you need to make a large
-    /// number of onion service connections, or if you are using the Tor protocol
-    /// in a way that lets an attacker control how many onion services connections that you make -
-    /// for example, when using Arti's SOCKS support from a web browser such as Tor Browser.
+    /// configuration option, which is in turn enabled by default.
     #[cfg(feature = "onion-service-client")]
     pub fn connect_to_onion_services(
         &mut self,
@@ -848,6 +863,7 @@ impl<R: Runtime> TorClient<R> {
 
     /// Implementation of `create_unbootstrapped`, split out in order to avoid manually specifying
     /// double error conversions.
+    #[instrument(skip_all, level = "trace")]
     pub(crate) fn create_inner(
         runtime: R,
         config: &TorClientConfig,
@@ -880,6 +896,7 @@ impl<R: Runtime> TorClient<R> {
         let statemgr = FsStateMgr::from_path_and_mistrust(&state_dir, mistrust)
             .map_err(ErrorDetail::StateMgrSetup)?;
         // Try to take state ownership early, so we'll know if we have it.
+        // Note that this `try_lock()` may return `Ok` even if we can't acquire the lock.
         // (At this point we don't yet care if we have it.)
         let _ignore_status = statemgr.try_lock().map_err(ErrorDetail::StateMgrSetup)?;
 
@@ -891,7 +908,7 @@ impl<R: Runtime> TorClient<R> {
         };
         let chanmgr = Arc::new(tor_chanmgr::ChanMgr::new(
             runtime.clone(),
-            &config.channel,
+            ChanMgrConfig::new(config.channel.clone()),
             dormant.into(),
             &NetParameters::from_map(&config.override_net_params),
             memquota.clone(),
@@ -1086,6 +1103,7 @@ impl<R: Runtime> TorClient<R> {
     ///
     /// If the bootstrapping process fails, returns an error. This function can safely be called
     /// again later to attempt to bootstrap another time.
+    #[instrument(skip_all, level = "trace")]
     pub async fn bootstrap(&self) -> crate::Result<()> {
         self.bootstrap_inner().await.map_err(ErrorDetail::into)
     }
@@ -1160,6 +1178,7 @@ impl<R: Runtime> TorClient<R> {
     ///
     /// Check whether a bootstrap is in progress; if one is, wait until it finishes
     /// and then return. (Otherwise, return immediately.)
+    #[instrument(skip_all, level = "trace")]
     async fn wait_for_bootstrap(&self) -> StdResult<(), ErrorDetail> {
         match self.should_bootstrap {
             BootstrapBehavior::OnDemand => {
@@ -1208,6 +1227,7 @@ impl<R: Runtime> TorClient<R> {
     /// Changing some options do not take effect immediately on all open streams
     /// and circuits, but rather affect only future streams and circuits.  Those
     /// are also explicitly documented.
+    #[instrument(skip_all, level = "trace")]
     pub fn reconfigure(
         &self,
         new_config: &TorClientConfig,
@@ -1243,6 +1263,7 @@ impl<R: Runtime> TorClient<R> {
     /// This is split out from `reconfigure` so we can do the all-or-nothing
     /// check without recursion. the caller to this method must hold the
     /// `reconfigure_lock`.
+    #[instrument(level = "trace", skip_all)]
     fn reconfigure_inner(
         &self,
         new_config: &TorClientConfig,
@@ -1389,6 +1410,7 @@ impl<R: Runtime> TorClient<R> {
     /// # Ok(())
     /// # }
     /// ```
+    #[instrument(skip_all, level = "trace")]
     pub async fn connect<A: IntoTorAddr>(&self, target: A) -> crate::Result<DataStream> {
         self.connect_with_prefs(target, &self.connect_prefs).await
     }
@@ -1399,6 +1421,7 @@ impl<R: Runtime> TorClient<R> {
     /// Note that because Tor prefers to do DNS resolution on the remote
     /// side of the network, this function takes its address as a string.
     /// (See [`TorClient::connect()`] for more information.)
+    #[instrument(skip_all, level = "trace")]
     pub async fn connect_with_prefs<A: IntoTorAddr>(
         &self,
         target: A,
@@ -1406,19 +1429,41 @@ impl<R: Runtime> TorClient<R> {
     ) -> crate::Result<DataStream> {
         let addr = target.into_tor_addr().map_err(wrap_err)?;
         let mut stream_parameters = prefs.stream_parameters();
+        // This macro helps prevent code duplication in the match below.
+        //
+        // Ideally, the match should resolve to a tuple consisting of the
+        // tunnel, and the address, port and stream params,
+        // but that's not currently possible because
+        // the Exit and Hs branches use different tunnel types.
+        //
+        // TODO: replace with an async closure (when our MSRV allows it),
+        // or with a more elegant approach.
+        macro_rules! begin_stream {
+            ($tunnel:expr, $addr:expr, $port:expr, $stream_params:expr) => {{
+                let fut = $tunnel.begin_stream($addr, $port, $stream_params);
+                self.runtime
+                    .timeout(self.timeoutcfg.get().connect_timeout, fut)
+                    .await
+                    .map_err(|_| ErrorDetail::ExitTimeout)?
+                    .map_err(|cause| ErrorDetail::StreamFailed {
+                        cause,
+                        kind: "data",
+                    })
+            }};
+        }
 
-        let (circ, addr, port) = match addr.into_stream_instructions(&self.addrcfg.get(), prefs)? {
+        let stream = match addr.into_stream_instructions(&self.addrcfg.get(), prefs)? {
             StreamInstructions::Exit {
                 hostname: addr,
                 port,
             } => {
                 let exit_ports = [prefs.wrap_target_port(port)];
-                let circ = self
-                    .get_or_launch_exit_circ(&exit_ports, prefs)
+                let tunnel = self
+                    .get_or_launch_exit_tunnel(&exit_ports, prefs)
                     .await
                     .map_err(wrap_err)?;
                 debug!("Got a circuit for {}:{}", sensitive(&addr), port);
-                (circ, addr, port)
+                begin_stream!(tunnel, &addr, port, Some(stream_parameters))
             }
 
             #[cfg(not(feature = "onion-service-client"))]
@@ -1435,6 +1480,8 @@ impl<R: Runtime> TorClient<R> {
                 hostname,
                 port,
             } => {
+                use safelog::DisplayRedacted as _;
+
                 self.wait_for_bootstrap().await?;
                 let netdir = self.netdir(Timeliness::Timely, "connect to a hidden service")?;
 
@@ -1447,7 +1494,10 @@ impl<R: Runtime> TorClient<R> {
                         keymgr.get::<HsClientDescEncKeypair>(&desc_enc_key_spec)?;
 
                     if let Some(ks_hsc_desc_enc) = ks_hsc_desc_enc {
-                        debug!("Found descriptor decryption key for {hsid}");
+                        debug!(
+                            "Found descriptor decryption key for {}",
+                            hsid.display_redacted()
+                        );
                         hs_client_secret_keys_builder.ks_hsc_desc_enc(ks_hsc_desc_enc);
                     }
                 };
@@ -1456,19 +1506,16 @@ impl<R: Runtime> TorClient<R> {
                     .build()
                     .map_err(ErrorDetail::Configuration)?;
 
-                let circ = self
+                let tunnel = self
                     .hsclient
-                    .get_or_launch_circuit(
+                    .get_or_launch_tunnel(
                         &netdir,
                         hsid,
                         hs_client_secret_keys,
                         self.isolation(prefs),
                     )
                     .await
-                    .map_err(|cause| ErrorDetail::ObtainHsCircuit {
-                        cause,
-                        hsid: hsid.into(),
-                    })?;
+                    .map_err(|cause| ErrorDetail::ObtainHsCircuit { cause, hsid })?;
                 // On connections to onion services, we have to suppress
                 // everything except the port from the BEGIN message.  We also
                 // disable optimistic data.
@@ -1476,23 +1523,12 @@ impl<R: Runtime> TorClient<R> {
                     .suppress_hostname()
                     .suppress_begin_flags()
                     .optimistic(false);
-                (circ, hostname, port)
+
+                begin_stream!(tunnel, &hostname, port, Some(stream_parameters))
             }
         };
 
-        let stream_future = circ.begin_stream(&addr, port, Some(stream_parameters));
-        // This timeout is needless but harmless for optimistic streams.
-        let stream = self
-            .runtime
-            .timeout(self.timeoutcfg.get().connect_timeout, stream_future)
-            .await
-            .map_err(|_| ErrorDetail::ExitTimeout)?
-            .map_err(|cause| ErrorDetail::StreamFailed {
-                cause,
-                kind: "data",
-            })?;
-
-        Ok(stream)
+        Ok(stream?)
     }
 
     /// Sets the default preferences for future connections made with this client.
@@ -1519,11 +1555,13 @@ impl<R: Runtime> TorClient<R> {
     }
 
     /// On success, return a list of IP addresses.
+    #[instrument(skip_all, level = "trace")]
     pub async fn resolve(&self, hostname: &str) -> crate::Result<Vec<IpAddr>> {
         self.resolve_with_prefs(hostname, &self.connect_prefs).await
     }
 
     /// On success, return a list of IP addresses, but use prefs.
+    #[instrument(skip_all, level = "trace")]
     pub async fn resolve_with_prefs(
         &self,
         hostname: &str,
@@ -1536,7 +1574,7 @@ impl<R: Runtime> TorClient<R> {
 
         match addr.into_resolve_instructions(&self.addrcfg.get(), prefs)? {
             ResolveInstructions::Exit(hostname) => {
-                let circ = self.get_or_launch_exit_circ(&[], prefs).await?;
+                let circ = self.get_or_launch_exit_tunnel(&[], prefs).await?;
 
                 let resolve_future = circ.resolve(&hostname);
                 let addrs = self
@@ -1558,6 +1596,7 @@ impl<R: Runtime> TorClient<R> {
     /// Perform a remote DNS reverse lookup with the provided IP address.
     ///
     /// On success, return a list of hostnames.
+    #[instrument(skip_all, level = "trace")]
     pub async fn resolve_ptr(&self, addr: IpAddr) -> crate::Result<Vec<String>> {
         self.resolve_ptr_with_prefs(addr, &self.connect_prefs).await
     }
@@ -1565,12 +1604,13 @@ impl<R: Runtime> TorClient<R> {
     /// Perform a remote DNS reverse lookup with the provided IP address.
     ///
     /// On success, return a list of hostnames.
+    #[instrument(level = "trace", skip_all)]
     pub async fn resolve_ptr_with_prefs(
         &self,
         addr: IpAddr,
         prefs: &StreamPrefs,
     ) -> crate::Result<Vec<String>> {
-        let circ = self.get_or_launch_exit_circ(&[], prefs).await?;
+        let circ = self.get_or_launch_exit_tunnel(&[], prefs).await?;
 
         let resolve_ptr_future = circ.resolve_ptr(addr);
         let hostnames = self
@@ -1662,17 +1702,18 @@ impl<R: Runtime> TorClient<R> {
 
     /// Get or launch an exit-suitable circuit with a given set of
     /// exit ports.
-    async fn get_or_launch_exit_circ(
+    #[instrument(skip_all, level = "trace")]
+    async fn get_or_launch_exit_tunnel(
         &self,
         exit_ports: &[TargetPort],
         prefs: &StreamPrefs,
-    ) -> StdResult<Arc<ClientCirc>, ErrorDetail> {
+    ) -> StdResult<ClientDataTunnel, ErrorDetail> {
         // TODO HS probably this netdir ought to be made in connect_with_prefs
         // like for StreamInstructions::Hs.
         self.wait_for_bootstrap().await?;
         let dir = self.netdir(Timeliness::Timely, "build a circuit")?;
 
-        let circ = self
+        let tunnel = self
             .circmgr
             .get_or_launch_exit(
                 dir.as_ref().into(),
@@ -1688,7 +1729,7 @@ impl<R: Runtime> TorClient<R> {
             })?;
         drop(dir); // This decreases the refcount on the netdir.
 
-        Ok(circ)
+        Ok(tunnel)
     }
 
     /// Return an overall [`Isolation`] for this `TorClient` and a `StreamPrefs`.
@@ -1713,6 +1754,8 @@ impl<R: Runtime> TorClient<R> {
 
     /// Try to launch an onion service with a given configuration.
     ///
+    /// Returns `Ok(None)` if the service specified is disabled in the config.
+    ///
     /// This onion service will not actually handle any requests on its own: you
     /// will need to
     /// pull [`RendRequest`](tor_hsservice::RendRequest) objects from the returned stream,
@@ -1725,13 +1768,26 @@ impl<R: Runtime> TorClient<R> {
     /// If you want to forward all the requests from an onion service to a set
     /// of local ports, you may want to use the `tor-hsrproxy` crate.
     #[cfg(feature = "onion-service-service")]
+    #[instrument(skip_all, level = "trace")]
     pub fn launch_onion_service(
         &self,
         config: tor_hsservice::OnionServiceConfig,
-    ) -> crate::Result<(
-        Arc<tor_hsservice::RunningOnionService>,
-        impl futures::Stream<Item = tor_hsservice::RendRequest>,
-    )> {
+    ) -> crate::Result<
+        Option<(
+            Arc<tor_hsservice::RunningOnionService>,
+            impl futures::Stream<Item = tor_hsservice::RendRequest> + use<R>,
+        )>,
+    > {
+        let nickname = config.nickname();
+
+        if !config.enabled() {
+            info!(
+                nickname=%nickname,
+                "Skipping onion service because it was disabled in the config"
+            );
+            return Ok(None);
+        }
+
         let keymgr = self
             .inert_client
             .keymgr
@@ -1749,16 +1805,14 @@ impl<R: Runtime> TorClient<R> {
             .state_dir(state_dir)
             .build()
             .map_err(ErrorDetail::LaunchOnionService)?;
-        let (service, stream) = service
+        Ok(service
             .launch(
                 self.runtime.clone(),
                 self.dirmgr.clone().upcast_arc(),
                 self.hs_circ_pool.clone(),
                 Arc::clone(&self.path_resolver),
             )
-            .map_err(ErrorDetail::LaunchOnionService)?;
-
-        Ok((service, stream))
+            .map_err(ErrorDetail::LaunchOnionService)?)
     }
 
     /// Try to launch an onion service with a given configuration and provided
@@ -1766,6 +1820,8 @@ impl<R: Runtime> TorClient<R> {
     /// associated `HsIdKeypair`  in this `TorClient`'s `KeyMgr`, then this operation
     /// fails rather than overwriting the existing key.
     ///
+    /// Returns `Ok(None)` if the service specified is disabled in the config.
+    ///
     /// The specified `HsIdKeypair` will be inserted in the primary keystore.
     ///
     /// **Important**: depending on the configuration of your
@@ -1786,14 +1842,17 @@ impl<R: Runtime> TorClient<R> {
     /// If you want to forward all the requests from an onion service to a set
     /// of local ports, you may want to use the `tor-hsrproxy` crate.
     #[cfg(all(feature = "onion-service-service", feature = "experimental-api"))]
+    #[instrument(skip_all, level = "trace")]
     pub fn launch_onion_service_with_hsid(
         &self,
         config: tor_hsservice::OnionServiceConfig,
         id_keypair: HsIdKeypair,
-    ) -> crate::Result<(
-        Arc<tor_hsservice::RunningOnionService>,
-        impl futures::Stream<Item = tor_hsservice::RendRequest>,
-    )> {
+    ) -> crate::Result<
+        Option<(
+            Arc<tor_hsservice::RunningOnionService>,
+            impl futures::Stream<Item = tor_hsservice::RendRequest> + use<R>,
+        )>,
+    > {
         let nickname = config.nickname();
         let hsid_spec = HsIdKeypairSpecifier::new(nickname.clone());
         let selector = KeystoreSelector::Primary;
@@ -1838,14 +1897,6 @@ impl<R: Runtime> TorClient<R> {
         feature = "experimental-api",
         feature = "keymgr"
     ))]
-    #[cfg_attr(
-        docsrs,
-        doc(cfg(all(
-            feature = "onion-service-client",
-            feature = "experimental-api",
-            feature = "keymgr"
-        )))
-    )]
     pub fn generate_service_discovery_key(
         &self,
         selector: KeystoreSelector,
@@ -1994,28 +2045,31 @@ impl<R: Runtime> TorClient<R> {
     /// [`OnionService`](tor_hsservice::OnionService)
     /// using the given configuration.
     ///
+    /// This is useful for managing an onion service without needing to start a `TorClient` or the
+    /// onion service itself.
+    /// If you only wish to run the onion service, see
+    /// [`TorClient::launch_onion_service()`]
+    /// which allows you to launch an onion service from a running `TorClient`.
+    ///
     /// The returned `OnionService` can be launched using
     /// [`OnionService::launch()`](tor_hsservice::OnionService::launch).
+    /// Note that `launch()` requires a [`NetDirProvider`],
+    /// [`HsCircPool`](tor_circmgr::hspool::HsCircPool), etc,
+    /// which you should obtain from a running `TorClient`.
+    /// But these are only accessible from a `TorClient` if the "experimental-api" feature is
+    /// enabled.
+    /// The behaviour is not specified if you create the `OnionService` with
+    /// `create_onion_service()` using one [`TorClientConfig`],
+    /// but launch it using a `TorClient` generated from a different `TorClientConfig`.
+    // TODO #2249: Look into this behaviour more, and possibly error if there is a different config.
     #[cfg(feature = "onion-service-service")]
+    #[instrument(skip_all, level = "trace")]
     pub fn create_onion_service(
         config: &TorClientConfig,
         svc_config: tor_hsservice::OnionServiceConfig,
     ) -> crate::Result<tor_hsservice::OnionService> {
         let inert_client = InertTorClient::new(config)?;
-        let keymgr = inert_client.keymgr.ok_or(ErrorDetail::KeystoreRequired {
-            action: "create onion service",
-        })?;
-
-        let (state_dir, mistrust) = config.state_dir()?;
-        let state_dir =
-            self::StateDirectory::new(state_dir, mistrust).map_err(ErrorDetail::StateAccess)?;
-
-        Ok(tor_hsservice::OnionService::builder()
-            .config(svc_config)
-            .keymgr(keymgr)
-            .state_dir(state_dir)
-            .build()
-            .map_err(ErrorDetail::OnionServiceSetup)?)
+        inert_client.create_onion_service(config, svc_config)
     }
 
     /// Return a current [`status::BootstrapStatus`] describing how close this client
@@ -2050,10 +2104,13 @@ impl<R: Runtime> TorClient<R> {
             .borrow_mut() = Some(mode);
     }
 
-    /// Return a [`Future`](futures::Future) which resolves
+    /// Return a [`Future`] which resolves
     /// once this TorClient has stopped.
     #[cfg(feature = "experimental-api")]
-    pub fn wait_for_stop(&self) -> impl futures::Future<Output = ()> + Send + Sync + 'static {
+    #[instrument(skip_all, level = "trace")]
+    pub fn wait_for_stop(
+        &self,
+    ) -> impl futures::Future<Output = ()> + Send + Sync + 'static + use<R> {
         // We defer to the "wait for unlock" handle on our statemgr.
         //
         // The statemgr won't actually be unlocked until it is finally
@@ -2061,6 +2118,12 @@ impl<R: Runtime> TorClient<R> {
         // dropped—which is what we want.
         self.statemgr.wait_for_unlock()
     }
+
+    /// Getter for keymgr.
+    #[cfg(feature = "onion-service-cli-extra")]
+    pub fn keymgr(&self) -> crate::Result<&KeyMgr> {
+        self.inert_client.keymgr()
+    }
 }
 
 /// Monitor `dormant_mode` and enable/disable periodic tasks as applicable
@@ -2121,7 +2184,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -2146,6 +2209,19 @@ mod test {
                 .create_unbootstrapped()
                 .unwrap();
         });
+        tor_rtcompat::test_with_one_runtime!(|rt| async {
+            let state_dir = tempfile::tempdir().unwrap();
+            let cache_dir = tempfile::tempdir().unwrap();
+            let cfg = TorClientConfigBuilder::from_directories(state_dir, cache_dir)
+                .build()
+                .unwrap();
+            let _ = TorClient::with_runtime(rt)
+                .config(cfg)
+                .bootstrap_behavior(BootstrapBehavior::Manual)
+                .create_unbootstrapped_async()
+                .await
+                .unwrap();
+        });
     }
 
     #[test]
@@ -2156,6 +2232,7 @@ mod test {
             let cfg = TorClientConfigBuilder::from_directories(state_dir, cache_dir)
                 .build()
                 .unwrap();
+            // Test sync
             let client = TorClient::with_runtime(rt)
                 .config(cfg)
                 .bootstrap_behavior(BootstrapBehavior::Manual)
@@ -2165,6 +2242,25 @@ mod test {
             assert!(result.is_err());
             assert_eq!(result.err().unwrap().kind(), ErrorKind::BootstrapRequired);
         });
+        // Need a separate test for async because Runtime and TorClientConfig are consumed by the
+        // builder
+        tor_rtcompat::test_with_one_runtime!(|rt| async {
+            let state_dir = tempfile::tempdir().unwrap();
+            let cache_dir = tempfile::tempdir().unwrap();
+            let cfg = TorClientConfigBuilder::from_directories(state_dir, cache_dir)
+                .build()
+                .unwrap();
+            // Test sync
+            let client = TorClient::with_runtime(rt)
+                .config(cfg)
+                .bootstrap_behavior(BootstrapBehavior::Manual)
+                .create_unbootstrapped_async()
+                .await
+                .unwrap();
+            let result = client.connect("example.com:80").await;
+            assert!(result.is_err());
+            assert_eq!(result.err().unwrap().kind(), ErrorKind::BootstrapRequired);
+        });
     }
 
     #[test]
@@ -2261,5 +2357,21 @@ mod test {
                 .reconfigure(&cfg, Reconfigure::AllOrNothing)
                 .unwrap();
         });
+        tor_rtcompat::test_with_one_runtime!(|rt| async {
+            let state_dir = tempfile::tempdir().unwrap();
+            let cache_dir = tempfile::tempdir().unwrap();
+            let cfg = TorClientConfigBuilder::from_directories(state_dir, cache_dir)
+                .build()
+                .unwrap();
+            let tor_client = TorClient::with_runtime(rt)
+                .config(cfg.clone())
+                .bootstrap_behavior(BootstrapBehavior::Manual)
+                .create_unbootstrapped_async()
+                .await
+                .unwrap();
+            tor_client
+                .reconfigure(&cfg, Reconfigure::AllOrNothing)
+                .unwrap();
+        });
     }
 }
