@@ -137,6 +137,7 @@ pub(crate) enum ResponseBody {
 impl ResponseBody {
     /// Return true if this body type indicates that no future responses will be
     /// sent for this request.
+    #[cfg(test)]
     pub(crate) fn is_final(&self) -> bool {
         match self {
             ResponseBody::Error(_) | ResponseBody::Success(_) => true,
@@ -177,7 +178,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -288,7 +289,7 @@ mod test {
             r#"{ "id": 3, "obj": "hello", "meta": { "updates": 3}, "method": "x-test:dummy", "params": {} }"#
         );
         expect_err!(
-            RPE::MethodNotFound,
+            RPE::NoSuchMethod,
             r#"{ "id": 3, "obj": "hello", "method": "arti:this-is-not-a-method", "params": {} }"#
         );
         expect_err!(
@@ -316,12 +317,10 @@ mod test {
         // exact order.
         assert_eq!(s, r#"{"id":7,"result":{"hello":99,"world":"foo"}}"#);
 
-        let resp = BoxedResponse {
-            id: None,
-            body: ResponseBody::Error(Box::new(rpc::RpcError::from(
-                crate::err::RequestParseError::IdMissing,
-            ))),
-        };
+        let resp = BoxedResponse::from_error(
+            None,
+            rpc::RpcError::from(crate::err::RequestParseError::IdMissing),
+        );
         let s = serde_json::to_string(&resp).unwrap();
         // NOTE: as above.
         assert_eq!(
@@ -329,4 +328,25 @@ mod test {
             r#"{"error":{"message":"Request did not have any `id` field.","code":-32600,"kinds":["rpc:InvalidRequest"]}}"#
         );
     }
+
+    #[test]
+    fn response_body_is_final() {
+        let response_body_error = ResponseBody::from(rpc::RpcError::new(
+            "This is a test!".to_string(),
+            rpc::RpcErrorKind::ObjectNotFound,
+        ));
+        assert!(response_body_error.is_final());
+
+        let response_body_success = ResponseBody::Success(Box::new(DummyResponse {
+            hello: 99,
+            world: "foo".into(),
+        }));
+        assert!(response_body_success.is_final());
+
+        let response_body_update = ResponseBody::Update(Box::new(DummyResponse {
+            hello: 99,
+            world: "foo".into(),
+        }));
+        assert!(!response_body_update.is_final());
+    }
 }
