@@ -78,7 +78,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -89,6 +89,8 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 #![allow(clippy::print_stderr)] // Allowed in this crate only.
@@ -101,13 +103,13 @@ mod traces;
 
 use arti::ArtiCombinedConfig;
 use arti_client::TorClient;
-use futures::task::SpawnExt;
 use rt::badtcp::BrokenTcpProvider;
 use tor_config::ConfigurationSources;
 use tor_dirmgr::filter::DirFilter;
+use tor_rtcompat::SpawnExt;
 use tor_rtcompat::{PreferredRuntime, Runtime, RuntimeSubstExt as _, SleepProviderExt};
 
-use anyhow::{anyhow, Result};
+use anyhow::{Result, anyhow};
 use tracing_subscriber::prelude::*;
 //use std::path::PathBuf;
 use std::str::FromStr;
