@@ -42,6 +42,7 @@
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
 #![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 // TODO #1645 (either remove this, or decide to have it everywhere)
@@ -50,8 +51,6 @@
 #![allow(clippy::print_stderr)]
 #![allow(clippy::print_stdout)]
 
-pub mod cfg;
-pub mod logging;
 #[cfg(not(feature = "onion-service-service"))]
 mod onion_proxy_disabled;
 
@@ -83,10 +82,29 @@ macro_rules! semipublic_mod {
     }
 }
 
+/// Helper:
+/// Import a set of items as public if experimental_api is set,
+/// and as non-public otherwise.
+macro_rules! semipublic_use {
+    {
+        $dflt_vis:vis use $($tok:tt)+
+    } => {
+        cfg_if::cfg_if! {
+            if #[cfg(feature="experimental-api")] {
+                pub use $($tok)+
+            } else {
+                $dflt_vis use $($tok)+
+            }
+        }
+    }
+}
+
 semipublic_mod! {
+    mod cfg;
     #[cfg(feature = "dns-proxy")]
     mod dns;
     mod exit;
+    mod logging;
     #[cfg(feature="onion-service-service")]
     mod onion_proxy;
     mod process;
@@ -99,12 +117,16 @@ mod rpc;
 use std::ffi::OsString;
 use std::fmt::Write;
 
-pub use cfg::{
-    ARTI_EXAMPLE_CONFIG, ApplicationConfig, ApplicationConfigBuilder, ArtiCombinedConfig,
-    ArtiConfig, ArtiConfigBuilder, ProxyConfig, ProxyConfigBuilder, SystemConfig,
-    SystemConfigBuilder,
-};
-pub use logging::{LoggingConfig, LoggingConfigBuilder};
+semipublic_use! {
+    use cfg::{
+        ARTI_EXAMPLE_CONFIG, ApplicationConfig, ApplicationConfigBuilder, ArtiCombinedConfig,
+        ArtiConfig, ArtiConfigBuilder, ProxyConfig, ProxyConfigBuilder, SystemConfig,
+        SystemConfigBuilder,
+    };
+}
+semipublic_use! {
+    use logging::{LoggingConfig, LoggingConfigBuilder};
+}
 
 use arti_client::TorClient;
 use arti_client::config::default_config_files;
@@ -127,7 +149,7 @@ use clap::Subcommand as _;
 
 #[cfg(feature = "experimental-api")]
 #[cfg_attr(docsrs, doc(cfg(feature = "experimental-api")))]
-pub use subcommands::proxy::run_proxy as run;
+pub use subcommands::proxy::run_proxy;
 
 /// Create a runtime for Arti to use.
 fn create_runtime() -> std::io::Result<impl ToplevelRuntime> {
@@ -204,9 +226,13 @@ where
         write!(config_file_help, " Defaults to {:?}", default).expect("Can't write to string");
     }
 
-    // We create the runtime now so that we can use its `Debug` impl to describe it for
-    // the version string.
     let runtime = create_runtime()?;
+
+    // Configure tor-log-ratelim early before we begin logging.
+    tor_log_ratelim::install_runtime(runtime.clone())
+        .context("Failed to initialze tor-log-ratelim")?;
+
+    // Use the runtime's `Debug` impl to describe it for the version string.
     let features = list_enabled_features();
     let long_version = format!(
         "{}\nusing runtime: {:?}\noptional features: {}",
