@@ -1,20 +1,20 @@
 //! Configure and implement onion service reverse-proxy feature.
 
 use std::{
-    collections::{btree_map::Entry, BTreeMap, HashSet},
+    collections::{BTreeMap, HashSet, btree_map::Entry},
     sync::{Arc, Mutex},
 };
 
 use arti_client::config::onion_service::{OnionServiceConfig, OnionServiceConfigBuilder};
-use futures::task::SpawnExt;
+use futures::StreamExt as _;
 use tor_config::{
-    define_list_builder_helper, impl_standard_builder, ConfigBuildError, Flatten, Reconfigure,
-    ReconfigureError,
+    ConfigBuildError, Flatten, Reconfigure, ReconfigureError, define_list_builder_helper,
+    impl_standard_builder,
 };
 use tor_error::warn_report;
-use tor_hsrproxy::{config::ProxyConfigBuilder, OnionServiceReverseProxy, ProxyConfig};
+use tor_hsrproxy::{OnionServiceReverseProxy, ProxyConfig, config::ProxyConfigBuilder};
 use tor_hsservice::{HsNickname, RunningOnionService};
-use tor_rtcompat::Runtime;
+use tor_rtcompat::{Runtime, SpawnExt};
 use tracing::debug;
 
 /// Configuration for running an onion service from `arti`.
@@ -25,7 +25,8 @@ use tracing::debug;
 /// behavior, consider using
 /// [`TorClient::launch_onion_service`](arti_client::TorClient::launch_onion_service).
 #[derive(Clone, Debug, Eq, PartialEq)]
-pub struct OnionServiceProxyConfig {
+#[cfg_attr(feature = "experimental-api", visibility::make(pub))]
+pub(crate) struct OnionServiceProxyConfig {
     /// Configuration for the onion service itself.
     pub(crate) svc_cfg: OnionServiceConfig,
     /// Configuration for the reverse proxy that handles incoming connections
@@ -40,38 +41,43 @@ pub struct OnionServiceProxyConfig {
 // pieces that we need.
 #[derive(Clone, Debug, serde::Serialize, serde::Deserialize, Default)]
 #[serde(transparent)]
-pub struct OnionServiceProxyConfigBuilder(Flatten<OnionServiceConfigBuilder, ProxyConfigBuilder>);
+#[cfg_attr(feature = "experimental-api", visibility::make(pub))]
+pub(crate) struct OnionServiceProxyConfigBuilder(
+    Flatten<OnionServiceConfigBuilder, ProxyConfigBuilder>,
+);
 
 impl OnionServiceProxyConfigBuilder {
     /// Try to construct an [`OnionServiceProxyConfig`].
     ///
     /// Returns an error if any part of this builder is invalid.
-    pub fn build(&self) -> Result<OnionServiceProxyConfig, ConfigBuildError> {
-        let svc_cfg = self.0 .0.build()?;
-        let proxy_cfg = self.0 .1.build()?;
+    #[cfg_attr(feature = "experimental-api", visibility::make(pub))]
+    pub(crate) fn build(&self) -> Result<OnionServiceProxyConfig, ConfigBuildError> {
+        let svc_cfg = self.0.0.build()?;
+        let proxy_cfg = self.0.1.build()?;
         Ok(OnionServiceProxyConfig { svc_cfg, proxy_cfg })
     }
 
     /// Return a mutable reference to an onion-service configuration sub-builder.
-    pub fn service(&mut self) -> &mut OnionServiceConfigBuilder {
-        &mut self.0 .0
+    #[cfg_attr(feature = "experimental-api", visibility::make(pub))]
+    pub(crate) fn service(&mut self) -> &mut OnionServiceConfigBuilder {
+        &mut self.0.0
     }
 
     /// Return a mutable reference to a proxy configuration sub-builder.
-    pub fn proxy(&mut self) -> &mut ProxyConfigBuilder {
-        &mut self.0 .1
+    #[cfg_attr(feature = "experimental-api", visibility::make(pub))]
+    pub(crate) fn proxy(&mut self) -> &mut ProxyConfigBuilder {
+        &mut self.0.1
     }
 }
 
 impl_standard_builder! { OnionServiceProxyConfig: !Default }
 
-/// Alias for a `BTreeMap` of `OnionServiceProxyConfig`; used to make derive_builder
-/// happy.
+/// Alias for a `BTreeMap` of [`OnionServiceProxyConfig`]; used to make [`derive_builder`] happy.
 #[cfg(feature = "onion-service-service")]
 pub(crate) type OnionServiceProxyConfigMap = BTreeMap<HsNickname, OnionServiceProxyConfig>;
 
-/// The serialized format of an OnionServiceProxyConfigListBuilder:
-/// a map from nickname to `OnionServiceConfigBuilder`
+/// The serialized format of an [`OnionServiceProxyConfigMapBuilder`]:
+/// a map from [`HsNickname`] to [`OnionServiceConfigBuilder`].
 type ProxyBuilderMap = BTreeMap<HsNickname, OnionServiceProxyConfigBuilder>;
 
 // TODO: Someday we might want to have an API for a MapBuilder that is distinct
@@ -79,7 +85,8 @@ type ProxyBuilderMap = BTreeMap<HsNickname, OnionServiceProxyConfigBuilder>;
 // key, and that keys are distinct.
 #[cfg(feature = "onion-service-service")]
 define_list_builder_helper! {
-    pub struct OnionServiceProxyConfigMapBuilder {
+#[cfg_attr(feature = "experimental-api", visibility::make(pub))]
+    pub(crate) struct OnionServiceProxyConfigMapBuilder {
         services: [OnionServiceProxyConfigBuilder],
     }
     built: OnionServiceProxyConfigMap = build_list(services)?;
@@ -87,13 +94,11 @@ define_list_builder_helper! {
     #[serde(try_from="ProxyBuilderMap", into="ProxyBuilderMap")]
 }
 
-/// Construct a OnionServiceProxyConfigList from a vec of OnionServiceProxyConfig;
-/// enforce that nicknames are unique.
+/// Construct a [`OnionServiceProxyConfigMap`] from a `Vec` of [`OnionServiceProxyConfig`];
+/// enforce that [`HsNickname`]s are unique.
 fn build_list(
     services: Vec<OnionServiceProxyConfig>,
 ) -> Result<OnionServiceProxyConfigMap, ConfigBuildError> {
-    // TODO: Add a test for this function.
-    //
     // It *is* reachable from OnionServiceProxyConfigMapBuilder::build(), since
     // that builder's API uses push() to add OnionServiceProxyConfigBuilders to
     // an internal _list_.  Alternatively, we might want to have a distinct
@@ -120,7 +125,7 @@ impl TryFrom<ProxyBuilderMap> for OnionServiceProxyConfigMapBuilder {
     fn try_from(value: ProxyBuilderMap) -> Result<Self, Self::Error> {
         let mut list_builder = OnionServiceProxyConfigMapBuilder::default();
         for (nickname, mut cfg) in value {
-            match cfg.0 .0.peek_nickname() {
+            match cfg.0.0.peek_nickname() {
                 Some(n) if n == &nickname => (),
                 None => (),
                 Some(other) => {
@@ -130,7 +135,7 @@ impl TryFrom<ProxyBuilderMap> for OnionServiceProxyConfigMapBuilder {
                     });
                 }
             }
-            cfg.0 .0.nickname(nickname);
+            cfg.0.0.nickname(nickname);
             list_builder.access().push(cfg);
         }
         Ok(list_builder)
@@ -147,7 +152,7 @@ impl From<OnionServiceProxyConfigMapBuilder> for ProxyBuilderMap {
     fn from(value: OnionServiceProxyConfigMapBuilder) -> Self {
         let mut map = BTreeMap::new();
         for cfg in value.services.into_iter().flatten() {
-            let nickname = cfg.0 .0.peek_nickname().cloned().unwrap_or_else(|| {
+            let nickname = cfg.0.0.peek_nickname().cloned().unwrap_or_else(|| {
                 "Unnamed"
                     .to_string()
                     .try_into()
@@ -178,18 +183,31 @@ struct Proxy {
 impl Proxy {
     /// Create and launch a new onion service proxy, using a given `client`,
     /// to handle connections according to `config`.
+    ///
+    /// Returns `Ok(None)` if the service specified is disabled in the config.
     pub(crate) fn launch_new<R: Runtime>(
         client: &arti_client::TorClient<R>,
         config: OnionServiceProxyConfig,
-    ) -> anyhow::Result<Self> {
+    ) -> anyhow::Result<Option<Self>> {
         let OnionServiceProxyConfig { svc_cfg, proxy_cfg } = config;
         let nickname = svc_cfg.nickname().clone();
-        let (svc, request_stream) = client.launch_onion_service(svc_cfg)?;
+
+        let (svc, request_stream) = match client.launch_onion_service(svc_cfg)? {
+            Some(running_service) => running_service,
+            None => {
+                debug!(
+                    "Onion service {} didn't start (disabled in config)",
+                    nickname
+                );
+                return Ok(None);
+            }
+        };
         let proxy = OnionServiceReverseProxy::new(proxy_cfg);
 
         {
             let proxy = proxy.clone();
             let runtime_clone = client.runtime().clone();
+            let nickname_clone = nickname.clone();
             client.runtime().spawn(async move {
                 match proxy
                     .handle_requests(runtime_clone, nickname.clone(), request_stream)
@@ -203,9 +221,21 @@ impl Proxy {
                     }
                 }
             })?;
+
+            let mut status_stream = svc.status_events();
+            client.runtime().spawn(async move {
+                while let Some(status) = status_stream.next().await {
+                    debug!(
+                        nickname=%nickname_clone,
+                        status=?status.state(),
+                        problem=?status.current_problem(),
+                        "Onion service status change",
+                    );
+                }
+            })?;
         }
 
-        Ok(Proxy { svc, proxy })
+        Ok(Some(Proxy { svc, proxy }))
     }
 
     /// Reconfigure this proxy, using the new configuration `config` and the
@@ -222,7 +252,7 @@ impl Proxy {
         self.reconfigure_inner(config, how)
     }
 
-    /// Helper for reconfigure: Run `reconfigure` on each part of this `Proxy`.
+    /// Helper for `reconfigure`: Run `reconfigure` on each part of this `Proxy`.
     fn reconfigure_inner(
         &mut self,
         config: OnionServiceProxyConfig,
@@ -254,7 +284,14 @@ impl<R: Runtime> ProxySet<R> {
     ) -> anyhow::Result<Self> {
         let proxies: BTreeMap<_, _> = config_list
             .into_iter()
-            .map(|(nickname, cfg)| Ok((nickname, Proxy::launch_new(client, cfg)?)))
+            .filter_map(|(nickname, cfg)| {
+                // Filter out services which are disabled in the config
+                match Proxy::launch_new(client, cfg) {
+                    Ok(Some(running_service)) => Some(Ok((nickname, running_service))),
+                    Err(error) => Some(Err(error)),
+                    Ok(None) => None,
+                }
+            })
             .collect::<anyhow::Result<BTreeMap<_, _>>>()?;
 
         Ok(Self {
@@ -297,9 +334,15 @@ impl<R: Runtime> ProxySet<R> {
                     // We do not have a proxy by this name, so we try to launch
                     // one.
                     match Proxy::launch_new(&self.client, cfg) {
-                        Ok(new_proxy) => {
+                        Ok(Some(new_proxy)) => {
                             ent.insert(new_proxy);
                         }
+                        Ok(None) => {
+                            debug!(
+                                "Onion service {} didn't start (disabled in config)",
+                                ent.key()
+                            );
+                        }
                         Err(err) => {
                             warn_report!(err, "Unable to launch onion service {}", ent.key());
                         }
@@ -320,6 +363,11 @@ impl<R: Runtime> ProxySet<R> {
 
         Ok(())
     }
+
+    /// Whether this `ProxySet` is empty.
+    pub(crate) fn is_empty(&self) -> bool {
+        self.proxies.lock().expect("lock poisoned").is_empty()
+    }
 }
 
 impl<R: Runtime> crate::reload_cfg::ReconfigurableModule for ProxySet<R> {
@@ -328,3 +376,65 @@ impl<R: Runtime> crate::reload_cfg::ReconfigurableModule for ProxySet<R> {
         Ok(())
     }
 }
+
+#[cfg(test)]
+mod tests {
+    // @@ begin test lint list maintained by maint/add_warning @@
+    #![allow(clippy::bool_assert_comparison)]
+    #![allow(clippy::clone_on_copy)]
+    #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
+    #![allow(clippy::print_stderr)]
+    #![allow(clippy::print_stdout)]
+    #![allow(clippy::single_char_pattern)]
+    #![allow(clippy::unwrap_used)]
+    #![allow(clippy::unchecked_time_subtraction)]
+    #![allow(clippy::useless_vec)]
+    #![allow(clippy::needless_pass_by_value)]
+    //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
+    use super::*;
+
+    use tor_config::ConfigBuildError;
+    use tor_hsservice::HsNickname;
+
+    /// Get an [`OnionServiceProxyConfig`] with its `svc_cfg` field having the nickname `nick`.
+    fn get_onion_service_proxy_config(nick: &HsNickname) -> OnionServiceProxyConfig {
+        let mut builder = OnionServiceProxyConfigBuilder::default();
+        builder.service().nickname(nick.clone());
+        builder.build().unwrap()
+    }
+
+    /// Test `super::build_list` with unique and duplicate [`HsNickname`]s.
+    #[test]
+    fn fn_build_list() {
+        let nick_1 = HsNickname::new("nick_1".to_string()).unwrap();
+        let nick_2 = HsNickname::new("nick_2".to_string()).unwrap();
+
+        let proxy_configs: Vec<OnionServiceProxyConfig> = [&nick_1, &nick_2]
+            .into_iter()
+            .map(get_onion_service_proxy_config)
+            .collect();
+        let actual = build_list(proxy_configs.clone()).unwrap();
+
+        let expected =
+            OnionServiceProxyConfigMap::from_iter([nick_1, nick_2].into_iter().zip(proxy_configs));
+
+        assert_eq!(actual, expected);
+
+        let nick = HsNickname::new("nick".to_string()).unwrap();
+        let proxy_configs_dup: Vec<OnionServiceProxyConfig> = [&nick, &nick]
+            .into_iter()
+            .map(get_onion_service_proxy_config)
+            .collect();
+        let actual = build_list(proxy_configs_dup).unwrap_err();
+        let ConfigBuildError::Inconsistent { fields, problem } = actual else {
+            panic!("Unexpected error from `build_list`: {actual:?}");
+        };
+
+        assert_eq!(fields, vec!["nickname".to_string()]);
+        assert_eq!(
+            problem,
+            format!("Multiple onion services with the nickname {nick}")
+        );
+    }
+}
