@@ -4,16 +4,16 @@
 //! See [the spec](spec.torproject.org/http-connect.html)
 //! for more information.
 
-use super::{ListenerIsolation, ProxyContext, copy_interactive};
+use super::{ListenerIsolation, ProxyContext};
 use anyhow::{Context as _, anyhow};
 use arti_client::{StreamPrefs, TorAddr};
-use futures::task::SpawnExt as _;
-use futures::{AsyncRead, AsyncReadExt, AsyncWrite, FutureExt as _, io::BufReader};
+use futures::{AsyncRead, AsyncWrite, io::BufReader};
 use http::{Method, StatusCode, response::Builder as ResponseBuilder};
 use hyper::{Response, server::conn::http1::Builder as ServerBuilder, service::service_fn};
 use safelog::{Sensitive as Sv, sensitive as sv};
 use tor_error::{ErrorKind, ErrorReport as _, HasKind, into_internal, warn_report};
 use tor_rtcompat::Runtime;
+use tor_rtcompat::SpawnExt as _;
 use tracing::{instrument, warn};
 
 use hyper_futures_io::FuturesIoCompat;
@@ -52,9 +52,23 @@ pub(super) struct Isolation {
     tor_isolation: Option<String>,
 }
 
+impl Isolation {
+    /// Return true if no isolation field in this object is set.
+    pub(super) fn is_empty(&self) -> bool {
+        let Isolation {
+            proxy_auth,
+            x_tor_isolation,
+            tor_isolation,
+        } = self;
+        proxy_auth.as_ref().is_none_or(ProxyAuthorization::is_empty)
+            && x_tor_isolation.as_ref().is_none_or(String::is_empty)
+            && tor_isolation.as_ref().is_none_or(String::is_empty)
+    }
+}
+
 /// Constants and code for the HTTP headers we use.
 mod hdr {
-    pub(super) use http::header::{CONTENT_TYPE, PROXY_AUTHORIZATION, SERVER, VIA};
+    pub(super) use http::header::{CONTENT_TYPE, HOST, PROXY_AUTHORIZATION, SERVER, VIA};
 
     /// Client-to-proxy: Which IP family should we use?
     pub(super) const TOR_FAMILY_PREFERENCE: &str = "Tor-Family-Preference";
@@ -75,6 +89,19 @@ mod hdr {
     /// Proxy-to-client: A machine-readable list of failure reasons.
     pub(super) const TOR_REQUEST_FAILED: &str = "Tor-Request-Failed";
 
+    /// A list of all the headers that we support from client-to-proxy.
+    ///
+    /// Does not include headers that we check for HTTP conformance,
+    /// but not for any other purpose.
+    pub(super) const ALL_REQUEST_HEADERS: &[&str] = &[
+        TOR_FAMILY_PREFERENCE,
+        TOR_RPC_TARGET,
+        X_TOR_STREAM_ISOLATION,
+        TOR_STREAM_ISOLATION,
+        // Can't use 'PROXY_AUTHORIZATION', since it isn't a str, and its as_str() isn't const.
+        "Proxy-Authorization",
+    ];
+
     /// Return the unique string-valued value of the header `name`;
     /// or None if the header doesn't exist,
     /// or an error if the header is duplicated or not UTF-8.
@@ -144,8 +171,27 @@ where
     R: Runtime,
     S: AsyncRead + AsyncWrite + Send + Sync + Unpin + 'static,
 {
+    // Avoid cross-site attacks based on DNS forgery by validating that the Host
+    // header is in fact localhost.  In these cases, we don't want to reply at all,
+    // _even with an error message_, since our headers could be used to tell a hostile
+    // webpage information about the local arti process.
+    //
+    // We don't do this for CONNECT requests, since those are forbidden by
+    // XHR and JS fetch(), and since Host _will_ be non-localhost for those.
+    if request.method() != Method::CONNECT {
+        match hdr::uniq_utf8(request.headers(), hdr::HOST) {
+            Err(e) => return Err(e).context("Host header invalid. Rejecting request."),
+            Ok(Some(host)) if !host_is_localhost(host) => {
+                return Err(anyhow!(
+                    "Host header {host:?} was not localhost. Rejecting request."
+                ));
+            }
+            Ok(_) => {}
+        }
+    }
+
     match *request.method() {
-        Method::OPTIONS => handle_options_request(request).await,
+        Method::OPTIONS => handle_options_request(&request),
         Method::CONNECT => {
             handle_connect_request::<R, S>(request, context, listener_isolation).await
         }
@@ -159,7 +205,7 @@ where
 }
 
 /// Return an appropriate reply to the given OPTIONS request.
-async fn handle_options_request(request: Request) -> Result<Response<Body>, anyhow::Error> {
+fn handle_options_request(request: &Request) -> Result<Response<Body>, anyhow::Error> {
     use hyper::body::Body as _;
 
     let target = request.uri().to_string();
@@ -246,12 +292,11 @@ where
     // We have connected.  We need to launch a separate task to actually be the proxy, though,
     // since IIUC hyper::upgrade::on won't return an answer
     // until after the response is given to the client.
-    let runtime = context.tor_client.runtime().clone();
     context
         .tor_client
         .runtime()
         .spawn(async move {
-            match transfer::<R, S>(runtime, request, tor_stream).await {
+            match transfer::<S>(request, tor_stream).await {
                 Ok(()) => {}
                 Err(e) => {
                     warn_report!(e, "Error while launching transfer");
@@ -361,6 +406,14 @@ impl ProxyAuthorization {
             None
         }
     }
+
+    /// Return true if this ProxyAuthorization has no authorization information.
+    fn is_empty(&self) -> bool {
+        match self {
+            ProxyAuthorization::Legacy(s) => s.is_empty(),
+            ProxyAuthorization::Modern(v) => v.is_empty(),
+        }
+    }
 }
 
 /// Look up the connection target given the value of an Tor-RPC-Target header.
@@ -432,9 +485,21 @@ impl RespBldExt for ResponseBuilder {
     }
 }
 
+/// Return a string representing our capabilities.
+fn capabilities() -> &'static str {
+    use std::sync::LazyLock;
+    static CAPS: LazyLock<String> = LazyLock::new(|| {
+        let mut caps = hdr::ALL_REQUEST_HEADERS.to_vec();
+        caps.sort();
+        caps.join(" ")
+    });
+
+    CAPS.as_str()
+}
+
 /// Add all common headers to the builder `bld`, and return a new builder.
 fn add_common_headers(mut bld: ResponseBuilder, method: &Method) -> ResponseBuilder {
-    bld = bld.header(hdr::TOR_CAPABILITIES, "");
+    bld = bld.header(hdr::TOR_CAPABILITIES, capabilities());
     if let (Some(software), Some(version)) = (
         option_env!("CARGO_PKG_NAME"),
         option_env!("CARGO_PKG_VERSION"),
@@ -517,6 +582,9 @@ impl HttpConnectError {
     fn status_code(&self) -> StatusCode {
         use HttpConnectError as HCE; // Not a Joyce reference
         use StatusCode as SC;
+        if let Some(end_reason) = self.remote_end_reason() {
+            return end_reason_to_http_status(end_reason);
+        }
         match self {
             HCE::InvalidStreamTarget(_, _)
             | HCE::DuplicateHeader
@@ -524,105 +592,78 @@ impl HttpConnectError {
             | HCE::InvalidFamilyPreference
             | HCE::RpcObjectNotFound
             | HCE::NoRpcSupport => SC::BAD_REQUEST,
-            HCE::ConnectFailed(_, e) => kind_to_status(e.kind()),
-            HCE::Internal(e) => kind_to_status(e.kind()),
+            HCE::ConnectFailed(_, e) => e.kind().http_status_code(),
+            HCE::Internal(e) => e.kind().http_status_code(),
         }
     }
 
     /// If possible, return a response that we should give to this error.
     fn try_into_response(self) -> Result<Response<Body>, HttpConnectError> {
         let error_kind = self.kind();
+        let end_reason = self.remote_end_reason();
         let status_code = self.status_code();
-        // TODO: It would be neat to also include specific END reasons here.  But to get them we'd
-        // need to depend on the "details" feature of arti-client, which I'm not sure we want to do.
-        //
-        // If we _do_ get a way to extract END reasons, we can also use them manually alongside
-        // kind_to_status.
+        let mut request_failed = format!("arti/{error_kind:?}");
+        if let Some(end_reason) = end_reason {
+            request_failed.push_str(&format!(" end/{end_reason}"));
+        }
+
         ResponseBuilder::new()
             .status(status_code)
-            .header(hdr::TOR_REQUEST_FAILED, format!("arti/{error_kind:?}"))
+            .header(hdr::TOR_REQUEST_FAILED, request_failed)
             .err(&Method::CONNECT, self.report().to_string())
     }
+
+    /// Return the end reason for this error, if this error does in fact represent an END message
+    /// from the remote side of a stream.
+    //
+    // TODO: This function is a bit fragile; it forces us to use APIs from tor-proto and
+    // tor-cell that are not re-exported from arti-client.  It also relies on the fact that
+    // there is a single error type way down in `tor-proto` representing a received END message.
+    fn remote_end_reason(&self) -> Option<tor_cell::relaycell::msg::EndReason> {
+        use tor_proto::Error::EndReceived;
+        if let Some(EndReceived(reason)) = super::extract_proto_err(self) {
+            Some(*reason)
+        } else {
+            None
+        }
+    }
 }
 
-/// Convert an ErrorKind into a StatusCode.
-//
-// TODO: Perhaps move this to tor-error, so it can be an exhaustive match.
-fn kind_to_status(kind: ErrorKind) -> StatusCode {
-    use http::StatusCode as SC;
-    use tor_error::ErrorKind as EK;
-    match kind {
-        EK::ArtiShuttingDown
-        | EK::BadApiUsage
-        | EK::BootstrapRequired
-        | EK::CacheAccessFailed
-        | EK::CacheCorrupted
-        | EK::ClockSkew
-        | EK::DirectoryExpired
-        | EK::ExternalToolFailed
-        | EK::FsPermissions
-        | EK::Internal
-        | EK::InvalidConfig
-        | EK::InvalidConfigTransition
-        | EK::KeystoreAccessFailed
-        | EK::KeystoreCorrupted
-        | EK::NoHomeDirectory
-        | EK::Other
-        | EK::PersistentStateAccessFailed
-        | EK::PersistentStateCorrupted
-        | EK::SoftwareDeprecated
-        | EK::TorDirectoryUnusable
-        | EK::TransientFailure
-        | EK::ReactorShuttingDown
-        | EK::RelayIdMismatch
-        | EK::RelayTooBusy
-        | EK::TorAccessFailed
-        | EK::TorDirectoryError => SC::INTERNAL_SERVER_ERROR,
+/// Return the appropriate HTTP status code for a remote END reason.
+///
+/// Return `None` if the END reason is unrecognized and we should use the `ErrorKind`
+///
+/// (We  _could_ use the ErrorKind unconditionally,
+/// but the mapping from END reason to ErrorKind is [given in the spec][spec],
+/// so we try to obey it.)
+///
+/// [spec]: https://spec.torproject.org/http-connect.html#error-codes
+fn end_reason_to_http_status(end_reason: tor_cell::relaycell::msg::EndReason) -> StatusCode {
+    use StatusCode as S;
+    use tor_cell::relaycell::msg::EndReason as R;
+    match end_reason {
+        //
+        R::CONNECTREFUSED => S::FORBIDDEN, // 403
+        // 500: Internal server error.
+        R::MISC | R::NOTDIRECTORY => S::INTERNAL_SERVER_ERROR,
 
-        EK::FeatureDisabled | EK::NotImplemented => SC::NOT_IMPLEMENTED,
-
-        EK::CircuitCollapse
-        | EK::CircuitRefused
-        | EK::ExitPolicyRejected
-        | EK::LocalNetworkError
-        | EK::LocalProtocolViolation
-        | EK::LocalResourceAlreadyInUse
-        | EK::LocalResourceExhausted
-        | EK::NoExit
-        | EK::NoPath => SC::SERVICE_UNAVAILABLE,
-
-        EK::TorProtocolViolation | EK::RemoteProtocolViolation | EK::RemoteNetworkFailed => {
-            SC::BAD_GATEWAY
+        // 502: Bad Gateway.
+        R::DESTROY | R::DONE | R::HIBERNATING | R::INTERNAL | R::RESOURCELIMIT | R::TORPROTOCOL => {
+            S::BAD_GATEWAY
         }
+        // 503: Service unavailable
+        R::CONNRESET | R::EXITPOLICY | R::NOROUTE | R::RESOLVEFAILED => S::SERVICE_UNAVAILABLE,
 
-        EK::ExitTimeout | EK::TorNetworkTimeout | EK::RemoteNetworkTimeout => SC::GATEWAY_TIMEOUT,
+        // 504: Gateway timeout.
+        R::TIMEOUT => S::GATEWAY_TIMEOUT,
 
-        EK::ForbiddenStreamTarget => SC::FORBIDDEN,
-
-        #[cfg(feature = "onion-service-client")]
-        EK::OnionServiceAddressInvalid | EK::InvalidStreamTarget => SC::BAD_REQUEST,
-        #[cfg(feature = "onion-service-client")]
-        EK::OnionServiceWrongClientAuth => SC::FORBIDDEN,
-        #[cfg(feature = "onion-service-client")]
-        EK::OnionServiceConnectionFailed
-        | EK::OnionServiceMissingClientAuth
-        | EK::OnionServiceNotFound
-        | EK::OnionServiceNotRunning
-        | EK::OnionServiceProtocolViolation => SC::SERVICE_UNAVAILABLE,
-
-        EK::RemoteConnectionRefused
-        | EK::RemoteHostNotFound
-        | EK::RemoteHostResolutionFailed
-        | EK::RemoteStreamClosed
-        | EK::RemoteStreamError
-        | EK::RemoteStreamReset => SC::SERVICE_UNAVAILABLE,
-
-        _ => SC::INTERNAL_SERVER_ERROR,
+        // This is possible if the other side sent an unrecognized error code.
+        _ => S::INTERNAL_SERVER_ERROR, // 500
     }
 }
 
 /// Recover the original stream from a [`hyper::upgrade::Upgraded`].
-fn deconstruct_upgrade<S>(upgraded: hyper::upgrade::Upgraded) -> Result<S, anyhow::Error>
+fn deconstruct_upgrade<S>(upgraded: hyper::upgrade::Upgraded) -> Result<BufReader<S>, anyhow::Error>
 where
     S: AsyncRead + AsyncWrite + Unpin + 'static,
 {
@@ -638,47 +679,47 @@ where
         ));
     }
     let io: BufReader<S> = io.0;
-    if !io.buffer().is_empty() {
-        // TODO Figure out whether this can happen due to possible race conditions if the client gets the OK
-        // before we check this?
-        return Err(anyhow!(
-            "Extraneous data on BufReader after upgrade to proxy mode"
-        ));
-    }
-    Ok(io.into_inner())
+    Ok(io)
 }
 
 /// Recover the application stream from `request`, and launch tasks to transfer data between the application and
 /// the `tor_stream`.
-async fn transfer<R, S>(
-    runtime: R,
-    request: Request,
-    tor_stream: arti_client::DataStream,
-) -> anyhow::Result<()>
+async fn transfer<S>(request: Request, tor_stream: arti_client::DataStream) -> anyhow::Result<()>
 where
-    R: Runtime,
     S: AsyncRead + AsyncWrite + Send + Sync + Unpin + 'static,
 {
     let upgraded = hyper::upgrade::on(request)
         .await
         .context("Unable to upgrade connection")?;
-    let app_stream: S = deconstruct_upgrade(upgraded)?;
+    let app_stream: BufReader<S> = deconstruct_upgrade(upgraded)?;
+    let tor_stream = BufReader::with_capacity(super::APP_STREAM_BUF_LEN, tor_stream);
 
-    let (app_r, app_w) = app_stream.split();
-    let (tor_r, tor_w) = tor_stream.split();
-
-    // Finally, spawn two background tasks to relay traffic between
-    // the application stream and the tor stream.
-    runtime
-        .spawn(copy_interactive(app_r, tor_w).map(|_| ()))
-        .context("Spawning task")?;
-    runtime
-        .spawn(copy_interactive(tor_r, app_w).map(|_| ()))
-        .context("Spawning task")?;
+    // Finally. relay traffic between
+    // the application stream and the tor stream, forever.
+    let _ = futures_copy::copy_buf_bidirectional(
+        app_stream,
+        tor_stream,
+        futures_copy::eof::Close,
+        futures_copy::eof::Close,
+    )
+    .await?;
 
     Ok(())
 }
 
+/// Return true if `host` is a possible value for a Host header addressing localhost.
+fn host_is_localhost(host: &str) -> bool {
+    if let Ok(addr) = host.parse::<std::net::SocketAddr>() {
+        addr.ip().is_loopback()
+    } else if let Ok(ip) = host.parse::<std::net::IpAddr>() {
+        ip.is_loopback()
+    } else if let Some((addr, port)) = host.split_once(':') {
+        port.parse::<std::num::NonZeroU16>().is_ok() && addr.eq_ignore_ascii_case("localhost")
+    } else {
+        host.eq_ignore_ascii_case("localhost")
+    }
+}
+
 /// Helper module: Make `futures` types usable by `hyper`.
 //
 // TODO: We may want to expose this as a separate crate, or move it into tor-async-utils,
@@ -762,11 +803,17 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
+    use arti_client::{BootstrapBehavior, TorClient, config::TorClientConfigBuilder};
+    use futures::{AsyncReadExt as _, AsyncWriteExt as _};
+    use tor_rtmock::{MockRuntime, io::stream_pair};
+
+    use super::*;
+
     // Make sure that HeaderMap is case-insensitive as the documentation implies.
     #[test]
     fn headermap_casei() {
@@ -785,4 +832,107 @@ mod test {
             b"a-secret"
         );
     }
+
+    #[test]
+    fn host_header_localhost() {
+        assert_eq!(host_is_localhost("localhost"), true);
+        assert_eq!(host_is_localhost("localhost:9999"), true);
+        assert_eq!(host_is_localhost("localHOSt:9999"), true);
+        assert_eq!(host_is_localhost("127.0.0.1:9999"), true);
+        assert_eq!(host_is_localhost("[::1]:9999"), true);
+        assert_eq!(host_is_localhost("127.1.2.3:1234"), true);
+        assert_eq!(host_is_localhost("127.0.0.1"), true);
+        assert_eq!(host_is_localhost("::1"), true);
+
+        assert_eq!(host_is_localhost("[::1]"), false); // not in the right format!
+        assert_eq!(host_is_localhost("www.torproject.org"), false);
+        assert_eq!(host_is_localhost("www.torproject.org:1234"), false);
+        assert_eq!(host_is_localhost("localhost:0"), false);
+        assert_eq!(host_is_localhost("localhost:999999"), false);
+        assert_eq!(host_is_localhost("plocalhost:1234"), false);
+        assert_eq!(host_is_localhost("[::0]:1234"), false);
+        assert_eq!(host_is_localhost("192.0.2.55:1234"), false);
+        assert_eq!(host_is_localhost("3fff::1"), false);
+        assert_eq!(host_is_localhost("[3fff::1]:1234"), false);
+    }
+
+    fn interactive_test_setup(
+        rt: &MockRuntime,
+    ) -> anyhow::Result<(
+        tor_rtmock::io::LocalStream,
+        impl Future<Output = anyhow::Result<()>>,
+        tempfile::TempDir,
+    )> {
+        let (s1, s2) = stream_pair();
+        let s1: BufReader<_> = BufReader::new(s1);
+
+        let iso: ListenerIsolation = (7, "127.0.0.1".parse().unwrap());
+        let dir = tempfile::TempDir::new().unwrap();
+        let cfg = TorClientConfigBuilder::from_directories(
+            dir.as_ref().join("state"),
+            dir.as_ref().join("cache"),
+        )
+        .build()
+        .unwrap();
+        let tor_client = TorClient::with_runtime(rt.clone())
+            .config(cfg)
+            .bootstrap_behavior(BootstrapBehavior::Manual)
+            .create_unbootstrapped()?;
+        let context: ProxyContext<_> = ProxyContext {
+            tor_client,
+            #[cfg(feature = "rpc")]
+            rpc_mgr: None,
+        };
+        let handle = rt.spawn_join("HTTP Handler", handle_http_conn(context, s1, iso));
+        Ok((s2, handle, dir))
+    }
+
+    #[test]
+    fn successful_options_test() -> anyhow::Result<()> {
+        // Try an OPTIONS request and make sure we get a plausible-looking answer.
+        //
+        // (This test is mostly here to make sure that invalid_host_test() isn't failing because
+        // of anything besides the Host header.)
+        MockRuntime::try_test_with_various(async |rt| -> anyhow::Result<()> {
+            let (mut s, join, _dir) = interactive_test_setup(&rt)?;
+
+            s.write_all(b"OPTIONS * HTTP/1.0\r\nHost: localhost\r\n\r\n")
+                .await?;
+            let mut buf = Vec::new();
+            let _n_read = s.read_to_end(&mut buf).await?;
+            let () = join.await?;
+
+            let reply = std::str::from_utf8(&buf)?;
+            assert!(dbg!(reply).starts_with("HTTP/1.0 200 OK\r\n"));
+
+            Ok(())
+        })
+    }
+
+    #[test]
+    fn invalid_host_test() -> anyhow::Result<()> {
+        // Try a hostname that looks like a CSRF attempt and make sure that we discard it without
+        // any reply.
+        MockRuntime::try_test_with_various(async |rt| -> anyhow::Result<()> {
+            let (mut s, join, _dir) = interactive_test_setup(&rt)?;
+
+            s.write_all(b"OPTIONS * HTTP/1.0\r\nHost: csrf.example.com\r\n\r\n")
+                .await?;
+            let mut buf = Vec::new();
+            let n_read = s.read_to_end(&mut buf).await?;
+            let http_outcome = join.await;
+
+            assert_eq!(n_read, 0);
+            assert!(buf.is_empty());
+            assert!(http_outcome.is_err());
+
+            let error_msg = http_outcome.unwrap_err().source().unwrap().to_string();
+            assert_eq!(
+                error_msg,
+                r#"Host header "csrf.example.com" was not localhost. Rejecting request."#
+            );
+
+            Ok(())
+        })
+    }
 }
