@@ -7,7 +7,7 @@ use cfg_if::cfg_if;
 use clap::ArgMatches;
 #[allow(unused)]
 use tor_config_path::CfgPathResolver;
-use tracing::{info, warn};
+use tracing::{info, instrument, warn};
 
 use arti_client::TorClientConfig;
 use tor_config::{ConfigurationSources, Listen};
@@ -15,7 +15,11 @@ use tor_rtcompat::ToplevelRuntime;
 
 #[cfg(feature = "dns-proxy")]
 use crate::dns;
-use crate::{exit, process, reload_cfg, socks, ArtiConfig, TorClient};
+use crate::{
+    ArtiConfig, TorClient, exit, process,
+    proxy::{self, port_info},
+    reload_cfg,
+};
 
 #[cfg(feature = "rpc")]
 use crate::rpc;
@@ -27,6 +31,8 @@ use crate::onion_proxy;
 type PinnedFuture<T> = std::pin::Pin<Box<dyn futures::Future<Output = T>>>;
 
 /// Run the `proxy` subcommand.
+#[instrument(skip_all, level = "trace")]
+#[allow(clippy::cognitive_complexity)]
 pub(crate) fn run<R: ToplevelRuntime>(
     runtime: R,
     proxy_matches: &ArgMatches,
@@ -34,21 +40,30 @@ pub(crate) fn run<R: ToplevelRuntime>(
     config: ArtiConfig,
     client_config: TorClientConfig,
 ) -> Result<()> {
-    // Override configured SOCKS and DNS listen addresses from the command line.
+    // Override configured listen addresses from the command line.
     // This implies listening on localhost ports.
+
+    // TODO: Parse a string rather than calling new_localhost.
     let socks_listen = match proxy_matches.get_one::<String>("socks-port") {
-        Some(p) => Listen::new_localhost(p.parse().expect("Invalid port specified")),
+        Some(p) => Listen::new_localhost(
+            p.parse()
+                .with_context(|| format!("Invalid -p argument (SOCKS port): \"{p}\""))?,
+        ),
         None => config.proxy().socks_listen.clone(),
     };
 
+    // TODO: Parse a string rather than calling new_localhost.
     let dns_listen = match proxy_matches.get_one::<String>("dns-port") {
-        Some(p) => Listen::new_localhost(p.parse().expect("Invalid port specified")),
+        Some(p) => Listen::new_localhost(
+            p.parse()
+                .with_context(|| format!("Invalid -d argument (DNS port): \"{p}\""))?,
+        ),
         None => config.proxy().dns_listen.clone(),
     };
 
     if !socks_listen.is_empty() {
         info!(
-            "Starting Arti {} in SOCKS proxy mode on {} ...",
+            "Starting Arti {} in proxy mode on {} ...",
             env!("CARGO_PKG_VERSION"),
             socks_listen
         );
@@ -102,6 +117,7 @@ pub(crate) fn run<R: ToplevelRuntime>(
 /// Currently, might panic if things go badly enough wrong
 #[cfg_attr(feature = "experimental-api", visibility::make(pub))]
 #[cfg_attr(docsrs, doc(cfg(feature = "experimental-api")))]
+#[instrument(skip_all, level = "trace")]
 async fn run_proxy<R: ToplevelRuntime>(
     runtime: R,
     socks_listen: Listen,
@@ -115,10 +131,8 @@ async fn run_proxy<R: ToplevelRuntime>(
     use arti_client::BootstrapBehavior::OnDemand;
     use futures::FutureExt;
 
-    // TODO RPC: We may instead want to provide a way to get these items out of TorClient.
-    #[allow(unused)]
+    // TODO: We may instead want to provide a way to get these items out of TorClient.
     let fs_mistrust = client_config.fs_mistrust().clone();
-    #[allow(unused)]
     let path_resolver: CfgPathResolver = AsRef::<CfgPathResolver>::as_ref(&client_config).clone();
 
     let client_builder = TorClient::with_runtime(runtime.clone())
@@ -171,25 +185,38 @@ async fn run_proxy<R: ToplevelRuntime>(
         }
     }
 
-    let mut proxy: Vec<PinnedFuture<(Result<()>, &str)>> = Vec::new();
+    let mut proxy: Vec<PinnedFuture<Result<()>>> = Vec::new();
+    let mut ports = Vec::new();
     if !socks_listen.is_empty() {
         let runtime = runtime.clone();
         let client = client.isolated_client();
         let socks_listen = socks_listen.clone();
-        proxy.push(Box::pin(async move {
-            let res = socks::run_socks_proxy(runtime, client, socks_listen, rpc_data).await;
-            (res, "SOCKS")
-        }));
+        #[cfg(feature = "http-connect")]
+        let listener_type = "SOCKS+HTTP";
+        #[cfg(not(feature = "http-connect"))]
+        let listener_type = "SOCKS";
+
+        let (proxy_future, socks_ports) =
+            proxy::launch_proxy(runtime, client, socks_listen, rpc_data)
+                .await
+                .with_context(|| format!("Unable to launch {listener_type} proxy"))?;
+        let failure_message = format!("{listener_type} proxy died unexpectedly");
+        let proxy_future = proxy_future.map(|future_result| future_result.context(failure_message));
+        proxy.push(Box::pin(proxy_future));
+        ports.extend(socks_ports);
     }
 
     #[cfg(feature = "dns-proxy")]
     if !dns_listen.is_empty() {
         let runtime = runtime.clone();
         let client = client.isolated_client();
-        proxy.push(Box::pin(async move {
-            let res = dns::run_dns_resolver(runtime, client, dns_listen).await;
-            (res, "DNS")
-        }));
+        let (proxy_future, dns_ports) = dns::launch_dns_resolver(runtime, client, dns_listen)
+            .await
+            .context("Unable to launch DNS proxy")?;
+        let proxy_future =
+            proxy_future.map(|future_result| future_result.context("DNS proxy died unexpectedly"));
+        proxy.push(Box::pin(proxy_future));
+        ports.extend(dns_ports);
     }
 
     #[cfg(not(feature = "dns-proxy"))]
@@ -202,7 +229,13 @@ async fn run_proxy<R: ToplevelRuntime>(
 
     if proxy.is_empty() {
         if !launched_onion_svc {
-            warn!("No proxy port set; specify -p PORT (for `socks_port`) or -d PORT (for `dns_port`). Alternatively, use the `socks_port` or `dns_port` configuration option.");
+            // TODO: rename "socks_listen" to "proxy_listen", preserving compat, once http-connect is stable.
+            warn!(
+                "No proxy address set; \
+                specify -p PORT (to override `socks_listen`) \
+                or -d PORT (to override `dns_listen`). \
+                Alternatively, use the `socks_listen` or `dns_listen` configuration options."
+            );
             return Ok(());
         } else {
             // Push a dummy future to appease future::select_all,
@@ -211,16 +244,28 @@ async fn run_proxy<R: ToplevelRuntime>(
         }
     }
 
+    {
+        let port_info = port_info::PortInfo { ports };
+        let port_info_file = arti_config
+            .storage()
+            .port_info_file
+            .path(&path_resolver)
+            .context("Can't find path for port_info_file")?;
+        if port_info_file.to_str() != Some("") {
+            port_info.write_to_file(&fs_mistrust, &port_info_file)?;
+        }
+    }
+
     let proxy = futures::future::select_all(proxy).map(|(finished, _index, _others)| finished);
     futures::select!(
         r = exit::wait_for_ctrl_c().fuse()
             => r.context("waiting for termination signal"),
         r = proxy.fuse()
-            => r.0.context(format!("{} proxy failure", r.1)),
+            => r,
         r = async {
             client.bootstrap().await?;
             if !socks_listen.is_empty() {
-                info!("Sufficiently bootstrapped; system SOCKS now functional.");
+                info!("Sufficiently bootstrapped; proxy now functional.");
             } else {
                 info!("Sufficiently bootstrapped.");
             }
