@@ -55,6 +55,11 @@ pub(crate) enum Disable {
 /// None.
 #[allow(clippy::match_like_matches_macro)]
 fn from_env_var_value(input: std::result::Result<String, VarError>) -> Option<Status> {
+    // WARNING: This behaviour of the environment variable parsing/evaluation is considered
+    // stable and should not be modified unless necessary.
+    // This behaviour is part of the public interface of applications which use fs-mistrust,
+    // so changing the behaviour of this function may result in a breaking change for applications.
+
     let mut s = match input {
         Ok(s) => s,
         Err(VarError::NotPresent) => return None,
@@ -62,7 +67,9 @@ fn from_env_var_value(input: std::result::Result<String, VarError>) -> Option<St
     };
 
     s.make_ascii_lowercase();
-    match s.as_ref() {
+    let s = s.trim();
+
+    match s {
         "" => None,
         "0" | "no" | "never" | "false" | "n" => Some(Status::CheckPermissions),
         _ => Some(Status::DisableChecks),
@@ -101,7 +108,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -123,11 +130,18 @@ mod test {
         }
 
         assert_eq!(from_env_var_value(Ok("".into())), None);
+        assert_eq!(from_env_var_value(Ok(" ".into())), None);
 
         assert_eq!(from_env_var_value(Err(VarError::NotPresent)), None);
         assert_eq!(
             from_env_var_value(Err(VarError::NotUnicode("".into()))),
             Some(Status::DisableChecks)
         );
+
+        // see https://gitlab.torproject.org/tpo/core/arti/-/issues/1782
+        assert_eq!(
+            from_env_var_value(Ok(" false ".to_string())),
+            Some(Status::CheckPermissions),
+        );
     }
 }
