@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,21 +30,25 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
+#![deny(clippy::mod_module_files)]
 #![allow(clippy::let_unit_value)] // This can reasonably be done for explicitness
 #![allow(clippy::uninlined_format_args)]
 #![allow(clippy::significant_drop_in_scrutinee)] // arti/-/merge_requests/588/#note_2812945
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
+#![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 mod key_data;
 
 pub use slotmap::{
-    new_key_type, secondary, DefaultKey, Key, KeyData, SecondaryMap, SparseSecondaryMap,
+    DefaultKey, Key, KeyData, SecondaryMap, SparseSecondaryMap, new_key_type, secondary,
 };
 
 use key_data::key_version_serde as key_version;
@@ -104,7 +108,7 @@ impl<V> Entry<V> {
 
 /// Helper: Define a wrapper for a single SlotMap type.
 ///
-/// This works for SlotMap, DenseSlotMap, and HopSlotMap.
+/// This works for SlotMap, and DenseSlotMap.
 ///
 /// (The alternative to using a macro here would be to define a new trait
 /// implemented by all of the SlotMaps, and then to define our own SlotMap as a wrapper around an
@@ -606,8 +610,6 @@ define_implementation! { SlotMap }
 
 define_implementation! { DenseSlotMap }
 
-define_implementation! { HopSlotMap }
-
 /// Return true if this key is apparently valid.
 ///
 /// We should use debug_assert! to test this on every new key, every time an entry is inserted.
@@ -654,7 +656,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -983,5 +985,4 @@ mod test {
 
     tests_for! {SlotMap}
     tests_for! {DenseSlotMap}
-    tests_for! {HopSlotMap}
 }
