@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -42,6 +42,7 @@
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
 #![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 use std::collections::BinaryHeap;
@@ -51,9 +52,11 @@ use std::ops::{RangeInclusive, RangeToInclusive};
 use std::path::Path;
 use std::time::Duration;
 
+pub mod intern;
 pub mod iter;
 pub mod n_key_list;
 pub mod n_key_set;
+pub mod rand_hostname;
 pub mod rangebounds;
 pub mod retry;
 pub mod test_rng;
@@ -594,8 +597,9 @@ macro_rules! derive_serde_raw { {
 
 /// Flatten a `Result<Result<T, E>, E>` into a `Result<T, E>`.
 ///
-/// See the nightly [`Result::flatten`].
-// TODO MSRV TBD: When `Result::flatten` is stable and our MSRV allows,
+/// See [`Result::flatten`], which is not available
+/// at our current MSRV.
+// TODO MSRV 1.89: When our MSRV is at least 1.89,
 // remove this function and replace uses with `Result::flatten`.
 pub fn flatten<T, E>(x: Result<Result<T, E>, E>) -> Result<T, E> {
     match x {
@@ -606,6 +610,25 @@ pub fn flatten<T, E>(x: Result<Result<T, E>, E>) -> Result<T, E> {
 
 // ----------------------------------------------------------------------
 
+/// Asserts that the type of the expression implements the given trait.
+///
+/// Example:
+///
+/// ```
+/// # use tor_basic_utils::assert_val_impl_trait;
+/// let x: u32 = 0;
+/// assert_val_impl_trait!(x, Clone);
+/// ```
+#[macro_export]
+macro_rules! assert_val_impl_trait {
+    ($check:expr, $trait:path $(,)?) => {{
+        fn ensure_trait<T: $trait>(_s: &T) {}
+        ensure_trait(&$check);
+    }};
+}
+
+// ----------------------------------------------------------------------
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
@@ -617,7 +640,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
