@@ -24,17 +24,14 @@ use tor_memquota::{derive_deftly_template_HasMemoryCost, memory_cost_structural_
 use bitflags::bitflags;
 
 #[cfg(feature = "conflux")]
-#[cfg_attr(docsrs, doc(cfg(feature = "conflux")))]
 pub use super::conflux::{ConfluxLink, ConfluxLinked, ConfluxLinkedAck, ConfluxSwitch};
 
 #[cfg(feature = "hs")]
-#[cfg_attr(docsrs, doc(cfg(feature = "hs")))]
 pub use super::hs::{
     EstablishRendezvous, IntroEstablished, Introduce1, Introduce2, IntroduceAck, Rendezvous1,
     Rendezvous2, RendezvousEstablished, est_intro::EstablishIntro,
 };
 #[cfg(feature = "experimental-udp")]
-#[cfg_attr(docsrs, doc(cfg(feature = "experimental-udp")))]
 pub use super::udp::{ConnectUdp, ConnectedUdp, Datagram};
 
 crate::restrict::restricted_msg! {
@@ -1347,6 +1344,7 @@ macro_rules! empty_body {
        }
    }
 }
+#[allow(unused_imports)] // TODO #1060
 pub(crate) use empty_body;
 
 empty_body! {
@@ -1442,7 +1440,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
