@@ -9,6 +9,8 @@ use tor_llcrypto as ll;
 
 use digest::Digest;
 
+use crate::CertType;
+
 /// A RSA->Ed25519 cross-certificate
 ///
 /// This kind of certificate is used in the channel handshake to prove
@@ -46,6 +48,11 @@ impl RsaCrosscert {
         other == &self.subject_key
     }
 
+    /// Return this certificate cert type.
+    pub fn cert_type(&self) -> CertType {
+        CertType::RSA_ID_V_IDENTITY
+    }
+
     /// Decode a slice of bytes into an RSA crosscert.
     pub fn decode(bytes: &[u8]) -> tor_bytes::Result<UncheckedRsaCrosscert> {
         let mut r = Reader::from_slice(bytes);
