@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -42,6 +42,7 @@
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
 #![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 pub mod builder;
@@ -57,25 +58,32 @@ pub(crate) mod util;
 
 use futures::StreamExt;
 use futures::select_biased;
-use futures::task::SpawnExt;
 use std::result::Result as StdResult;
 use std::sync::{Arc, Weak};
 use std::time::Duration;
 use tor_config::ReconfigureError;
 use tor_error::error_report;
-use tor_keymgr::KeyMgr;
 use tor_linkspec::{ChanTarget, OwnedChanTarget};
 use tor_netdir::{NetDirProvider, params::NetParameters};
 use tor_proto::channel::Channel;
 #[cfg(feature = "experimental-api")]
 use tor_proto::memquota::ChannelAccount;
 use tor_proto::memquota::ToplevelAccount;
+use tor_rtcompat::SpawnExt;
 use tracing::debug;
+use tracing::instrument;
 use void::{ResultVoidErrExt, Void};
 
+#[cfg(feature = "relay")]
+use {
+    async_trait::async_trait, safelog::Sensitive, std::net::IpAddr,
+    tor_proto::relay::channel_provider::ChannelProvider,
+};
+
 pub use err::Error;
 
 pub use config::{ChannelConfig, ChannelConfigBuilder};
+pub use mgr::ChanMgrConfig;
 
 use tor_rtcompat::Runtime;
 
@@ -130,9 +138,8 @@ pub struct ChanMgr<R: Runtime> {
     /// Stream of [`ConnStatus`] events.
     bootstrap_status: event::ConnStatusEvents,
 
-    /// This currently isn't actually used, but we're keeping a PhantomData here
-    /// since probably we'll want it again, sooner or later.
-    runtime: std::marker::PhantomData<fn(R) -> R>,
+    /// The runtime. Needed to possibly spawn tasks.
+    runtime: R,
 }
 
 /// Description of how we got a channel.
@@ -218,11 +225,10 @@ impl<R: Runtime> ChanMgr<R> {
     /// other relays. Pass `None` for a client.
     pub fn new(
         runtime: R,
-        config: &ChannelConfig,
+        config: ChanMgrConfig,
         dormancy: Dormancy,
         netparams: &NetParameters,
         memquota: ToplevelAccount,
-        keymgr: Option<Arc<KeyMgr>>,
     ) -> Self
     where
         R: 'static,
@@ -231,18 +237,28 @@ impl<R: Runtime> ChanMgr<R> {
         let sender = Arc::new(std::sync::Mutex::new(sender));
         let reporter = BootstrapReporter(sender);
         let transport = transport::DefaultTransport::new(runtime.clone());
-        let builder = builder::ChanBuilder::new(runtime, transport, keymgr);
+        let builder = builder::ChanBuilder::new(runtime.clone(), transport);
+
+        // Set relay identity keys if we have any. Remember, bridges are relays but will be acting
+        // as a client when establishing channels.
+        #[cfg(feature = "relay")]
+        let builder = if let Some(ids) = &config.identities {
+            builder.with_identities(ids.clone())
+        } else {
+            builder
+        };
+
         let factory = factory::CompoundFactory::new(
             Arc::new(builder),
             #[cfg(feature = "pt-client")]
             None,
         );
         let mgr =
-            mgr::AbstractChanMgr::new(factory, config, dormancy, netparams, reporter, memquota);
+            mgr::AbstractChanMgr::new(factory, config.cfg, dormancy, netparams, reporter, memquota);
         ChanMgr {
             mgr,
             bootstrap_status: receiver,
-            runtime: std::marker::PhantomData,
+            runtime,
         }
     }
 
@@ -250,6 +266,7 @@ impl<R: Runtime> ChanMgr<R> {
     ///
     /// Returns a [`TaskHandle`] that can be used to manage
     /// those daemon tasks that poll periodically.
+    #[instrument(level = "trace", skip_all)]
     pub fn launch_background_tasks(
         self: &Arc<Self>,
         runtime: &R,
@@ -274,18 +291,20 @@ impl<R: Runtime> ChanMgr<R> {
 
     /// Build a channel for an incoming stream.
     ///
-    /// The channel may or may not be authenticated.
-    /// This method will wait until the channel is usable,
-    /// and may return an error if we already have an existing channel to this peer,
-    /// or if there are already too many open connections with this
-    /// peer or subnet (as a dos defence).
+    /// The `my_addrs` are the IP address(es) that are advertised by the relay in the consensus. We
+    /// need to pass them so they can be sent in the NETINFO cell.
+    ///
+    /// The channel may or may not be authenticated. This method will wait until the channel is
+    /// usable, and may return an error if we already have an existing channel to this peer, or if
+    /// there are already too many open connections with this peer or subnet (as a dos defence).
     #[cfg(feature = "relay")]
     pub async fn handle_incoming(
         &self,
-        src: std::net::SocketAddr,
+        src: Sensitive<std::net::SocketAddr>,
+        my_addrs: Vec<IpAddr>,
         stream: <R as tor_rtcompat::NetStreamProvider>::Stream,
     ) -> Result<Arc<Channel>> {
-        self.mgr.handle_incoming(src, stream).await
+        self.mgr.handle_incoming(src, my_addrs, stream).await
     }
 
     /// Try to get a suitable channel to the provided `target`,
@@ -294,6 +313,7 @@ impl<R: Runtime> ChanMgr<R> {
     /// If there is already a channel launch attempt in progress, this
     /// function will wait until that launch is complete, and succeed
     /// or fail depending on its outcome.
+    #[instrument(level = "trace", skip_all)]
     pub async fn get_or_launch<T: ChanTarget + ?Sized>(
         &self,
         target: &T,
@@ -381,6 +401,7 @@ impl<R: Runtime> ChanMgr<R> {
     /// make sure that no other code with access to this `ChanMgr` will be able
     /// to use the channel.
     #[cfg(feature = "experimental-api")]
+    #[instrument(level = "trace", skip_all)]
     pub async fn build_unmanaged_channel(
         &self,
         target: impl tor_linkspec::IntoOwnedChanTarget,
@@ -402,6 +423,7 @@ impl<R: Runtime> ChanMgr<R> {
     ///
     /// This is a daemon task that runs indefinitely in the background,
     /// and exits when we find that `chanmgr` is dropped.
+    #[instrument(level = "trace", skip_all)]
     async fn continually_update_channels_config(
         self_: Weak<Self>,
         netdir: Arc<dyn NetDirProvider>,
@@ -443,6 +465,7 @@ impl<R: Runtime> ChanMgr<R> {
     /// Exist when we find that `chanmgr` is dropped
     ///
     /// This is a daemon task that runs indefinitely in the background
+    #[instrument(level = "trace", skip_all)]
     async fn continually_expire_channels(mut sched: TaskSchedule<R>, chanmgr: Weak<Self>) {
         while sched.next().await.is_some() {
             let Some(cm) = Weak::upgrade(&chanmgr) else {
@@ -455,3 +478,35 @@ impl<R: Runtime> ChanMgr<R> {
         }
     }
 }
+
+#[cfg(feature = "relay")]
+#[async_trait]
+impl<R: Runtime> ChannelProvider for ChanMgr<R> {
+    type BuildSpec = OwnedChanTarget;
+
+    async fn get_or_launch(
+        self: Arc<Self>,
+        reactor_id: tor_proto::circuit::UniqId,
+        target: Self::BuildSpec,
+        tx: tor_proto::relay::channel_provider::OutboundChanSender,
+    ) -> tor_proto::Result<()> {
+        use tor_error::into_internal;
+
+        debug!("Get or launch channel to {target} for circuit reactor {reactor_id}");
+
+        let chanmgr = self.clone();
+        self.runtime
+            .spawn(async move {
+                let r = chanmgr
+                    .mgr
+                    .get_or_launch(target, ChannelUsage::UserTraffic)
+                    .await
+                    .map_err(|e| tor_proto::Error::ChanProto(e.to_string())); // Is it a ChanProto?
+                // Send back the channel.
+                tx.send(r.map(|(chan, _)| chan));
+            })
+            .map_err(into_internal!("Failed to launch channel provider task"))?;
+
+        Ok(())
+    }
+}
