@@ -1,15 +1,13 @@
 //! Abstract implementation of a channel manager
 
+use crate::factory::BootstrapReporter;
 use crate::mgr::state::{ChannelForTarget, PendingChannelHandle};
 use crate::util::defer::Defer;
 use crate::{ChanProvenance, ChannelConfig, ChannelUsage, Dormancy, Error, Result};
 
-use crate::factory::BootstrapReporter;
 use async_trait::async_trait;
 use futures::future::Shared;
 use oneshot_fused_workaround as oneshot;
-#[cfg(feature = "relay")]
-use safelog::Sensitive;
 use std::result::Result as StdResult;
 use std::sync::Arc;
 use std::time::Duration;
@@ -21,6 +19,9 @@ use tor_proto::channel::params::ChannelPaddingInstructionsUpdates;
 use tor_proto::memquota::{ChannelAccount, SpecificAccount as _, ToplevelAccount};
 use tracing::{instrument, trace};
 
+#[cfg(feature = "relay")]
+use {safelog::Sensitive, std::net::IpAddr, tor_proto::RelayIdentities};
+
 mod select;
 mod state;
 
@@ -93,11 +94,41 @@ pub(crate) trait AbstractChannelFactory {
     async fn build_channel_using_incoming(
         &self,
         peer: Sensitive<std::net::SocketAddr>,
+        my_addrs: Vec<IpAddr>,
         stream: Self::Stream,
         memquota: ChannelAccount,
     ) -> Result<Arc<Self::Channel>>;
 }
 
+/// This is the configuration for a [`ChanMgr`](crate::ChanMgr) given to the constructor.
+#[derive(Default)]
+pub struct ChanMgrConfig {
+    /// Channel configuration which usually comes from a configuration file.
+    pub(crate) cfg: ChannelConfig,
+    /// Relay identities needed for relay channels.
+    #[cfg(feature = "relay")]
+    pub(crate) identities: Option<Arc<RelayIdentities>>,
+    // TODO: Would be good to add more things such as NetParameters and Dormancy maybe?
+}
+
+impl ChanMgrConfig {
+    /// Constructor.
+    pub fn new(cfg: ChannelConfig) -> Self {
+        Self {
+            cfg,
+            #[cfg(feature = "relay")]
+            identities: None,
+        }
+    }
+
+    /// Set the relay identities and return itself.
+    #[cfg(feature = "relay")]
+    pub fn with_identities(mut self, ids: Arc<RelayIdentities>) -> Self {
+        self.identities = Some(ids);
+        self
+    }
+}
+
 /// A type- and network-agnostic implementation for [`ChanMgr`](crate::ChanMgr).
 ///
 /// This type does the work of keeping track of open channels and pending
@@ -132,14 +163,14 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
     /// Make a new empty channel manager.
     pub(crate) fn new(
         connector: CF,
-        config: &ChannelConfig,
+        config: ChannelConfig,
         dormancy: Dormancy,
         netparams: &NetParameters,
         reporter: BootstrapReporter,
         memquota: ToplevelAccount,
     ) -> Self {
         AbstractChanMgr {
-            channels: state::MgrState::new(connector, config.clone(), dormancy, netparams),
+            channels: state::MgrState::new(connector, config, dormancy, netparams),
             reporter,
             memquota,
         }
@@ -166,12 +197,13 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
     pub(crate) async fn handle_incoming(
         &self,
         src: Sensitive<std::net::SocketAddr>,
+        my_addrs: Vec<IpAddr>,
         stream: CF::Stream,
     ) -> Result<Arc<CF::Channel>> {
         let chan_builder = self.channels.builder();
         let memquota = ChannelAccount::new(&self.memquota)?;
         let _outcome = chan_builder
-            .build_channel_using_incoming(src, stream, memquota)
+            .build_channel_using_incoming(src, my_addrs, stream, memquota)
             .await?;
 
         // TODO RELAY: we need to do something with the channel here now that we've created it
@@ -545,7 +577,7 @@ mod test {
         let cf = FakeChannelFactory::new(runtime);
         AbstractChanMgr::new(
             cf,
-            &ChannelConfig::default(),
+            Default::default(),
             Default::default(),
             &Default::default(),
             BootstrapReporter::fake(),
@@ -613,6 +645,7 @@ mod test {
         async fn build_channel_using_incoming(
             &self,
             _peer: Sensitive<std::net::SocketAddr>,
+            _my_addrs: Vec<IpAddr>,
             _stream: Self::Stream,
             _memquota: ChannelAccount,
         ) -> Result<Arc<Self::Channel>> {
