@@ -23,7 +23,7 @@ use tor_proto::channel::kist::{KistMode, KistParams};
 use tor_proto::channel::padding::Parameters as PaddingParameters;
 use tor_proto::channel::padding::ParametersBuilder as PaddingParametersBuilder;
 use tor_units::{BoundedInt32, IntegerMilliseconds};
-use tracing::info;
+use tracing::{info, instrument};
 use void::{ResultVoidExt as _, Void};
 
 #[cfg(test)]
@@ -496,6 +496,7 @@ impl<C: AbstractChannelFactory> MgrState<C> {
 
     /// Upgrade the pending channel identified by its `handle` by replacing it with a new open
     /// `channel`.
+    #[instrument(skip_all, level = "trace")]
     pub(crate) fn upgrade_pending_channel_to_open(
         &self,
         handle: PendingChannelHandle,
@@ -862,7 +863,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -874,6 +875,8 @@ mod test {
     use tor_llcrypto::pk::ed25519::Ed25519Identity;
     use tor_proto::channel::params::ChannelPaddingInstructionsUpdates;
     use tor_proto::memquota::ChannelAccount;
+    #[cfg(feature = "relay")]
+    use {safelog::Sensitive, std::net::IpAddr};
 
     fn new_test_state() -> MgrState<FakeChannelFactory> {
         MgrState::new(
@@ -908,7 +911,8 @@ mod test {
         #[cfg(feature = "relay")]
         async fn build_channel_using_incoming(
             &self,
-            _peer: std::net::SocketAddr,
+            _peer: Sensitive<std::net::SocketAddr>,
+            _my_addrs: Vec<IpAddr>,
             _stream: Self::Stream,
             _memquota: ChannelAccount,
         ) -> Result<Arc<Self::Channel>> {
