@@ -5,7 +5,6 @@ use crate::timeouts::{self, Action};
 use crate::{Error, Result};
 use async_trait::async_trait;
 use futures::Future;
-use futures::task::SpawnExt;
 use oneshot_fused_workaround as oneshot;
 use std::sync::{
     Arc,
@@ -20,8 +19,10 @@ use tor_netdir::params::NetParameters;
 use tor_proto::ccparams::{self, AlgorithmType};
 use tor_proto::client::circuit::{CircParameters, PendingClientTunnel};
 use tor_proto::{CellCount, ClientTunnel, FlowCtrlParameters};
+use tor_rtcompat::SpawnExt;
 use tor_rtcompat::{Runtime, SleepProviderExt};
 use tor_units::Percentage;
+use tracing::instrument;
 
 #[cfg(all(feature = "vanguards", feature = "hs-common"))]
 use tor_guardmgr::vanguards::VanguardMgr;
@@ -89,6 +90,7 @@ pub(crate) trait Buildable: Sized {
 ///
 /// This is common code, shared by all the first-hop functions in the
 /// implementation of `Buildable` for `ClientTunnel`.
+#[instrument(level = "trace", skip_all)]
 async fn create_common<RT: Runtime>(
     chan: Arc<tor_proto::channel::Channel>,
     timeouts: Arc<dyn tor_proto::client::circuit::TimeoutEstimator>,
@@ -100,11 +102,13 @@ async fn create_common<RT: Runtime>(
             .await
             .map_err(|error| Error::Protocol {
                 error,
-                peer: None, // we don't blame the peer, because new_circ() does no networking.
+                peer: None, // we don't blame the peer, because new_tunnel() does no networking.
                 action: "initializing circuit",
                 unique_id: None,
             })?;
 
+    tracing::debug!("Spawning reactor...");
+
     rt.spawn(async {
         let _ = reactor.run().await;
     })
@@ -117,6 +121,7 @@ async fn create_common<RT: Runtime>(
 impl Buildable for ClientTunnel {
     type Chan = tor_proto::channel::Channel;
 
+    #[instrument(level = "trace", skip_all)]
     async fn open_channel<RT: Runtime>(
         chanmgr: &ChanMgr<RT>,
         target: &OwnedChanTarget,
@@ -148,6 +153,7 @@ impl Buildable for ClientTunnel {
         }
     }
 
+    #[instrument(level = "trace", skip_all)]
     async fn create_chantarget<RT: Runtime>(
         chan: Arc<Self::Chan>,
         rt: &RT,
@@ -167,6 +173,7 @@ impl Buildable for ClientTunnel {
                 unique_id,
             })
     }
+    #[instrument(level = "trace", skip_all)]
     async fn create<RT: Runtime>(
         chan: Arc<Self::Chan>,
         rt: &RT,
@@ -251,6 +258,7 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
     ///
     /// (TODO: Find
     /// a better design there.)
+    #[instrument(level = "trace", skip_all)]
     async fn build_notimeout(
         self: Arc<Self>,
         path: OwnedPath,
@@ -301,6 +309,7 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
     }
 
     /// Build a circuit from an [`OwnedPath`].
+    #[instrument(level = "trace", skip_all)]
     async fn build_owned(
         self: &Arc<Self>,
         path: OwnedPath,
@@ -446,6 +455,7 @@ impl<R: Runtime> TunnelBuilder<R> {
     }
 
     /// Reload persistent state from disk, if we don't have storage permission.
+    #[instrument(level = "trace", skip_all)]
     pub(crate) fn reload_state(&self) -> Result<()> {
         if !self.storage.can_store() {
             self.builder
@@ -464,6 +474,7 @@ impl<R: Runtime> TunnelBuilder<R> {
     }
 
     /// Like `build`, but construct a new circuit from an [`OwnedPath`].
+    #[instrument(level = "trace", skip_all)]
     pub(crate) async fn build_owned(
         &self,
         path: OwnedPath,
@@ -482,6 +493,7 @@ impl<R: Runtime> TunnelBuilder<R> {
     /// This circuit is _not_ automatically registered with any
     /// circuit manager; if you don't hang on it it, it will
     /// automatically go away when the last reference is dropped.
+    #[instrument(level = "trace", skip_all)]
     pub async fn build(
         &self,
         path: &TorPath<'_>,
@@ -718,7 +730,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -726,7 +738,6 @@ mod test {
     use crate::timeouts::TimeoutEstimator;
     use futures::FutureExt;
     use std::sync::Mutex;
-    use tor_chanmgr::ChannelConfig;
     use tor_chanmgr::ChannelUsage as CU;
     use tor_linkspec::ChanTarget;
     use tor_linkspec::{HasRelayIds, RelayIdType, RelayIds};
@@ -1071,11 +1082,10 @@ mod test {
     ) -> (Result<FakeCirc>, Vec<(bool, u8, Duration)>) {
         let chanmgr = Arc::new(ChanMgr::new(
             rt.clone(),
-            &ChannelConfig::default(),
+            Default::default(),
             Default::default(),
             &Default::default(),
             ToplevelAccount::new_noop(),
-            None,
         ));
         // always has 3 second timeout, 100 second abandon.
         let timeouts = match advance_on_timeout {
