@@ -3,6 +3,7 @@
 use std::time::SystemTime;
 
 use rand::Rng;
+use tracing::instrument;
 
 use super::{AnonymousPathBuilder, TorPath};
 use crate::path::pick_path;
@@ -75,7 +76,6 @@ impl ExitPathBuilder {
     }
 
     #[cfg(feature = "geoip")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "geoip")))]
     /// Create a new builder that will try to get an exit relay in `country`,
     /// containing all the ports in `ports`.
     ///
@@ -104,6 +104,7 @@ impl ExitPathBuilder {
 
     /// Try to create and return a path corresponding to the requirements of
     /// this builder.
+    #[instrument(skip_all, level = "trace")]
     pub(crate) fn pick_path<'a, R: Rng, RT: Runtime>(
         &self,
         rng: &mut R,
@@ -202,7 +203,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
