@@ -65,7 +65,7 @@ use std::fmt::{self, Display};
 use std::marker::PhantomData;
 use std::mem;
 
-use derive_deftly::{define_derive_deftly, derive_deftly_adhoc, Deftly};
+use derive_deftly::{Deftly, define_derive_deftly, derive_deftly_adhoc};
 use paste::paste;
 use serde::de::{self, DeserializeSeed, Deserializer, Error as _, IgnoredAny, MapAccess, Visitor};
 use serde::{Deserialize, Serialize, Serializer};
@@ -105,11 +105,12 @@ define_derive_deftly! {
     //  https://docs.rs/derive-deftly/latest/derive_deftly/macro.define_derive_deftly.html#exporting-a-template-for-use-by-other-crates
     //
     // The semantic behaviour of the template *does* have semver implications.
-    pub Flattenable for struct, expect items =
+    export Flattenable for struct, expect items:
 
-    impl tor_config::Flattenable for $ttype {
+    impl<$tgens> $crate::Flattenable for $ttype
+    where $twheres {
         fn has_field(s: &str) -> bool {
-            let fnames = tor_config::flattenable_extract_fields::<'_, Self>();
+            let fnames = $crate::flattenable_extract_fields::<'_, Self>();
             IntoIterator::into_iter(fnames).any(|f| *f == s)
 
         }
@@ -123,9 +124,10 @@ define_derive_deftly! {
         // and thereby ensures that we didn't have a mismatch that
         // allows broken impls to slip through.
         // (We know the type is at least similar because we go via the Flattenable impl.)
-        let _: bool = <$ttype as tor_config::Flattenable>::has_field("");
+        let _: bool = <$ttype as $crate::Flattenable>::has_field("");
     }
 }
+pub use derive_deftly_template_Flattenable;
 
 /// Helper for flattening deserialisation, compatible with [`serde_ignored`]
 ///
@@ -554,12 +556,11 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
-    use crate as tor_config; // for the benefit of the macros
 
     use std::collections::HashMap;
 
