@@ -32,7 +32,8 @@
 //!  * Defines a configuration structure `EmbedderConfig`,
 //!    for additional configuration settings for the added features.
 //!  * Establishes some configuration sources
-//!    (the trivial empty `ConfigSources`, to avoid clutter in the example)
+//!    (the trivial empty [`ConfigurationSources`](crate::ConfigurationSources),
+//!    to avoid clutter in the example)
 //!  * Reads those sources into a single configuration taxonomy [`ConfigurationTree`].
 //!  * Processes that configuration into a 3-tuple of configuration
 //!    structs for the three components, namely:
@@ -91,7 +92,7 @@ use std::fmt::{self, Display};
 use std::iter;
 use std::mem;
 
-use itertools::{chain, izip, Itertools};
+use itertools::{Itertools, chain, izip};
 use serde::de::DeserializeOwned;
 use thiserror::Error;
 use tracing::warn;
@@ -111,10 +112,26 @@ pub enum ConfigResolveError {
     Build(#[from] ConfigBuildError),
 }
 
-/// A type that can be built from a builder via a build method
+/// A type that can be built from a builder.
+pub trait Buildable {
+    /// The type that constructs this Buildable.
+    ///
+    /// Typically, this type will implement [`Builder`].
+    /// If it does, then `<Self::Builder>::Built` should be `Self`.
+    type Builder;
+
+    /// Return a new Builder for this type.
+    fn builder() -> Self::Builder;
+}
+
+/// A type that can build some buildable type via a build method.
 pub trait Builder {
-    /// The type that this builder constructs
+    /// The type that this builder constructs.
+    ///
+    /// Typically, this type will implement [`Buildable`].
+    /// If it does, then `<Self::Built as Buildable>::Builder` should be `Self`.
     type Built;
+
     /// Build into a `Built`
     ///
     /// Often shadows an inherent `build` method
@@ -495,8 +512,8 @@ where
 
 /// Turns a [`serde_ignored::Path`] (which is borrowed) into an owned `DisfavouredKey`
 fn copy_path(mut path: &serde_ignored::Path) -> DisfavouredKey {
-    use serde_ignored::Path as SiP;
     use PathEntry as PE;
+    use serde_ignored::Path as SiP;
 
     let mut descend = vec![];
     loop {
@@ -603,7 +620,7 @@ fn intersect_unrecognized_lists(
             let item = inputs
                 .iter_mut()
                 .map(|input| input.next().expect("but peeked"))
-                .last()
+                .next_back()
                 .expect("wrong #");
             output.insert(item);
             continue;
@@ -689,7 +706,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
