@@ -26,7 +26,7 @@ use tor_dirclient::DirResponse;
 use tor_error::{info_report, warn_report};
 use tor_rtcompat::Runtime;
 use tor_rtcompat::scheduler::TaskSchedule;
-use tracing::{debug, info, trace, warn};
+use tracing::{debug, info, instrument, trace, warn};
 
 use crate::storage::Store;
 #[cfg(test)]
@@ -273,6 +273,7 @@ pub(crate) fn make_requests_for_documents<R: Runtime>(
 }
 
 /// Launch a single client request and get an associated response.
+#[instrument(level = "trace", skip_all)]
 async fn fetch_single<R: Runtime>(
     rt: &R,
     request: ClientRequest,
@@ -305,6 +306,7 @@ static CANNED_RESPONSE: LazyLock<Mutex<Vec<String>>> = LazyLock::new(|| Mutex::n
 ///
 /// Don't launch more than `parallelism` requests at once.
 #[allow(clippy::cognitive_complexity)] // TODO: maybe refactor?
+#[instrument(level = "trace", skip_all)]
 async fn fetch_multiple<R: Runtime>(
     dirmgr: Arc<DirMgr<R>>,
     attempt_id: AttemptId,
@@ -366,7 +368,7 @@ async fn fetch_multiple<R: Runtime>(
 }
 
 /// Try to update `state` by loading cached information from `dirmgr`.
-async fn load_once<R: Runtime>(
+fn load_once<R: Runtime>(
     dirmgr: &Arc<DirMgr<R>>,
     state: &mut Box<dyn DirState>,
     attempt_id: AttemptId,
@@ -402,8 +404,8 @@ async fn load_once<R: Runtime>(
 ///
 /// No downloads are performed; the provided state will not be reset.
 #[allow(clippy::cognitive_complexity)] // TODO: Refactor? Somewhat due to tracing.
-pub(crate) async fn load<R: Runtime>(
-    dirmgr: Arc<DirMgr<R>>,
+pub(crate) fn load<R: Runtime>(
+    dirmgr: &Arc<DirMgr<R>>,
     mut state: Box<dyn DirState>,
     attempt_id: AttemptId,
 ) -> Result<Box<dyn DirState>> {
@@ -411,7 +413,7 @@ pub(crate) async fn load<R: Runtime>(
     loop {
         trace!(attempt=%attempt_id, state=%state.describe(), "Loading from cache");
         let mut changed = false;
-        let outcome = load_once(&dirmgr, &mut state, attempt_id, &mut changed).await;
+        let outcome = load_once(dirmgr, &mut state, attempt_id, &mut changed);
         {
             let mut store = dirmgr.store.lock().expect("store lock poisoned");
             dirmgr.apply_netdir_changes(&mut state, &mut **store)?;
@@ -458,6 +460,7 @@ pub(crate) async fn load<R: Runtime>(
 /// This can launch one or more download requests, but will not launch more
 /// than `parallelism` requests at a time.
 #[allow(clippy::cognitive_complexity)] // TODO: Refactor?
+#[instrument(level = "trace", skip_all)]
 async fn download_attempt<R: Runtime>(
     dirmgr: &Arc<DirMgr<R>>,
     state: &mut Box<dyn DirState>,
@@ -542,6 +545,7 @@ async fn download_attempt<R: Runtime>(
 /// The first time that the state becomes ["usable"](Readiness::Usable), notify
 /// the sender in `on_usable`.
 #[allow(clippy::cognitive_complexity)] // TODO: Refactor!
+#[instrument(level = "trace", skip_all)]
 pub(crate) async fn download<R: Runtime>(
     dirmgr: Weak<DirMgr<R>>,
     state: &mut Box<dyn DirState>,
@@ -564,7 +568,7 @@ pub(crate) async fn download<R: Runtime>(
             let dirmgr = upgrade_weak_ref(&dirmgr)?;
             let mut changed = false;
             trace!(attempt=%attempt_id, state=%state.describe(),"Attempting to load directory information from cache.");
-            let load_result = load_once(&dirmgr, state, attempt_id, &mut changed).await;
+            let load_result = load_once(&dirmgr, state, attempt_id, &mut changed);
             trace!(attempt=%attempt_id, state=%state.describe(), outcome=?load_result, "Load attempt complete.");
             if let Err(e) = &load_result {
                 // If the load failed but the error can be blamed on a directory
@@ -733,7 +737,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -906,9 +910,7 @@ mod test {
 
             // Try just a load.
             let state = Box::new(DemoState::new1());
-            let result = super::load(Arc::clone(&mgr), state, attempt_id)
-                .await
-                .unwrap();
+            let result = super::load(&mgr, state, attempt_id).unwrap();
             assert!(result.is_ready(Readiness::Complete));
 
             // Try a bootstrap that could (but won't!) download.
