@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -42,6 +42,7 @@
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
 #![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 // This clippy lint produces a false positive on `use strum`, below.
@@ -83,12 +84,12 @@ use tor_netdir::params::NetParameters;
 use tor_netdir::{DirEvent, MdReceiver, NetDir, NetDirProvider};
 
 use async_trait::async_trait;
-use futures::{stream::BoxStream, task::SpawnExt};
+use futures::stream::BoxStream;
 use oneshot_fused_workaround as oneshot;
 use tor_netdoc::doc::netstatus::ProtoStatuses;
-use tor_rtcompat::Runtime;
 use tor_rtcompat::scheduler::{TaskHandle, TaskSchedule};
-use tracing::{debug, info, trace, warn};
+use tor_rtcompat::{Runtime, SpawnExt};
+use tracing::{debug, info, instrument, trace, warn};
 
 use std::marker::PhantomData;
 use std::sync::atomic::{AtomicBool, Ordering};
@@ -233,6 +234,7 @@ impl<R: Runtime> DirProvider for Arc<DirMgr<R>> {
         DirMgr::reconfigure(self, new_config, how)
     }
 
+    #[instrument(level = "trace", skip_all)]
     async fn bootstrap(&self) -> Result<()> {
         DirMgr::bootstrap(self).await
     }
@@ -361,14 +363,14 @@ impl<R: Runtime> DirMgr<R> {
     /// In general, you shouldn't use this function in a long-running
     /// program; it's only suitable for command-line or batch tools.
     // TODO: I wish this function didn't have to be async or take a runtime.
-    pub async fn load_once(runtime: R, config: DirMgrConfig) -> Result<Arc<NetDir>> {
+    pub fn load_once(runtime: R, config: DirMgrConfig) -> Result<Arc<NetDir>> {
         let store = DirMgrStore::new(&config, runtime.clone(), true)?;
         let dirmgr = Arc::new(Self::from_config(config, runtime, store, None, true)?);
 
         // TODO: add some way to return a directory that isn't up-to-date
         let attempt = AttemptId::next();
         trace!(%attempt, "Trying to load a full directory from cache");
-        let outcome = dirmgr.load_directory(attempt).await;
+        let outcome = dirmgr.load_directory(attempt);
         trace!(%attempt, "Load result: {outcome:?}");
         let _success = outcome?;
 
@@ -437,6 +439,7 @@ impl<R: Runtime> DirMgr<R> {
     /// Panics if the `DirMgr` passed to this function was not created in online mode, such as
     /// via `load_once`.
     #[allow(clippy::cognitive_complexity)] // TODO: Refactor
+    #[instrument(level = "trace", skip_all)]
     pub async fn bootstrap(self: &Arc<Self>) -> Result<()> {
         if self.offline {
             return Err(Error::OfflineMode);
@@ -477,7 +480,7 @@ impl<R: Runtime> DirMgr<R> {
         // Try to load from the cache.
         let attempt_id = AttemptId::next();
         trace!(attempt=%attempt_id, "Starting to bootstrap directory");
-        let have_directory = self.load_directory(attempt_id).await?;
+        let have_directory = self.load_directory(attempt_id)?;
 
         let (mut sender, receiver) = if have_directory {
             info!("Loaded a good directory from cache.");
@@ -549,6 +552,7 @@ impl<R: Runtime> DirMgr<R> {
 
     /// Return a new directory manager from a given configuration,
     /// bootstrapping from the network as necessary.
+    #[instrument(level = "trace", skip_all)]
     pub async fn bootstrap_from_config(
         config: DirMgrConfig,
         runtime: R,
@@ -625,7 +629,7 @@ impl<R: Runtime> DirMgr<R> {
             {
                 let dirmgr = upgrade_weak_ref(weak)?;
                 trace!("Trying to load from the directory cache");
-                if dirmgr.load_directory(attempt_id).await? {
+                if dirmgr.load_directory(attempt_id)? {
                     // Successfully loaded a bootstrapped directory.
                     if let Some(send_done) = on_complete.take() {
                         let _ = send_done.send(());
@@ -644,6 +648,7 @@ impl<R: Runtime> DirMgr<R> {
     /// If we have begin to have a bootstrapped directory, send a
     /// message using `on_complete`.
     #[allow(clippy::cognitive_complexity)] // TODO: Refactor?
+    #[instrument(level = "trace", skip_all)]
     async fn download_forever(
         weak: Weak<Self>,
         schedule: &mut TaskSchedule<R>,
@@ -940,7 +945,7 @@ impl<R: Runtime> DirMgr<R> {
     /// cache, if it is newer than the one we have.
     ///
     /// Return false if there is no such consensus.
-    async fn load_directory(self: &Arc<Self>, attempt_id: AttemptId) -> Result<bool> {
+    fn load_directory(self: &Arc<Self>, attempt_id: AttemptId) -> Result<bool> {
         let state = state::GetConsensusState::new(
             self.runtime.clone(),
             self.config.get(),
@@ -951,7 +956,7 @@ impl<R: Runtime> DirMgr<R> {
                 .clone()
                 .unwrap_or_else(|| Arc::new(crate::filter::NilFilter)),
         );
-        let _ = bootstrap::load(Arc::clone(self), Box::new(state), attempt_id).await?;
+        let _ = bootstrap::load(self, Box::new(state), attempt_id)?;
 
         Ok(self.netdir.get().is_some())
     }
@@ -1190,7 +1195,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
