@@ -38,27 +38,114 @@
 //! eventually resulting in a calling a synchronous function taking the
 //! transaction handle to perform the lion's share of the operation.
 
+// TODO DIRMIRROR: This could benefit from methods by wrapping the pool into a
+// custom type.
+
 use std::{
+    fmt::Display,
+    io::{Cursor, Write},
     num::NonZero,
     ops::{Add, Sub},
     path::Path,
     time::{Duration, SystemTime},
 };
 
+use flate2::write::{DeflateEncoder, GzEncoder};
 use r2d2::Pool;
 use r2d2_sqlite::SqliteConnectionManager;
 use rusqlite::{
-    params,
-    types::{FromSql, FromSqlResult, ToSqlOutput, ValueRef},
+    named_params, params,
+    types::{FromSql, FromSqlError, FromSqlResult, ToSqlOutput, ValueRef},
     ToSql, Transaction, TransactionBehavior,
 };
 use saturating_time::SaturatingTime;
+use sha2::Digest;
+use tor_error::into_internal;
 
 use crate::err::DatabaseError;
 
-/// Representation of a Sha256 hash in hexadecimal (upper-case)
-// TODO: Make this a real type that actually enforces the constraints.
-pub(crate) type Sha256 = String;
+/// The identifier for documents in the content-addressable cache.
+///
+/// Right now, this is a Sha256 hash, but this may change in future.
+#[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
+pub(crate) struct DocumentId([u8; 32]);
+
+impl DocumentId {
+    /// Computes the [`DocumentId`] from arbitrary data.
+    pub(crate) fn digest(data: &[u8]) -> Self {
+        Self(sha2::Sha256::digest(data).into())
+    }
+}
+
+impl Display for DocumentId {
+    /// Formats the [`DocumentId`] in uppercase hexadecimal.
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}", hex::encode_upper(self.0))
+    }
+}
+
+impl FromSql for DocumentId {
+    fn column_result(value: ValueRef<'_>) -> FromSqlResult<Self> {
+        // We read the document id as a hexadecimal string from the database.
+        // Afterwards, we convert it to binary data, which should succeed due
+        // to database check constraints.  Finally, we verify the length to see
+        // whether it actually constitutes a valid SHA256 checksum.
+        let data: [u8; 32] = value
+            .as_str()
+            .map(hex::decode)?
+            .map_err(|e| {
+                FromSqlError::Other(Box::new(tor_error::internal!(
+                    "non hex data in database? {e}"
+                )))
+            })?
+            .try_into()
+            .map_err(|_| {
+                FromSqlError::Other(Box::new(tor_error::internal!(
+                    "document id with invalid length in database?"
+                )))
+            })?;
+
+        Ok(Self(data))
+    }
+}
+
+impl ToSql for DocumentId {
+    fn to_sql(&self) -> rusqlite::Result<ToSqlOutput<'_>> {
+        // Because Self is only constructed with FromSql and digest data, it is
+        // safe to assume to be valid.  Even if not, database constraints will
+        // catch us from inserting invalid data.
+        Ok(ToSqlOutput::from(self.to_string()))
+    }
+}
+
+impl PartialEq<&str> for DocumentId {
+    fn eq(&self, other: &&str) -> bool {
+        self.to_string() == other.to_uppercase()
+    }
+}
+
+#[cfg(test)]
+impl From<[u8; 32]> for DocumentId {
+    fn from(value: [u8; 32]) -> Self {
+        Self(value)
+    }
+}
+
+/// The supported content encodings.
+#[derive(Debug, Clone, Copy, PartialEq, strum::EnumString, strum::Display, strum::EnumIter)]
+#[strum(serialize_all = "kebab-case", ascii_case_insensitive)]
+pub(crate) enum ContentEncoding {
+    /// RFC2616 section 3.5.
+    Identity,
+    /// RFC2616 section 3.5.
+    Deflate,
+    /// RFC2616 section 3.5.
+    Gzip,
+    /// The zstandard compression algorithm (www.zstd.net).
+    XZstd,
+    /// The lzma compression algorithm with a "present" value no higher than 6.
+    XTorLzma,
+}
 
 /// A wrapper around [`SystemTime`] with convenient features.
 ///
@@ -95,6 +182,12 @@ impl From<SystemTime> for Timestamp {
     }
 }
 
+impl From<Timestamp> for SystemTime {
+    fn from(value: Timestamp) -> Self {
+        value.0
+    }
+}
+
 impl Add<Duration> for Timestamp {
     type Output = Self;
 
@@ -158,13 +251,10 @@ macro_rules! sql {
 pub(crate) use sql;
 
 /// Version 1 of the database schema.
-///
-/// TODO DIRMIRROR: Should we rename arti_dirmirror_schema_version to say
-/// dirserver or something more generic?
 const V1_SCHEMA: &str = sql!(
     "
 -- Meta table to store the current schema version.
-CREATE TABLE arti_dirmirror_schema_version(
+CREATE TABLE arti_dirserver_schema_version(
     version TEXT NOT NULL -- currently, always `1`
 ) STRICT;
 
@@ -175,7 +265,7 @@ CREATE TABLE arti_dirmirror_schema_version(
 -- http://<hostname>/tor/status-vote/current/consensus-<FLAVOR>/diff/<HASH>/<FPRLIST>
 CREATE TABLE consensus(
     rowid               INTEGER PRIMARY KEY AUTOINCREMENT,
-    sha256              TEXT NOT NULL UNIQUE,
+    docid               TEXT NOT NULL UNIQUE,
     -- Required for consensus diffs.
     -- https://spec.torproject.org/dir-spec/directory-cache-operation.html#diff-format
     unsigned_sha3_256   TEXT NOT NULL UNIQUE,
@@ -183,7 +273,7 @@ CREATE TABLE consensus(
     valid_after         INTEGER NOT NULL,
     fresh_until         INTEGER NOT NULL,
     valid_until         INTEGER NOT NULL,
-    FOREIGN KEY(sha256) REFERENCES store(sha256),
+    FOREIGN KEY(docid) REFERENCES store(docid),
     CHECK(GLOB('*[^0-9A-F]*', unsigned_sha3_256) == 0),
     CHECK(LENGTH(unsigned_sha3_256) == 64),
     CHECK(flavor IN ('ns', 'md')),
@@ -199,10 +289,10 @@ CREATE TABLE consensus(
 -- http://<hostname>/tor/status-vote/current/consensus-<FLAVOR>/diff/<HASH>/<FPRLIST>
 CREATE TABLE consensus_diff(
     rowid                   INTEGER PRIMARY KEY AUTOINCREMENT,
-    sha256                  TEXT NOT NULL UNIQUE,
+    docid                   TEXT NOT NULL UNIQUE,
     old_consensus_rowid     INTEGER NOT NULL,
     new_consensus_rowid     INTEGER NOT NULL,
-    FOREIGN KEY(sha256) REFERENCES store(sha256),
+    FOREIGN KEY(docid) REFERENCES store(docid),
     FOREIGN KEY(old_consensus_rowid) REFERENCES consensus(rowid),
     FOREIGN KEY(new_consensus_rowid) REFERENCES consensus(rowid)
 ) STRICT;
@@ -215,16 +305,18 @@ CREATE TABLE consensus_diff(
 -- http://<hostname>/tor/server/all
 CREATE TABLE router_descriptor(
     rowid                   INTEGER PRIMARY KEY AUTOINCREMENT,
-    sha256                  TEXT NOT NULL UNIQUE,
+    docid                   TEXT NOT NULL UNIQUE,
     sha1                    TEXT NOT NULL UNIQUE,
+    sha2                    TEXT NOT NULL UNIQUE,
     kp_relay_id_rsa_sha1    TEXT NOT NULL,
     flavor                  TEXT NOT NULL,
     router_extra_info_rowid  INTEGER,
-    FOREIGN KEY(sha256) REFERENCES store(sha256),
+    FOREIGN KEY(docid) REFERENCES store(docid),
     FOREIGN KEY(router_extra_info_rowid) REFERENCES router_extra_info(rowid),
     CHECK(GLOB('*[^0-9A-F]*', sha1) == 0),
     CHECK(GLOB('*[^0-9A-F]*', kp_relay_id_rsa_sha1) == 0),
     CHECK(LENGTH(sha1) == 40),
+    CHECK(docid == sha2),
     CHECK(LENGTH(kp_relay_id_rsa_sha1) == 40),
     CHECK(flavor IN ('ns', 'md'))
 ) STRICT;
@@ -237,10 +329,10 @@ CREATE TABLE router_descriptor(
 -- http://<hostname>/tor/extra/authority
 CREATE TABLE router_extra_info(
     rowid                   INTEGER PRIMARY KEY AUTOINCREMENT,
-    sha256                  TEXT NOT NULL UNIQUE,
+    docid                   TEXT NOT NULL UNIQUE,
     sha1                    TEXT NOT NULL UNIQUE,
     kp_relay_id_rsa_sha1    TEXT NOT NULL,
-    FOREIGN KEY(sha256) REFERENCES store(sha256),
+    FOREIGN KEY(docid) REFERENCES store(docid),
     CHECK(GLOB('*[^0-9A-F]*', sha1) == 0),
     CHECK(GLOB('*[^0-9A-F]*', kp_relay_id_rsa_sha1) == 0),
     CHECK(LENGTH(sha1) == 40),
@@ -257,37 +349,40 @@ CREATE TABLE router_extra_info(
 -- http://<hostname>/tor/keys/sk/<F>-<S>
 CREATE TABLE authority_key_certificate(
     rowid                   INTEGER PRIMARY KEY AUTOINCREMENT,
-    sha256                  TEXT NOT NULL UNIQUE,
+    docid                   TEXT NOT NULL UNIQUE,
     kp_auth_id_rsa_sha1     TEXT NOT NULL,
     kp_auth_sign_rsa_sha1   TEXT NOT NULL,
+    dir_key_published       INTEGER NOT NULL,
     dir_key_expires         INTEGER NOT NULL,
-    FOREIGN KEY(sha256) REFERENCES store(sha256),
+    FOREIGN KEY(docid) REFERENCES store(docid),
     CHECK(GLOB('*[^0-9A-F]*', kp_auth_id_rsa_sha1) == 0),
     CHECK(GLOB('*[^0-9A-F]*', kp_auth_sign_rsa_sha1) == 0),
     CHECK(LENGTH(kp_auth_id_rsa_sha1) == 40),
     CHECK(LENGTH(kp_auth_sign_rsa_sha1) == 40),
-    CHECK(dir_key_expires >= 0)
+    CHECK(dir_key_published >= 0),
+    CHECK(dir_key_expires >= 0),
+    CHECK(dir_key_published < dir_key_expires)
 
 ) STRICT;
 
 -- Content addressable storage, storing all contents.
 CREATE TABLE store(
     rowid   INTEGER PRIMARY KEY AUTOINCREMENT, -- hex uppercase
-    sha256  TEXT NOT NULL UNIQUE,
+    docid   TEXT NOT NULL UNIQUE,
     content BLOB NOT NULL,
-    CHECK(GLOB('*[^0-9A-F]*', sha256) == 0),
-    CHECK(LENGTH(sha256) == 64)
+    CHECK(GLOB('*[^0-9A-F]*', docid) == 0),
+    CHECK(LENGTH(docid) == 64)
 ) STRICT;
 
 -- Stores compressed network documents.
 CREATE TABLE compressed_document(
     rowid               INTEGER PRIMARY KEY AUTOINCREMENT,
     algorithm           TEXT NOT NULL,
-    identity_sha256     TEXT NOT NULL,
-    compressed_sha256   TEXT NOT NULL,
-    FOREIGN KEY(identity_sha256) REFERENCES store(sha256),
-    FOREIGN KEY(compressed_sha256) REFERENCES store(sha256),
-    UNIQUE(algorithm, identity_sha256)
+    identity_docid      TEXT NOT NULL,
+    compressed_docid   TEXT NOT NULL,
+    FOREIGN KEY(identity_docid) REFERENCES store(docid),
+    FOREIGN KEY(compressed_docid) REFERENCES store(docid),
+    UNIQUE(algorithm, identity_docid)
 ) STRICT;
 
 -- Stores the N:M cardinality of which router descriptors are contained in which
@@ -314,7 +409,7 @@ CREATE TABLE consensus_authority_voter(
     FOREIGN KEY(authority_rowid) REFERENCES authority_key_certificate(rowid)
 ) STRICT;
 
-INSERT INTO arti_dirmirror_schema_version VALUES ('1');
+INSERT INTO arti_dirserver_schema_version VALUES ('1');
 "
 );
 
@@ -367,13 +462,13 @@ pub(crate) fn open<P: AsRef<Path>>(
         // 1. Checking the database schema.
         // 2. Upgrading (in future) or initializing the database schema (if empty).
 
-        let has_arti_dirmirror_schema_version = match tx.query_one(
+        let has_arti_dirserver_schema_version = match tx.query_one(
             sql!(
                 "
                 SELECT name
                 FROM sqlite_master
                   WHERE type = 'table'
-                    AND name = 'arti_dirmirror_schema_version'
+                    AND name = 'arti_dirserver_schema_version'
                 "
             ),
             params![],
@@ -384,9 +479,9 @@ pub(crate) fn open<P: AsRef<Path>>(
             Err(e) => return Err(DatabaseError::LowLevel(e)),
         };
 
-        if has_arti_dirmirror_schema_version {
+        if has_arti_dirserver_schema_version {
             let version = tx.query_one(
-                sql!("SELECT version FROM arti_dirmirror_schema_version WHERE rowid = 1"),
+                sql!("SELECT version FROM arti_dirserver_schema_version WHERE rowid = 1"),
                 params![],
                 |row| row.get::<_, String>(0),
             )?;
@@ -452,6 +547,106 @@ where
     Ok(res)
 }
 
+/// Inserts `data` into store while also compressing it with given encodings.
+///
+/// Returns the [`DocumentId`] of `data`.
+///
+/// This function inserts `data` into store and also compresses it into all
+/// given compression formats.
+///
+/// Duplicates get re-encoded and replaced in the database, including
+/// [`ContentEncoding::Identity`].
+pub(crate) fn store_insert<I: Iterator<Item = ContentEncoding>>(
+    tx: &Transaction,
+    data: &[u8],
+    encodings: I,
+) -> Result<DocumentId, DatabaseError> {
+    // The statement to insert some data into the store.
+    //
+    // Parameters:
+    // :docid - The docid.
+    // :content - The binary data.
+    let mut store_stmt = tx.prepare_cached(sql!(
+        "
+        INSERT OR REPLACE INTO store (docid, content)
+        VALUES
+        (:docid, :content)
+        "
+    ))?;
+
+    // The statement to insert a compressed document into the metatable.
+    //
+    // Parameters:
+    // :algorithm - The name of the encoding algorithm.
+    // :identity_docid - The docid of the plain-text document in the store.
+    // :compressed_docid - The docid of the encoded document in the store.
+    let mut compressed_stmt = tx.prepare_cached(sql!(
+        "
+        INSERT OR REPLACE INTO compressed_document (algorithm, identity_docid, compressed_docid)
+        VALUES
+        (:algorithm, :identity_docid, :compressed_docid)
+        "
+    ))?;
+
+    // Insert the plain document into the store.
+    let identity_docid = DocumentId::digest(data);
+    store_stmt.execute(named_params! {
+        ":docid": identity_docid,
+        ":content": data
+    })?;
+
+    // Compress it into all formats and insert it into store and compressed.
+    for encoding in encodings {
+        if encoding == ContentEncoding::Identity {
+            // Ignore identity because we inserted that above.
+            continue;
+        }
+
+        // We map a compression error to a bug because there is no good reason
+        // on why it should fail, given that we compress from memory data to
+        // memory data.  Probably because it uses the std::io::Writer interface
+        // which itself demands use of std::io::Result.
+        let compressed = compress(data, encoding).map_err(into_internal!("{encoding} failed?"))?;
+        let compressed_docid = DocumentId::digest(&compressed);
+        store_stmt.execute(named_params! {
+            ":docid": compressed_docid,
+            ":content": compressed,
+        })?;
+        compressed_stmt.execute(named_params! {
+            ":algorithm": encoding.to_string(),
+            ":identity_docid": identity_docid,
+            ":compressed_docid": compressed_docid,
+        })?;
+    }
+
+    Ok(identity_docid)
+}
+
+/// Compresses `data` into a specified [`ContentEncoding`].
+///
+/// Returns a [`Vec`] containing the encoded data.
+fn compress(data: &[u8], encoding: ContentEncoding) -> Result<Vec<u8>, std::io::Error> {
+    match encoding {
+        ContentEncoding::Identity => Ok(data.to_vec()),
+        ContentEncoding::Deflate => {
+            let mut w = DeflateEncoder::new(Vec::new(), Default::default());
+            w.write_all(data)?;
+            w.finish()
+        }
+        ContentEncoding::Gzip => {
+            let mut w = GzEncoder::new(Vec::new(), Default::default());
+            w.write_all(data)?;
+            w.finish()
+        }
+        ContentEncoding::XZstd => zstd::encode_all(data, Default::default()),
+        ContentEncoding::XTorLzma => {
+            let mut res = Vec::new();
+            lzma_rs::lzma_compress(&mut Cursor::new(data), &mut res)?;
+            Ok(res)
+        }
+    }
+}
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
@@ -468,11 +663,14 @@ mod test {
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use std::{
+        collections::HashSet,
+        io::Read,
         sync::{Arc, Once},
-        time::Duration,
     };
 
+    use flate2::read::{DeflateDecoder, GzDecoder};
     use rusqlite::Connection;
+    use strum::IntoEnumIterator;
     use tempfile::tempdir;
 
     use super::*;
@@ -488,7 +686,7 @@ mod test {
         // Check if the version was initialized properly.
         let version = conn
             .query_one(
-                "SELECT version FROM arti_dirmirror_schema_version WHERE rowid = 1",
+                "SELECT version FROM arti_dirserver_schema_version WHERE rowid = 1",
                 params![],
                 |row| row.get::<_, String>(0),
             )
@@ -497,7 +695,7 @@ mod test {
 
         // Set the version to something unknown.
         conn.execute(
-            "UPDATE arti_dirmirror_schema_version SET version = 42",
+            "UPDATE arti_dirserver_schema_version SET version = 42",
             params![],
         )
         .unwrap();
@@ -518,11 +716,11 @@ mod test {
 
         // Do a write transaction despite forbidden.
         super::read_tx(&pool, |tx| {
-            tx.execute_batch("DELETE FROM arti_dirmirror_schema_version")
+            tx.execute_batch("DELETE FROM arti_dirserver_schema_version")
                 .unwrap();
             let e = tx
                 .query_one(
-                    sql!("SELECT version FROM arti_dirmirror_schema_version"),
+                    sql!("SELECT version FROM arti_dirserver_schema_version"),
                     params![],
                     |row| row.get::<_, String>(0),
                 )
@@ -534,7 +732,7 @@ mod test {
         // Normal check.
         let version: String = super::read_tx(&pool, |tx| {
             tx.query_one(
-                sql!("SELECT version FROM arti_dirmirror_schema_version"),
+                sql!("SELECT version FROM arti_dirserver_schema_version"),
                 params![],
                 |row| row.get(0),
             )
@@ -553,7 +751,7 @@ mod test {
 
         // Do a write transaction.
         super::rw_tx(&pool, |tx| {
-            tx.execute_batch("DELETE FROM arti_dirmirror_schema_version")
+            tx.execute_batch("DELETE FROM arti_dirserver_schema_version")
                 .unwrap();
         })
         .unwrap();
@@ -562,7 +760,7 @@ mod test {
         super::read_tx(&pool, |tx| {
             let e = tx
                 .query_one(
-                    sql!("SELECT version FROM arti_dirmirror_schema_version"),
+                    sql!("SELECT version FROM arti_dirserver_schema_version"),
                     params![],
                     |row| row.get::<_, String>(0),
                 )
@@ -572,74 +770,238 @@ mod test {
         .unwrap();
     }
 
+    /// Tests whether our SQLite busy error handling works in normal situations.
+    ///
+    /// A normal situations means a situation where a lock is never held for
+    /// more than 1000ms.  In our case, we will work with two threads.
+    /// t1 will acquire an exclusive lock and inform t2 about it.  t2 waits
+    /// until t1 has acquired this lock and then immediately informs t1, that
+    /// it will now wait for a lock too.  Now, t1 will immediately terminate,
+    /// thereby releasing the lock and leading t2 to eventually acquire it.
     #[test]
     fn rw_tx_busy_timeout_working() {
         let db_dir = tempdir().unwrap();
         let db_path = db_dir.path().join("db");
-        let writer_acquired = Arc::new(Once::new());
         let pool = super::open(db_path).unwrap();
 
+        // t2 will wait on this before it starts doing stuff.
+        let t1_acquired_lock = Arc::new(Once::new());
+        // t1 will wait on this in order to terminate properly.
+        let t2_is_waiting = Arc::new(Once::new());
+
         let t1 = std::thread::spawn({
             let pool = pool.clone();
-            let writer_acquired = writer_acquired.clone();
+            let t1_acquired_lock = t1_acquired_lock.clone();
+            let t2_is_waiting = t2_is_waiting.clone();
             move || {
                 super::rw_tx(&pool, move |_tx| {
+                    // Inform t2 we have write lock.
+                    t1_acquired_lock.call_once(|| ());
                     println!("t1 acquired write lock");
-                    writer_acquired.call_once(|| ());
-                    std::thread::sleep(Duration::from_millis(500));
-                    println!("t2 released write lock");
+
+                    // Wait for t2 to start waiting.
+                    t2_is_waiting.wait();
                 })
+                .unwrap();
+                println!("t2 released write lock");
             }
         });
 
-        let t2 = std::thread::spawn(move || {
-            println!("t2 waits for t1 to acquire write lock");
-            writer_acquired.wait();
-            println!("t2 realized that t1 has acquired write lock");
-
-            super::rw_tx(&pool, move |_tx| {
-                println!("t2 acquired write lock");
-                std::thread::sleep(Duration::from_millis(500));
-                println!("t2 released write lock");
-            })
-        });
-
-        t1.join().unwrap().unwrap();
-        t2.join().unwrap().unwrap();
+        println!("t2 waits for t1 to acquire write lock");
+        t1_acquired_lock.wait();
+        t2_is_waiting.call_once(|| ());
+        super::rw_tx(&pool, |_| ()).unwrap();
+        println!("t2 acquired and released write lock");
+        t1.join().unwrap();
     }
 
+    /// Tests whether our SQLite busy error handlings fails as expected.
+    ///
+    /// We configure SQLite to fail after 1000ms.  This test works with two
+    /// threads.  t1 will acquire an exclusive lock on the database and will
+    /// inform t2 about it, which itself will wait until t1 has acquired the
+    /// lock.  t2 will then immediately try to also obtain an exclusive lock,
+    /// which should fail after about 1000ms.  After the failure, t2 informs
+    /// t1 that it has failed, causing t1 to terminate.
     #[test]
     fn rw_tx_busy_timeout_busy() {
         let db_dir = tempdir().unwrap();
         let db_path = db_dir.path().join("db");
-        let writer_acquired = Arc::new(Once::new());
         let pool = super::open(db_path).unwrap();
 
+        // t2 will wait on this before it starts doing stuff.
+        let t1_acquired_lock = Arc::new(Once::new());
+        // t1 will wait on this in order to terminate properly.
+        let t2_gave_up = Arc::new(Once::new());
+
         let t1 = std::thread::spawn({
             let pool = pool.clone();
-            let writer_acquired = writer_acquired.clone();
+            let t1_acquired_lock = t1_acquired_lock.clone();
+            let t2_gave_up = t2_gave_up.clone();
+
             move || {
                 super::rw_tx(&pool, move |_tx| {
+                    // Inform t2 we have the write lock.
+                    t1_acquired_lock.call_once(|| ());
                     println!("t1 acquired write lock");
-                    writer_acquired.call_once(|| ());
-                    std::thread::sleep(Duration::from_millis(1500));
-                    println!("t2 released write lock");
+                    // Wait for t2 to give up before we release (how mean from us).
+                    t2_gave_up.wait();
                 })
+                .unwrap();
+                println!("t1 released write lock");
             }
         });
 
-        let t2 = std::thread::spawn(move || {
-            println!("t2 waits for t1 to acquire write lock");
-            writer_acquired.wait();
-            println!("t2 realized that t1 has acquired write lock");
-
-            super::rw_tx(&pool, |_tx| unreachable!())
-        });
-
-        t1.join().unwrap().unwrap();
+        println!("t2 waits for t1 to acquire write lock");
+        t1_acquired_lock.wait();
+        let e = super::rw_tx(&pool, |_| ()).unwrap_err();
         assert_eq!(
-            t2.join().unwrap().unwrap_err().to_string(),
+            e.to_string(),
             "low-level rusqlite error: database is locked"
         );
+        println!("t2 gave up on acquiring write lock");
+        t2_gave_up.call_once(|| ());
+        t1.join().unwrap();
+    }
+
+    #[test]
+    fn store_insert() {
+        let db_dir = tempdir().unwrap();
+        let db_path = db_dir.path().join("db");
+
+        super::open(&db_path).unwrap();
+        let mut conn = Connection::open(&db_path).unwrap();
+        let tx = conn.transaction().unwrap();
+
+        let docid = super::store_insert(&tx, "foobar".as_bytes(), ContentEncoding::iter()).unwrap();
+        assert_eq!(
+            docid,
+            "C3AB8FF13720E8AD9047DD39466B3C8974E592C2FA383D4A3960714CAEF0C4F2"
+        );
+
+        let res = tx
+            .query_one(
+                sql!(
+                    "
+                    SELECT content
+                    FROM store
+                    WHERE docid = 'C3AB8FF13720E8AD9047DD39466B3C8974E592C2FA383D4A3960714CAEF0C4F2'
+                    "
+                ),
+                params![],
+                |row| row.get::<_, Vec<u8>>(0),
+            )
+            .unwrap();
+        assert_eq!(res, "foobar".as_bytes());
+
+        let mut stmt = tx.prepare_cached(sql!(
+            "
+            SELECT algorithm
+            FROM compressed_document
+            WHERE identity_docid = 'C3AB8FF13720E8AD9047DD39466B3C8974E592C2FA383D4A3960714CAEF0C4F2'
+            "
+        )).unwrap();
+
+        let algorithms = stmt
+            .query_map(params![], |row| row.get::<_, String>(0))
+            .unwrap();
+
+        let algorithms = algorithms.map(|x| x.unwrap()).collect::<HashSet<_>>();
+        assert_eq!(
+            algorithms,
+            HashSet::from([
+                "deflate".to_string(),
+                "gzip".to_string(),
+                "x-zstd".to_string(),
+                "x-tor-lzma".to_string()
+            ])
+        );
+
+        // Now insert the same thing a second time again and see whether the
+        // ON CONFLICT magic works.
+        let docid_second =
+            super::store_insert(&tx, "foobar".as_bytes(), ContentEncoding::iter()).unwrap();
+        assert_eq!(docid, docid_second);
+
+        // Remove a few compressed entries and get them again.
+        let n = tx
+            .execute(
+                sql!(
+                    "
+                    DELETE FROM
+                    compressed_document
+                    WHERE algorithm IN ('deflate', 'x-zstd')
+                    "
+                ),
+                params![],
+            )
+            .unwrap();
+        assert_eq!(n, 2);
+
+        let docid_third =
+            super::store_insert(&tx, "foobar".as_bytes(), ContentEncoding::iter()).unwrap();
+        assert_eq!(docid, docid_third);
+        let algorithms = stmt
+            .query_map(params![], |row| row.get::<_, String>(0))
+            .unwrap();
+        let algorithms = algorithms.map(|x| x.unwrap()).collect::<HashSet<_>>();
+        assert_eq!(
+            algorithms,
+            HashSet::from([
+                "deflate".to_string(),
+                "gzip".to_string(),
+                "x-zstd".to_string(),
+                "x-tor-lzma".to_string()
+            ])
+        );
+    }
+
+    #[test]
+    fn compress() {
+        /// Asserts that `res` contains `encoding`.
+        fn contains(encoding: ContentEncoding, res: &[(ContentEncoding, Vec<u8>)]) {
+            assert!(res.iter().any(|x| x.0 == encoding));
+        }
+
+        const INPUT: &[u8] = "foobar".as_bytes();
+
+        // Check whether everything was encoded.
+        let res = ContentEncoding::iter()
+            .map(|encoding| (encoding, super::compress(INPUT, encoding).unwrap()))
+            .collect::<Vec<_>>();
+        assert_eq!(res.len(), 5);
+        contains(ContentEncoding::Identity, &res);
+        contains(ContentEncoding::Deflate, &res);
+        contains(ContentEncoding::Gzip, &res);
+        contains(ContentEncoding::XTorLzma, &res);
+        contains(ContentEncoding::XZstd, &res);
+
+        // Check if we can decode it.
+        for (encoding, compressed) in res {
+            let mut decompressed = Vec::new();
+
+            match encoding {
+                ContentEncoding::Identity => decompressed = compressed,
+                ContentEncoding::Deflate => {
+                    DeflateDecoder::new(Cursor::new(compressed))
+                        .read_to_end(&mut decompressed)
+                        .unwrap();
+                }
+                ContentEncoding::Gzip => {
+                    GzDecoder::new(Cursor::new(compressed))
+                        .read_to_end(&mut decompressed)
+                        .unwrap();
+                }
+                ContentEncoding::XTorLzma => {
+                    lzma_rs::lzma_decompress(&mut Cursor::new(compressed), &mut decompressed)
+                        .unwrap();
+                }
+                ContentEncoding::XZstd => {
+                    decompressed = zstd::decode_all(Cursor::new(compressed)).unwrap();
+                }
+            }
+
+            assert_eq!(decompressed, INPUT);
+        }
     }
 }
