@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -42,6 +42,7 @@
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
 #![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 // TODO #1645 (either remove this, or decide to have it everywhere)
@@ -54,7 +55,6 @@
 //     filtered
 
 use futures::channel::mpsc;
-use futures::task::SpawnExt;
 use itertools::Either;
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
@@ -66,6 +66,7 @@ use tor_error::internal;
 use tor_linkspec::{OwnedChanTarget, OwnedCircTarget, RelayId, RelayIdSet};
 use tor_netdir::NetDirProvider;
 use tor_proto::ClockSkew;
+use tor_rtcompat::SpawnExt;
 use tor_units::BoundedInt32;
 use tracing::{debug, info, instrument, trace, warn};
 
@@ -113,7 +114,6 @@ pub use pending::{GuardMonitor, GuardStatus, GuardUsable};
 pub use skew::SkewEstimate;
 
 #[cfg(feature = "vanguards")]
-#[cfg_attr(docsrs, doc(cfg(feature = "vanguards")))]
 pub use vanguards::VanguardMgrError;
 
 use pending::{PendingRequest, RequestId};
@@ -325,6 +325,7 @@ impl<R: Runtime> GuardMgr<R> {
     ///
     /// It won't be able to hand out any guards until a [`NetDirProvider`] has
     /// been installed.
+    #[instrument(skip_all, level = "trace")]
     pub fn new<S>(
         runtime: R,
         state_mgr: S,
@@ -475,6 +476,7 @@ impl<R: Runtime> GuardMgr<R> {
     ///
     /// We only call this method if we _don't_ have the lock on the state
     /// files.  If we have the lock, we only want to save.
+    #[instrument(level = "trace", skip_all)]
     pub fn reload_persistent_state(&self) -> Result<(), GuardMgrError> {
         let mut inner = self.inner.lock().expect("Poisoned lock");
         if let Some(new_guards) = inner.storage.load()? {
@@ -486,6 +488,7 @@ impl<R: Runtime> GuardMgr<R> {
     /// Switch from having an unowned persistent state to having an owned one.
     ///
     /// Requires that we hold the lock on the state files.
+    #[instrument(level = "trace", skip_all)]
     pub fn upgrade_to_owned_persistent_state(&self) -> Result<(), GuardMgrError> {
         let mut inner = self.inner.lock().expect("Poisoned lock");
         debug_assert!(inner.storage.can_store());
@@ -540,6 +543,7 @@ impl<R: Runtime> GuardMgr<R> {
     }
 
     /// Replace the configuration in this `GuardMgr` with `config`.
+    #[instrument(level = "trace", skip_all)]
     pub fn reconfigure(
         &self,
         config: &impl GuardMgrConfig,
@@ -587,6 +591,7 @@ impl<R: Runtime> GuardMgr<R> {
     /// through the guard returned by this function, but you can't
     /// actually use it for traffic unless the [`GuardUsable`] future
     /// yields "true".
+    #[instrument(skip_all, level = "trace")]
     pub fn select_guard(
         &self,
         usage: GuardUsage,
@@ -899,6 +904,7 @@ impl GuardMgrInner {
 
     /// Replace our bridge configuration with the one from `new_config`.
     #[cfg(feature = "bridge-client")]
+    #[instrument(level = "trace", skip_all)]
     fn replace_bridge_config(
         &mut self,
         new_config: &impl GuardMgrConfig,
@@ -1078,6 +1084,7 @@ impl GuardMgrInner {
 
     /// Replace the active guard state with `new_state`, preserving
     /// non-persistent state for any guards that are retained.
+    #[instrument(level = "trace", skip_all)]
     fn replace_guards_with(
         &mut self,
         mut new_guards: GuardSets,
@@ -1158,6 +1165,7 @@ impl GuardMgrInner {
     }
 
     /// Replace the current GuardFilter with `filter`.
+    #[instrument(level = "trace", skip_all)]
     fn set_filter(&mut self, filter: GuardFilter, wallclock: SystemTime, now: Instant) {
         self.filter = filter;
         self.update(wallclock, now);
@@ -1443,6 +1451,7 @@ impl GuardMgrInner {
     }
 
     /// Try to select a guard, expanding the sample if the first attempt fails.
+    #[instrument(skip_all, level = "trace")]
     fn select_guard_with_expand(
         &mut self,
         usage: &GuardUsage,
@@ -1959,7 +1968,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
