@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -41,6 +41,8 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 mod connect;
@@ -55,19 +57,19 @@ mod state;
 use std::future::Future;
 use std::sync::{Arc, Mutex, MutexGuard};
 
-use futures::stream::BoxStream;
-use futures::task::SpawnExt as _;
 use futures::StreamExt as _;
+use futures::stream::BoxStream;
+use tor_rtcompat::SpawnExt as _;
 
 use educe::Educe;
-use tracing::debug;
+use tracing::{debug, instrument};
 
+use tor_circmgr::ClientOnionServiceDataTunnel;
 use tor_circmgr::hspool::HsCircPool;
 use tor_circmgr::isolation::StreamIsolation;
-use tor_error::{internal, Bug};
+use tor_error::{Bug, internal};
 use tor_hscrypto::pk::HsId;
 use tor_netdir::NetDir;
-use tor_proto::circuit::ClientCirc;
 use tor_rtcompat::Runtime;
 
 pub use err::FailedAttemptError;
@@ -76,7 +78,7 @@ pub use keys::{HsClientDescEncKeypairSpecifier, HsClientSecretKeys, HsClientSecr
 pub use relay_info::InvalidTarget;
 pub use state::HsClientConnectorConfig;
 
-use err::{rend_pt_identity_for_error, IntroPtIndex, RendPtIdentityForError};
+use err::{IntroPtIndex, RendPtIdentityForError, rend_pt_identity_for_error};
 use state::{Config, MockableConnectorData, Services};
 
 /// An object that negotiates connections with onion services
@@ -86,7 +88,7 @@ use state::{Config, MockableConnectorData, Services};
 /// and potentially different circuit isolation.
 ///
 /// The principal entrypoint is
-/// [`get_or_launch_connection()`](HsClientConnector::get_or_launch_connection).
+/// [`get_or_launch_tunnel()`](HsClientConnector::get_or_launch_tunnel).
 ///
 /// This object is handle-like: it is fairly cheap to clone,
 ///  and contains `Arc`s internally.
@@ -145,7 +147,7 @@ impl<R: Runtime> HsClientConnector<R, connect::Data> {
     /// a new circuit will be created.
     ///
     /// Once a circuit is returned, the caller can use it to open new streams to the
-    /// onion service. To do so, call [`ClientCirc::begin_stream`] on it.
+    /// onion service. To do so, call [`ClientOnionServiceDataTunnel::begin_stream`] on it.
     ///
     /// Each HS connection request must provide the appropriate
     /// service discovery keys to use -
@@ -156,13 +158,15 @@ impl<R: Runtime> HsClientConnector<R, connect::Data> {
     // Without this, it is possible for `Services::get_or_launch_connection`
     // to not return a `Send` future.
     // https://gitlab.torproject.org/tpo/core/arti/-/merge_requests/1034#note_2881718
-    pub fn get_or_launch_circuit<'r>(
+    #[instrument(skip_all, level = "trace")]
+    pub fn get_or_launch_tunnel<'r>(
         &'r self,
         netdir: &'r Arc<NetDir>,
         hs_id: HsId,
         secret_keys: HsClientSecretKeys,
         isolation: StreamIsolation,
-    ) -> impl Future<Output = Result<Arc<ClientCirc>, ConnError>> + Send + Sync + 'r {
+    ) -> impl Future<Output = Result<Arc<ClientOnionServiceDataTunnel>, ConnError>> + Send + Sync + 'r
+    {
         // As in tor-circmgr,  we take `StreamIsolation`, to ensure that callers in
         // arti-client pass us the final overall isolation,
         // including the per-TorClient isolation.
@@ -170,21 +174,6 @@ impl<R: Runtime> HsClientConnector<R, connect::Data> {
         let isolation = Box::new(isolation);
         Services::get_or_launch_connection(self, netdir, hs_id, isolation, secret_keys)
     }
-
-    /// A deprecated alias for `get_or_launch_circuit`.
-    ///
-    /// We renamed it to be
-    /// more clear about what exactly it is launching.
-    #[deprecated(since = "0.5.1", note = "Use get_or_launch_circuit instead.")]
-    pub fn get_or_launch_connection<'r>(
-        &'r self,
-        netdir: &'r Arc<NetDir>,
-        hs_id: HsId,
-        secret_keys: HsClientSecretKeys,
-        isolation: StreamIsolation,
-    ) -> impl Future<Output = Result<Arc<ClientCirc>, ConnError>> + Send + Sync + 'r {
-        self.get_or_launch_circuit(netdir, hs_id, secret_keys, isolation)
-    }
 }
 
 impl<R: Runtime, D: MockableConnectorData> HsClientConnector<R, D> {
@@ -254,7 +243,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
