@@ -8,7 +8,7 @@ use std::sync::{Arc, Mutex, MutexGuard};
 use std::time::{Duration, Instant};
 
 use futures::FutureExt as _;
-use futures::task::{SpawnError, SpawnExt as _};
+use futures::task::SpawnError;
 
 use async_trait::async_trait;
 use educe::Educe;
@@ -22,7 +22,7 @@ use tor_circmgr::isolation::Isolation;
 use tor_error::{Bug, ErrorReport as _, debug_report, error_report, internal};
 use tor_hscrypto::pk::HsId;
 use tor_netdir::NetDir;
-use tor_rtcompat::Runtime;
+use tor_rtcompat::{Runtime, SpawnExt as _};
 
 use crate::isol_map;
 use crate::{ConnError, HsClientConnector, HsClientSecretKeys};
@@ -745,7 +745,7 @@ pub(crate) mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -865,11 +865,10 @@ pub(crate) mod test {
     ) {
         let chanmgr = tor_chanmgr::ChanMgr::new(
             runtime.clone(),
-            &Default::default(),
+            Default::default(),
             tor_chanmgr::Dormancy::Dormant,
             &Default::default(),
             ToplevelAccount::new_noop(),
-            None,
         );
         let guardmgr = tor_guardmgr::GuardMgr::new(
             runtime.clone(),
