@@ -1,9 +1,15 @@
 //! Manipulate time periods (as used in the onion service system)
 
-use std::time::{Duration, SystemTime};
+use std::{
+    fmt::Display,
+    time::{Duration, SystemTime},
+};
 
+use humantime::format_rfc3339_seconds;
 use tor_units::IntegerMinutes;
 
+use serde::{Deserialize, Serialize};
+
 /// A period of time, as used in the onion service system.
 ///
 /// A `TimePeriod` is defined as a duration (in seconds), and the number of such
@@ -14,7 +20,7 @@ use tor_units::IntegerMinutes;
 ///
 /// These time periods are used to derive a different `BlindedOnionIdKey` during
 /// each period from each `OnionIdKey`.
-#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+#[derive(Deserialize, Serialize, Copy, Clone, Debug, Eq, PartialEq, Hash)]
 pub struct TimePeriod {
     /// Index of the time periods that have passed since the unix epoch.
     pub(crate) interval_num: u64,
@@ -41,6 +47,25 @@ impl PartialOrd for TimePeriod {
     }
 }
 
+impl Display for TimePeriod {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "#{} ", self.interval_num())?;
+        match self.range() {
+            Ok(r) => {
+                let mins = self.length().as_minutes();
+                write!(
+                    f,
+                    "{}..+{}:{:02}",
+                    format_rfc3339_seconds(r.start),
+                    mins / 60,
+                    mins % 60
+                )
+            }
+            Err(_) => write!(f, "overflow! {self:?}"),
+        }
+    }
+}
+
 impl TimePeriod {
     /// Construct a time period of a given `length` that contains `when`.
     ///
@@ -208,7 +233,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
