@@ -80,6 +80,7 @@ use tor_config::file_watcher::{
 use tor_config_path::{CfgPath, CfgPathResolver};
 use tor_dirclient::SourceInfo;
 use tor_netdir::{DirEvent, NetDir};
+use tracing::instrument;
 
 use crate::config::OnionServiceConfigPublisherView;
 use crate::config::restricted_discovery::{
@@ -357,6 +358,7 @@ impl<R: Runtime> Mockable for Real<R> {
         rand::rng()
     }
 
+    #[instrument(level = "trace", skip_all)]
     async fn get_or_launch_hs_dir<T>(
         &self,
         netdir: &NetDir,
@@ -1205,7 +1207,7 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
         let new_config = Arc::new(config.into());
         if self.replace_config_if_changed(Arc::clone(&new_config)) {
             self.update_file_watcher();
-            self.update_authorized_clients_if_changed().await?;
+            self.update_authorized_clients_if_changed();
 
             info!(nickname=%self.imm.nickname, "Config has changed, generating a new descriptor");
             self.mark_all_dirty();
@@ -1236,7 +1238,7 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
         // Update the file watcher, in case the change was triggered by a key_dir move.
         self.update_file_watcher();
 
-        if self.update_authorized_clients_if_changed().await? {
+        if self.update_authorized_clients_if_changed() {
             self.mark_all_dirty();
 
             // Schedule an upload, unless we're still waiting for IPTs.
@@ -1250,7 +1252,7 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
     /// Recreate the authorized_clients based on the current config.
     ///
     /// Returns `true` if the authorized clients have changed.
-    async fn update_authorized_clients_if_changed(&mut self) -> Result<bool, FatalError> {
+    fn update_authorized_clients_if_changed(&mut self) -> bool {
         let mut inner = self.inner.lock().expect("poisoned lock");
         let authorized_clients =
             Self::read_authorized_clients(&inner.config.restricted_discovery, &self.path_resolver);
@@ -1263,7 +1265,7 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
             *clients = authorized_clients;
         }
 
-        Ok(changed)
+        changed
     }
 
     /// Read the authorized `RestrictedDiscoveryKeys` from `config`.
@@ -2196,7 +2198,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
