@@ -8,13 +8,14 @@ use std::result::Result as StdResult;
 use std::str::FromStr as _;
 use std::sync::Arc;
 
-use crate::keystore::ctor::err::{CTorKeystoreError, MalformedClientKeyError};
 use crate::keystore::ctor::CTorKeystore;
-use crate::keystore::fs_utils::{checked_op, FilesystemAction, FilesystemError, RelKeyPath};
+use crate::keystore::ctor::err::{CTorKeystoreError, MalformedClientKeyError};
+use crate::keystore::fs_utils::{FilesystemAction, FilesystemError, RelKeyPath, checked_op};
 use crate::keystore::{EncodableItem, ErasedKey, KeySpecifier, Keystore};
+use crate::raw::{RawEntryId, RawKeystoreEntry};
 use crate::{
-    CTorPath, KeyPath, KeystoreEntryResult, KeystoreId, Result, UnrecognizedEntryError,
-    UnrecognizedEntryId,
+    CTorPath, KeyPath, KeystoreEntry, KeystoreEntryResult, KeystoreId, Result,
+    UnrecognizedEntryError,
 };
 
 use fs_mistrust::Mistrust;
@@ -42,9 +43,9 @@ use tracing::debug;
 ///
 /// This keystore implementation uses the [`CTorPath`] of the requested [`KeySpecifier`]
 /// and the [`KeystoreItemType`] to identify the appropriate restricted discovery keypair.
-/// If the requested `CTorPath` is not [`ClientHsDescEncKey`](CTorPath::ClientHsDescEncKey),
+/// If the requested `CTorPath` is not [`HsClientDescEncKeypair`](CTorPath::HsClientDescEncKeypair),
 /// the keystore will declare the key not found.
-/// If the requested `CTorPath` is [`ClientHsDescEncKey`](CTorPath::ClientHsDescEncKey),
+/// If the requested `CTorPath` is [`HsClientDescEncKeypair`](CTorPath::HsClientDescEncKeypair),
 /// but the `KeystoreItemType` is not [`X25519StaticKeypair`](KeyType::X25519StaticKeypair),
 /// an error is returned.
 pub struct CTorClientKeystore(CTorKeystore);
@@ -73,7 +74,7 @@ macro_rules! hsid_if_supported {
         };
 
         // This keystore only deals with service keys...
-        let CTorPath::ClientHsDescEncKey(hsid) = ctor_path else {
+        let CTorPath::HsClientDescEncKeypair { hs_id } = ctor_path else {
             return $ret;
         };
 
@@ -85,7 +86,7 @@ macro_rules! hsid_if_supported {
             .into());
         }
 
-        hsid
+        hs_id
     }};
 }
 
@@ -174,7 +175,7 @@ impl CTorClientKeystore {
             let content = match self.read_key(path) {
                 Ok(c) => c,
                 Err(e) => {
-                    debug_report!(e.clone(), "failed to read {}", path.display_lossy());
+                    debug_report!(&e, "failed to read {}", path.display_lossy());
                     return Some(Err(e));
                 }
             }?;
@@ -261,6 +262,11 @@ impl Keystore for CTorClientKeystore {
             .map(|k: curve25519::StaticKeypair| Box::new(k) as ErasedKey))
     }
 
+    #[cfg(feature = "onion-service-cli-extra")]
+    fn raw_entry_id(&self, raw_id: &str) -> Result<RawEntryId> {
+        Ok(RawEntryId::Path(PathBuf::from(raw_id.to_string())))
+    }
+
     fn insert(&self, _key: &dyn EncodableItem, _key_spec: &dyn KeySpecifier) -> Result<()> {
         Err(CTorKeystoreError::NotSupported { action: "insert" }.into())
     }
@@ -273,21 +279,37 @@ impl Keystore for CTorClientKeystore {
         Err(CTorKeystoreError::NotSupported { action: "remove" }.into())
     }
 
-    fn list(&self) -> Result<Vec<KeystoreEntryResult<(KeyPath, KeystoreItemType)>>> {
+    #[cfg(feature = "onion-service-cli-extra")]
+    fn remove_unchecked(&self, _entry_id: &RawEntryId) -> Result<()> {
+        Err(CTorKeystoreError::NotSupported {
+            action: "remove_unchecked",
+        }
+        .into())
+    }
+
+    fn list(&self) -> Result<Vec<KeystoreEntryResult<KeystoreEntry>>> {
         use CTorKeystoreError::*;
 
         let keys = self
             .list_keys()?
             .filter_map(|entry| match entry {
-                Ok((hsid, _)) => Some(Ok((
-                    CTorPath::ClientHsDescEncKey(hsid).into(),
-                    KeyType::X25519StaticKeypair.into(),
-                ))),
+                Ok((hs_id, _)) => {
+                    let key_path: KeyPath = CTorPath::HsClientDescEncKeypair { hs_id }.into();
+                    let key_type: KeystoreItemType = KeyType::X25519StaticKeypair.into();
+                    let raw_id = RawEntryId::Path(key_path.ctor()?.to_string().into());
+                    Some(Ok(KeystoreEntry::new(
+                        key_path,
+                        key_type,
+                        self.id(),
+                        raw_id,
+                    )))
+                }
                 Err(e) => match e {
-                    MalformedKey { ref path, err: _ } => Some(Err(UnrecognizedEntryError::new(
-                        UnrecognizedEntryId::Path(path.clone()),
-                        Arc::new(e),
-                    ))),
+                    MalformedKey { ref path, err: _ } => {
+                        let raw_id = RawEntryId::Path(path.clone());
+                        let entry = RawKeystoreEntry::new(raw_id, self.id().clone()).into();
+                        Some(Err(UnrecognizedEntryError::new(entry, Arc::new(e))))
+                    }
                     // `InvalidKeystoreItemType` variant is filtered out because it can't
                     // be returned by [`CTorClientKeystore::list_keys`].
                     InvalidKeystoreItemType { .. } => None,
@@ -315,15 +337,15 @@ mod tests {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
     use std::fs;
-    use tempfile::{tempdir, TempDir};
+    use tempfile::{TempDir, tempdir};
 
-    use crate::test_utils::{assert_found, DummyKey, TestCTorSpecifier};
+    use crate::test_utils::{DummyKey, TestCTorSpecifier, assert_found};
 
     #[cfg(unix)]
     use std::os::unix::fs::PermissionsExt;
@@ -377,7 +399,9 @@ mod tests {
     #[test]
     fn get() {
         let (keystore, _keystore_dir) = init_keystore("foo");
-        let path = CTorPath::ClientHsDescEncKey(HsId::from_str(HSID).unwrap());
+        let path = CTorPath::HsClientDescEncKeypair {
+            hs_id: HsId::from_str(HSID).unwrap(),
+        };
 
         // Not found!
         assert_found!(
@@ -391,7 +415,9 @@ mod tests {
             // Extract the HsId associated with this key.
             let onion = hsid.split(":").next().unwrap();
             let hsid = HsId::from_str(&format!("{onion}.onion")).unwrap();
-            let path = CTorPath::ClientHsDescEncKey(hsid.clone());
+            let path = CTorPath::HsClientDescEncKeypair {
+                hs_id: hsid.clone(),
+            };
 
             // Found!
             assert_found!(
@@ -411,15 +437,16 @@ mod tests {
 
         assert_eq!(keys.len(), 2);
         assert!(keys.iter().all(|entry| {
-            let (_, key_type) = entry.as_ref().unwrap();
-            *key_type == KeyType::X25519StaticKeypair.into()
+            entry.as_ref().unwrap().key_type() == &KeyType::X25519StaticKeypair.into()
         }));
     }
 
     #[test]
     fn unsupported_operation() {
         let (keystore, _keystore_dir) = init_keystore("foo");
-        let path = CTorPath::ClientHsDescEncKey(HsId::from_str(HSID).unwrap());
+        let path = CTorPath::HsClientDescEncKeypair {
+            hs_id: HsId::from_str(HSID).unwrap(),
+        };
 
         let err = keystore
             .remove(
@@ -440,7 +467,9 @@ mod tests {
     #[test]
     fn wrong_keytype() {
         let (keystore, _keystore_dir) = init_keystore("foo");
-        let path = CTorPath::ClientHsDescEncKey(HsId::from_str(HSID).unwrap());
+        let path = CTorPath::HsClientDescEncKeypair {
+            hs_id: HsId::from_str(HSID).unwrap(),
+        };
 
         let err = keystore
             .get(
