@@ -1,40 +1,10 @@
-//! `HasMemoryCost` and typed memory cost tracking
+//! `HasMemoryCost`-related traits, and typed memory cost tracking
 
 #![forbid(unsafe_code)] // if you remove this, enable (or write) miri tests (git grep miri)
 
 use crate::internal_prelude::*;
 
-/// Types whose memory usage is known (and stable)
-///
-/// ### Important guarantees
-///
-/// Implementors of this trait must uphold the guarantees in the API of
-/// [`memory_cost`](HasMemoryCost::memory_cost).
-///
-/// If these guarantees are violated, memory tracking may go wrong,
-/// with seriously bad implications for the whole program,
-/// including possible complete denial of service.
-///
-/// (Nevertheless, memory safety will not be compromised,
-/// so trait this is not `unsafe`.)
-pub trait HasMemoryCost {
-    /// Returns the memory cost of `self`, in bytes
-    ///
-    /// ### Return value must be stable
-    ///
-    /// It is vital that the return value does not change, for any particular `self`,
-    /// unless `self` is mutated through `&mut self` or similar.
-    /// Otherwise, memory accounting may go awry.
-    ///
-    /// If `self` has interior mutability. the changing internal state
-    /// must not change the memory cost.
-    ///
-    /// ### Panics - forbidden
-    ///
-    /// This method must not panic.
-    /// Otherwise, memory accounting may go awry.
-    fn memory_cost(&self, _: EnabledToken) -> usize;
-}
+pub use tor_memquota_cost::memory_cost::HasMemoryCost;
 
 /// A [`Participation`] for use only for tracking the memory use of objects of type `T`
 ///
@@ -238,7 +208,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
