@@ -59,13 +59,18 @@ use {
     crate::parse2::{self, ArgumentStream}, //
 };
 
+#[cfg(feature = "parse2")]
+pub use {
+    parse2_impls::ProtoStatusesNetdocParseAccumulator, //
+};
+
 use crate::doc::authcert::{AuthCert, AuthCertKeyIds};
 use crate::parse::keyword::Keyword;
 use crate::parse::parser::{Section, SectionRules, SectionRulesBuilder};
 use crate::parse::tokenize::{Item, ItemResult, NetDocReader};
 use crate::types::misc::*;
 use crate::util::PeekableIterator;
-use crate::{Error, NetdocErrorKind as EK, NormalItemArgument, Pos, Result};
+use crate::{Error, KeywordEncodable, NetdocErrorKind as EK, NormalItemArgument, Pos, Result};
 use std::collections::{BTreeSet, HashMap, HashSet};
 use std::fmt::{self, Display};
 use std::result::Result as StdResult;
@@ -75,7 +80,6 @@ use std::{net, result, time};
 use tor_error::{HasKind, internal};
 use tor_protover::Protocols;
 
-use bitflags::bitflags;
 use derive_deftly::{Deftly, define_derive_deftly};
 use digest::Digest;
 use std::sync::LazyLock;
@@ -98,8 +102,6 @@ ns_export_each_variety! {
     ty: RouterStatus, Preamble;
 }
 
-use void::ResultVoidExt as _;
-
 #[deprecated]
 #[cfg(feature = "ns_consensus")]
 pub use PlainConsensus as NsConsensus;
@@ -114,7 +116,7 @@ pub use UncheckedPlainConsensus as UncheckedNsConsensus;
 pub use UnvalidatedPlainConsensus as UnvalidatedNsConsensus;
 
 #[cfg(feature = "ns-vote")]
-pub use rs::RouterStatusMdDigestsVote;
+pub use rs::{RouterStatusMdDigestsVote, SoftwareVersion};
 
 /// `publiscation` field in routerstatus entry intro item other than in votes
 ///
@@ -428,7 +430,7 @@ impl HasKind for ProtocolSupportError {
 /// Represents the collection of four items: `{recommended,required}-{client,relay}-protocols`.
 ///
 /// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:required-relay-protocols>
-#[derive(Clone, Debug, PartialEq, serde::Serialize, serde::Deserialize)]
+#[derive(Clone, Debug, Default, PartialEq, serde::Serialize, serde::Deserialize)]
 pub struct ProtoStatuses {
     /// Lists of recommended and required subprotocol versions for clients
     client: ProtoStatus,
@@ -450,9 +452,13 @@ impl ProtoStatuses {
 
 /// A recognized 'flavor' of consensus document.
 ///
+/// The enum is exhaustive because the addition/removal of a consensus flavor
+/// should indeed be a breaking change, as it would inevitable require
+/// interfacing code to think about the handling of it.
+///
 /// <https://spec.torproject.org/dir-spec/computing-consensus.html#flavors>
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash, Ord, PartialOrd)]
-#[non_exhaustive]
+#[allow(clippy::exhaustive_enums)]
 pub enum ConsensusFlavor {
     /// A "microdesc"-flavored consensus.  This is the one that
     /// clients and relays use today.
@@ -524,9 +530,10 @@ pub struct SharedRandVal([u8; 32]);
 
 /// A shared-random value produced by the directory authorities,
 /// along with meta-information about that value.
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Deftly)]
 #[non_exhaustive]
-#[cfg_attr(feature = "parse2", derive(Deftly), derive_deftly(ItemValueParseable))]
+#[cfg_attr(feature = "parse2", derive_deftly(ItemValueParseable))]
+#[cfg_attr(feature = "encode", derive_deftly(ItemValueEncodable))]
 pub struct SharedRandStatus {
     /// How many authorities revealed shares that contributed to this value.
     pub n_reveals: u8,
@@ -566,69 +573,6 @@ pub struct DirSource {
     pub or_port: u16,
 }
 
-bitflags! {
-    /// A set of recognized directory flags on a single relay.
-    ///
-    /// These flags come from a consensus directory document, and are
-    /// used to describe what the authorities believe about the relay.
-    /// If the document contained any flags that we _didn't_ recognize,
-    /// they are not listed in this type.
-    ///
-    /// The bit values used to represent the flags have no meaning;
-    /// they may change between releases of this crate.  Relying on their
-    /// values may void your semver guarantees.
-    #[derive(Clone, Copy, Debug)]
-    pub struct RelayFlags: u16 {
-        /// Is this a directory authority?
-        const AUTHORITY = (1<<0);
-        /// Is this relay marked as a bad exit?
-        ///
-        /// Bad exits can be used as intermediate relays, but not to
-        /// deliver traffic.
-        const BAD_EXIT = (1<<1);
-        /// Is this relay marked as an exit for weighting purposes?
-        const EXIT = (1<<2);
-        /// Is this relay considered "fast" above a certain threshold?
-        const FAST = (1<<3);
-        /// Is this relay suitable for use as a guard relay?
-        ///
-        /// Clients choose their their initial relays from among the set
-        /// of Guard relays.
-        const GUARD = (1<<4);
-        /// Does this relay participate on the onion service directory
-        /// ring?
-        const HSDIR = (1<<5);
-        /// Set if this relay is considered "middle only", not suitable to run
-        /// as an exit or guard relay.
-        ///
-        /// Note that this flag is only used by authorities as part of
-        /// the voting process; clients do not and should not act
-        /// based on whether it is set.
-        const MIDDLE_ONLY = (1<<6);
-        /// If set, there is no consensus for the ed25519 key for this relay.
-        const NO_ED_CONSENSUS = (1<<7);
-        /// Is this relay considered "stable" enough for long-lived circuits?
-        const STABLE = (1<<8);
-        /// Set if the authorities are requesting a fresh descriptor for
-        /// this relay.
-        const STALE_DESC = (1<<9);
-        /// Set if this relay is currently running.
-        ///
-        /// This flag can appear in votes, but in consensuses, every relay
-        /// is assumed to be running.
-        const RUNNING = (1<<10);
-        /// Set if this relay is considered "valid" -- allowed to be on
-        /// the network.
-        ///
-        /// This flag can appear in votes, but in consensuses, every relay
-        /// is assumed to be valid.
-        const VALID = (1<<11);
-        /// Set if this relay supports a currently recognized version of the
-        /// directory protocol.
-        const V2DIR = (1<<12);
-    }
-}
-
 /// Recognized weight fields on a single relay in a consensus
 #[non_exhaustive]
 #[derive(Debug, Clone, Copy)]
@@ -1078,88 +1022,6 @@ impl ConsensusVoterInfo {
     }
 }
 
-impl std::str::FromStr for RelayFlags {
-    type Err = void::Void;
-    fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
-        Ok(match s {
-            "Authority" => RelayFlags::AUTHORITY,
-            "BadExit" => RelayFlags::BAD_EXIT,
-            "Exit" => RelayFlags::EXIT,
-            "Fast" => RelayFlags::FAST,
-            "Guard" => RelayFlags::GUARD,
-            "HSDir" => RelayFlags::HSDIR,
-            "MiddleOnly" => RelayFlags::MIDDLE_ONLY,
-            "NoEdConsensus" => RelayFlags::NO_ED_CONSENSUS,
-            "Stable" => RelayFlags::STABLE,
-            "StaleDesc" => RelayFlags::STALE_DESC,
-            "Running" => RelayFlags::RUNNING,
-            "Valid" => RelayFlags::VALID,
-            "V2Dir" => RelayFlags::V2DIR,
-            _ => RelayFlags::empty(),
-        })
-    }
-}
-
-/// Parsing helper for a relay flags line (eg `s` item in a routerdesc)
-struct RelayFlagsParser<'s> {
-    /// Flags so far, including the implied ones
-    flags: RelayFlags,
-
-    /// The previous argument, if any
-    ///
-    /// Used only for checking that the arguments are sorted, as per the spec.
-    prev: Option<&'s str>,
-}
-
-impl<'s> RelayFlagsParser<'s> {
-    /// Start parsing relay flags
-    fn new() -> Self {
-        // These flags are implicit.
-        RelayFlagsParser {
-            flags: RelayFlags::RUNNING | RelayFlags::VALID,
-            prev: None,
-        }
-    }
-    /// Parse the next relay flag argument
-    fn add(&mut self, arg: &'s str) -> StdResult<(), &'static str> {
-        if let Some(prev) = self.prev {
-            if prev >= arg {
-                // Arguments out of order.
-                return Err("Flags out of order");
-            }
-        }
-        let fl = arg.parse().void_unwrap();
-        self.flags |= fl;
-        self.prev = Some(arg);
-        Ok(())
-    }
-    /// Finish parsing relay flags
-    fn finish(self) -> RelayFlags {
-        self.flags
-    }
-}
-
-impl RelayFlags {
-    /// Parse a relay-flags entry from an "s" line.
-    fn from_item(item: &Item<'_, NetstatusKwd>) -> Result<RelayFlags> {
-        if item.kwd() != NetstatusKwd::RS_S {
-            return Err(
-                Error::from(internal!("Wrong keyword {:?} for S line", item.kwd()))
-                    .at_pos(item.pos()),
-            );
-        }
-        let mut flags = RelayFlagsParser::new();
-
-        for s in item.args() {
-            flags
-                .add(s)
-                .map_err(|msg| EK::BadArgument.at_pos(item.pos()).with_msg(msg))?;
-        }
-
-        Ok(flags.finish())
-    }
-}
-
 impl Default for RelayWeight {
     fn default() -> RelayWeight {
         RelayWeight::Unmeasured(0)
@@ -1210,8 +1072,78 @@ mod parse2_impls {
     use parse2::ArgumentError as AE;
     use parse2::ErrorProblem as EP;
     use parse2::{ArgumentStream, ItemArgumentParseable, ItemValueParseable};
+    use parse2::{KeywordRef, NetdocParseableFields, UnparsedItem};
+    use paste::paste;
     use std::result::Result;
 
+    /// Implements `NetdocParseableFields` for `ProtoStatuses`
+    ///
+    /// We have this macro so that it's impossible to write things like
+    /// ```text
+    ///      ProtoStatuses {
+    ///          client: ProtoStatus {
+    ///              recommended: something something recommended_relay_versions something,
+    /// ```
+    ///
+    /// (The structure of `ProtoStatuses` means the normal parse2 derive won't work for it.
+    /// Note the bug above: the recommended *relay* version info is put in the *client* field.
+    /// Preventing this bug must involve: avoiding writing twice the field name elements,
+    /// such as `relay` and `client`, during this kind of construction/conversion.)
+    macro_rules! impl_proto_statuses { { $( $rr:ident $cr:ident; )* } => { paste! {
+        #[derive(Deftly)]
+        #[derive_deftly(NetdocParseableFields)]
+        // Only ProtoStatusesParseNetdocParseAccumulator is exposed.
+        #[allow(unreachable_pub)]
+        pub struct ProtoStatusesParseHelper {
+            $(
+                #[deftly(netdoc(default))]
+                [<$rr _ $cr _protocols>]: Protocols,
+            )*
+        }
+
+        /// Partially parsed `ProtoStatuses`
+        pub use ProtoStatusesParseHelperNetdocParseAccumulator
+            as ProtoStatusesNetdocParseAccumulator;
+
+        impl NetdocParseableFields for ProtoStatuses {
+            type Accumulator = ProtoStatusesNetdocParseAccumulator;
+            fn is_item_keyword(kw: KeywordRef<'_>) -> bool {
+                ProtoStatusesParseHelper::is_item_keyword(kw)
+            }
+            fn accumulate_item(
+                acc: &mut Self::Accumulator,
+                item: UnparsedItem<'_>,
+            ) -> Result<(), EP> {
+                ProtoStatusesParseHelper::accumulate_item(acc, item)
+            }
+            fn finish(acc: Self::Accumulator) -> Result<Self, EP> {
+                let parse = ProtoStatusesParseHelper::finish(acc)?;
+                let mut out = ProtoStatuses::default();
+                $(
+                    out.$cr.$rr = parse.[< $rr _ $cr _protocols >];
+                )*
+                Ok(out)
+            }
+        }
+    } } }
+
+    impl_proto_statuses! {
+        required client;
+        required relay;
+        recommended client;
+        recommended relay;
+    }
+
+    impl ItemValueParseable for NetParams<i32> {
+        fn from_unparsed(item: parse2::UnparsedItem<'_>) -> Result<Self, EP> {
+            item.check_no_object()?;
+            item.args_copy()
+                .into_remaining()
+                .parse()
+                .map_err(item.invalid_argument_handler("parameters"))
+        }
+    }
+
     impl ItemValueParseable for RelayWeight {
         fn from_unparsed(item: parse2::UnparsedItem<'_>) -> Result<Self, EP> {
             item.check_no_object()?;
@@ -1223,20 +1155,7 @@ mod parse2_impls {
         }
     }
 
-    impl ItemValueParseable for RelayFlags {
-        fn from_unparsed(item: parse2::UnparsedItem<'_>) -> Result<Self, EP> {
-            item.check_no_object()?;
-            let mut flags = RelayFlagsParser::new();
-            for arg in item.args_copy() {
-                flags
-                    .add(arg)
-                    .map_err(item.invalid_argument_handler("flags"))?;
-            }
-            Ok(flags.finish())
-        }
-    }
-
-    impl ItemValueParseable for rs::Version {
+    impl ItemValueParseable for rs::SoftwareVersion {
         fn from_unparsed(mut item: parse2::UnparsedItem<'_>) -> Result<Self, EP> {
             item.check_no_object()?;
             item.args_mut()
@@ -1324,7 +1243,7 @@ impl Signature {
     /// Return true if this signature has the identity key and signing key
     /// that match a given cert.
     fn matches_cert(&self, cert: &AuthCert) -> bool {
-        cert.key_ids() == &self.key_ids
+        cert.key_ids() == self.key_ids
     }
 
     /// If possible, find the right certificate for checking this signature
@@ -1400,7 +1319,7 @@ impl SignatureGroup {
     /// authorities.  This API requires that `n_authorities` is the number of
     /// authorities we believe in, and that every cert in `certs` belongs
     /// to a real authority.
-    fn validate(&self, n_authorities: u16, certs: &[AuthCert]) -> bool {
+    fn validate(&self, n_authorities: usize, certs: &[AuthCert]) -> bool {
         // A set of the authorities (by identity) who have have signed
         // this document.  We use a set here in case `certs` has more
         // than one certificate for a single authority.
@@ -1435,7 +1354,7 @@ impl SignatureGroup {
             }
         }
 
-        ok.len() > (n_authorities / 2) as usize
+        ok.len() > (n_authorities / 2)
     }
 }
 
@@ -1450,7 +1369,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -1458,7 +1377,7 @@ mod test {
     use hex_literal::hex;
     #[cfg(all(feature = "ns-vote", feature = "parse2"))]
     use {
-        crate::parse2::{NetdocSigned as _, parse_netdoc},
+        crate::parse2::{NetdocSigned as _, ParseInput, parse_netdoc},
         std::fs,
     };
 
@@ -1490,7 +1409,7 @@ mod test {
             let cert = cert?.check_signature()?.dangerously_assume_timely();
             certs.push(cert);
         }
-        let auth_ids: Vec<_> = certs.iter().map(|c| &c.key_ids().id_fingerprint).collect();
+        let auth_ids: Vec<_> = certs.iter().map(|c| c.id_fingerprint()).collect();
 
         assert_eq!(certs.len(), 3);
 
@@ -1556,7 +1475,7 @@ mod test {
             let cert = cert?.check_signature()?.dangerously_assume_timely();
             certs.push(cert);
         }
-        let auth_ids: Vec<_> = certs.iter().map(|c| &c.key_ids().id_fingerprint).collect();
+        let auth_ids: Vec<_> = certs.iter().map(|c| c.id_fingerprint()).collect();
         assert_eq!(certs.len(), 4);
 
         let (_, _, consensus) = PlainConsensus::parse(PLAIN_CONSENSUS)?;
@@ -1582,7 +1501,8 @@ mod test {
         // TODO replace the poc struct here when we have parsing of proper whole votes
         use crate::parse2::poc::netstatus::NetworkStatusSignedVote;
 
-        let doc: NetworkStatusSignedVote = parse_netdoc(&text, file)?;
+        let input = ParseInput::new(&text, file);
+        let doc: NetworkStatusSignedVote = parse_netdoc(&input)?;
 
         println!("{doc:?}");
         println!("{:#?}", doc.inspect_unverified().0.r[0]);
