@@ -63,12 +63,12 @@ define_derive_deftly_module! {
     // Provide `$<selector_ $fname>` for every (suitable) field.
     ${define ITEM_SET_SELECTORS {
         $(
-          ${when not(any(F_FLATTEN))}
+          ${when not(any(F_FLATTEN, F_SKIP))}
 
           // See `mod multiplicity`.
         ${if not(all(F_INTRO, fmeta(netdoc(with)))) {
           // If the intro it has `with`, we don't check its trait impl, and this ends up unused
-          let $<selector_ $fname> = MultiplicitySelector::<$F_EFFECTIVE_TYPE>::default();
+          let $<selector_ $fname> = $F_SELECTOR_VALUE;
         }}
         )
     }}
@@ -77,11 +77,13 @@ define_derive_deftly_module! {
     // because the identifier `selector_` has different macro_rules hygiene here vs there!
     // TODO derive-deftly#130
     ${define F_SELECTOR $<selector_ $fname>}
-
+    // The selector value for this field.  Used where we don't want to bind a selector
+    // for every field with $ITEM_SET_SELECTORS (and within $ITEM_SET_SELECTORS).
+    ${define F_SELECTOR_VALUE {( MultiplicitySelector::<$F_EFFECTIVE_TYPE>::default() )}}
     // Check that every field type implements the necessary trait.
     ${define CHECK_FIELD_TYPES_PARSEABLE {
         $(
-          ${when not(any(F_FLATTEN))}
+          ${when not(any(F_FLATTEN, F_SKIP))}
 
           // Expands to `selector_FIELD.check_SOMETHING();`
           //
@@ -149,7 +151,7 @@ define_derive_deftly_module! {
     // so if `f_INTRO` is not trivially false, must be expanded within a field loop.
     ${define NONSTRUCTURAL_ACCUMULATE_ELSE {
         ${for fields {
-          ${when not(any(F_FLATTEN, F_SUBDOC))}
+          ${when not(any(F_FLATTEN, F_SUBDOC, F_SKIP))}
 
           if kw == $F_KEYWORD {
             ${select1
@@ -207,10 +209,14 @@ define_derive_deftly_module! {
               F_SUBDOC {
                   let $fpatname = $F_SELECTOR.finish_subdoc($fpatname)?;
               }
+              F_SKIP {
+                  #[allow(non_snake_case)]
+                  let $fpatname = Default::default();
+              }
             }
         }}
         $(
-            ${when not(F_INTRO)}
+            ${when not(any(F_INTRO, F_SKIP))}
             // These conditions are mirrored in NetdocSomeItemsEncodableCommon,
             // which is supposed to recognise netdoc(default) precisely when we do.
           ${if fmeta(netdoc(default)) {
@@ -336,6 +342,12 @@ define_derive_deftly! {
     ///
     ///   The field type must implement [`NetdocParseableFields`].
     ///
+    /// * **`#[deftly(netdoc(skip))]`**:
+    ///
+    ///   This field doesn't really appear in the network document.
+    ///   It won't be recognised during parsing.
+    ///   Instead, `Default::default()` will be used for the field value.
+    ///
     /// * **`#[deftly(netdoc(subdoc))]`**:
     ///
     ///   This field is a sub-document.
@@ -425,6 +437,24 @@ define_derive_deftly! {
             }}
         }
 
+        fn is_structural_keyword(kw: $P::KeywordRef<'_>) -> Option<$P::IsStructural> {
+            #[allow(unused_imports)] // not used if there are no subdocs
+            use $P::*;
+
+            if Self::is_intro_item_keyword(kw) {
+                return Some(IsStructural)
+            }
+
+            ${for fields {
+                ${when F_SUBDOC}
+                if let y @ Some(_) = $F_SELECTOR_VALUE.is_structural_keyword(kw) {
+                    return y;
+                }
+            }}
+
+            None
+        }
+
         //##### main parsing function #####
 
         #[allow(clippy::redundant_locals)] // let item = $THIS_ITEM, which might be item
@@ -512,6 +542,8 @@ define_derive_deftly! {
             }
             let $fpatname: $ftype = $ITEM_VALUE_FROM_UNPARSED;
 
+          } F_SKIP {
+
           } else {
 
             let mut $fpatname = $F_ACCUMULATE_TYPE::default();
@@ -746,6 +778,11 @@ define_derive_deftly! {
             $ttype::is_intro_item_keyword(kw)
         }
 
+        fn is_structural_keyword(kw: $P::KeywordRef<'_>) -> Option<$P::IsStructural> {
+            $ttype::is_structural_keyword(kw)
+                .or_else(|| $SIGS_TYPE::is_structural_keyword(kw))
+        }
+
         fn from_items<'s>(
             input: &mut $P::ItemStream<'s>,
             outer_stop: $P::stop_at!(),
