@@ -17,7 +17,7 @@ pub mod version;
 pub use misc::{Nickname, NotPresent, Unknown};
 
 pub use misc::B64;
-pub use misc::{Base64Fingerprint, Fingerprint, Ignored};
+pub use misc::{Base64Fingerprint, Fingerprint, Ignored, IgnoredItemOrObjectValue};
 pub use misc::{DigestName, IdentifiedDigest};
 pub use misc::{Iso8601TimeNoSp, Iso8601TimeSp};
 
@@ -25,3 +25,4 @@ use crate::NormalItemArgument;
 
 impl NormalItemArgument for std::net::Ipv4Addr {}
 impl NormalItemArgument for std::net::SocketAddr {}
+impl NormalItemArgument for std::net::SocketAddrV4 {}
