@@ -65,14 +65,16 @@ pub(crate) use crate::channel::reactor::Reactor;
 use crate::channel::reactor::{BoxedChannelSink, BoxedChannelStream};
 pub use crate::channel::unique_id::UniqId;
 use crate::client::channel::ClientChannelBuilder;
+use crate::client::circuit::PendingClientTunnel;
 use crate::client::circuit::padding::{PaddingController, QueuedCellPaddingInfo};
-use crate::client::circuit::{PendingClientTunnel, TimeoutEstimator};
 use crate::memquota::{ChannelAccount, CircuitAccount, SpecificAccount as _};
 use crate::util::err::ChannelClosed;
 use crate::util::oneshot_broadcast;
+use crate::util::timeout::TimeoutEstimator;
 use crate::util::ts::AtomicOptTimestamp;
 use crate::{ClockSkew, client};
 use crate::{Error, Result};
+use async_trait::async_trait;
 use cfg_if::cfg_if;
 use reactor::BoxedChannelStreamOps;
 use safelog::sensitive as sv;
@@ -126,6 +128,78 @@ use crate::channel::unique_id::CircUniqIdContext;
 
 use kist::KistParams;
 
+/// Module to seal traits in order to not allow implementation outside this crate.
+///
+/// This is used by VerifiableChannel and FinalizableChannel.
+pub(crate) mod seal {
+    /// Hidden trait to seal other traits.
+    pub trait Sealed {}
+}
+
+/// An verifiable channel trait which is public outside this crate.
+///
+/// An open channel that has not been verified as in the certificates and keys have not been
+/// validated yet. Upon successful validation, a boxed [`FinalizableChannel`] is returned.
+pub trait VerifiableChannel<T, S>: seal::Sealed
+where
+    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
+    S: CoarseTimeProvider + SleepProvider,
+{
+    /// Validate the certificates and keys in the relay's handshake only if the peer chooses to
+    /// authenticate (the peer is a responder or sends an AUTHENTICATE cell)
+    ///
+    /// 'peer' is the peer that we want to make sure we're connecting to.
+    ///
+    /// 'peer_cert' is the x.509 certificate that the peer presented during
+    /// its TLS handshake (ServerHello).
+    ///
+    /// 'now' is the time at which to check that certificates are
+    /// valid.  `None` means to use the current time. It can be used
+    /// for testing to override the current view of the time.
+    ///
+    /// This is a separate function because it's likely to be somewhat
+    /// CPU-intensive.
+    fn check(
+        self: Box<Self>,
+        peer: &OwnedChanTarget,
+        peer_cert: &[u8],
+        now: Option<std::time::SystemTime>,
+    ) -> Result<Box<dyn FinalizableChannel<T, S>>>;
+
+    /// Return the reported clock skew from this handshake.
+    ///
+    /// Note that the skew reported by this function might not be "true": the
+    /// relay might have its clock set wrong, or it might be lying to us.
+    ///
+    /// The clock skew reported here is not yet authenticated; if you need to
+    /// make sure that the skew is authenticated, use
+    /// [`Channel::clock_skew`](crate::channel::Channel::clock_skew) instead.
+    fn clock_skew(&self) -> ClockSkew;
+
+    /// Return the link protocol version of this channel.
+    #[cfg(test)]
+    fn link_protocol(&self) -> u16;
+}
+
+/// A finalizable channel trait used to yield a Channel.
+///
+/// It represents a channel that has been verified but the handshake has not been finalized as in
+/// the final cells have not been sent. Once finalized, a [`Channel`] and its Reactor are
+/// returned.
+#[async_trait]
+pub trait FinalizableChannel<T, S>: seal::Sealed
+where
+    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
+    S: CoarseTimeProvider + SleepProvider,
+{
+    /// Send a 'Netinfo' message to the relay to finish the handshake, and create an open channel
+    /// and reactor.
+    ///
+    /// The channel is used to send cells, and to create outgoing circuits. The reactor is used to
+    /// route incoming messages to their appropriate circuit.
+    async fn finish(mut self: Box<Self>) -> Result<(Arc<Channel>, Reactor<S>)>;
+}
+
 /// This indicate what type of channel it is. It allows us to decide for the correct channel cell
 /// state machines and authentication process (if any).
 ///
@@ -152,6 +226,15 @@ pub enum ChannelType {
     },
 }
 
+impl ChannelType {
+    /// Set that this channel type is now authenticated. This only applies to RelayResponder.
+    pub(crate) fn set_authenticated(&mut self) {
+        if let Self::RelayResponder { authenticated } = self {
+            *authenticated = true;
+        }
+    }
+}
+
 /// A channel cell frame used for sending and receiving cells on a channel. The handler takes care
 /// of the cell codec transition depending in which state the channel is.
 ///
