@@ -6,14 +6,14 @@ use futures::stream::StreamExt;
 use tor_cell::chancell::msg::AnyChanMsg;
 use tor_error::internal;
 
-use crate::channel::{ChannelFrame, ChannelType, UniqId, new_frame};
+use crate::channel::{ChannelFrame, UniqId};
 use crate::memquota::ChannelAccount;
 use crate::util::skew::ClockSkew;
 use crate::{Error, Result};
+use safelog::Redacted;
 use tor_cell::chancell::{AnyChanCell, ChanMsg, msg};
 use tor_rtcompat::{CoarseTimeProvider, SleepProvider, StreamOps};
 
-use std::net::SocketAddr;
 use std::sync::Arc;
 use std::time::SystemTime;
 
@@ -26,9 +26,6 @@ use digest::Digest;
 
 use tracing::{debug, instrument, trace};
 
-#[cfg(feature = "relay")]
-use crate::relay::channel::{RelayIdentities, handshake::ChannelAuthenticationData};
-
 /// A list of the link protocols that we support.
 pub(crate) static LINK_PROTOCOLS: &[u16] = &[4, 5];
 
@@ -204,66 +201,15 @@ where
     }
 }
 
-/// A raw client channel on which nothing has been done.
-pub struct ClientInitiatorHandshake<
+/// A base channel on which versions have been negotiated and the relay's handshake has been read,
+/// but where the certs have not been checked.
+///
+/// Both relay and client have specialized objects for an unverified channel which include this one
+/// as the base in order to share functionnalities.
+pub(crate) struct UnverifiedChannel<
     T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
     S: CoarseTimeProvider + SleepProvider,
 > {
-    /// Runtime handle (insofar as we need it)
-    sleep_prov: S,
-
-    /// Memory quota account
-    memquota: ChannelAccount,
-
-    /// Cell encoder/decoder wrapping the underlying TLS stream
-    ///
-    /// (We don't enforce that this is actually TLS, but if it isn't, the
-    /// connection won't be secure.)
-    framed_tls: ChannelFrame<T>,
-
-    /// Declared target method for this channel, if any.
-    target_method: Option<ChannelMethod>,
-
-    /// Logging identifier for this stream.  (Used for logging only.)
-    unique_id: UniqId,
-}
-
-/// Implement the base channel handshake trait.
-impl<T, S> ChannelBaseHandshake<T> for ClientInitiatorHandshake<T, S>
-where
-    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
-    S: CoarseTimeProvider + SleepProvider,
-{
-    fn framed_tls(&mut self) -> &mut ChannelFrame<T> {
-        &mut self.framed_tls
-    }
-    fn unique_id(&self) -> &UniqId {
-        &self.unique_id
-    }
-}
-
-/// Implement the initiator channel handshake trait.
-impl<T, S> ChannelInitiatorHandshake<T> for ClientInitiatorHandshake<T, S>
-where
-    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
-    S: CoarseTimeProvider + SleepProvider,
-{
-    fn is_expecting_auth_challenge(&self) -> bool {
-        // Client never authenticate with a responder, only relay do.
-        false
-    }
-}
-
-/// A client channel on which versions have been negotiated and the
-/// relay's handshake has been read, but where the certs have not
-/// been checked.
-// TODO(relay): Split this into a Client and relay version.
-pub struct UnverifiedChannel<
-    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
-    S: CoarseTimeProvider + SleepProvider,
-> {
-    /// Indicate what type of channel this is.
-    pub(crate) channel_type: ChannelType,
     /// Runtime handle (insofar as we need it)
     pub(crate) sleep_prov: S,
     /// Memory quota account
@@ -272,16 +218,10 @@ pub struct UnverifiedChannel<
     pub(crate) link_protocol: u16,
     /// The Source+Sink on which we're reading and writing cells.
     pub(crate) framed_tls: ChannelFrame<T>,
-    /// The certs cell that we got from the relay.
-    pub(crate) certs_cell: msg::Certs,
+    /// The certs cell that we might have got during the handshake.
+    pub(crate) certs_cell: Option<msg::Certs>,
     /// Declared target method for this channel, if any.
     pub(crate) target_method: Option<ChannelMethod>,
-    /// The netinfo cell that we got from the relay.
-    #[expect(unused)] // TODO(relay): Relays need this.
-    pub(crate) netinfo_cell: msg::Netinfo,
-    /// The AUTH_CHALLENGE cell that we got from the relay. Client ignore this field, only relay
-    /// care for authentication purposes.
-    pub(crate) auth_challenge_cell: Option<msg::AuthChallenge>,
     /// How much clock skew did we detect in this handshake?
     ///
     /// This value is _unauthenticated_, since we have not yet checked whether
@@ -289,127 +229,40 @@ pub struct UnverifiedChannel<
     pub(crate) clock_skew: ClockSkew,
     /// Logging identifier for this stream.  (Used for logging only.)
     pub(crate) unique_id: UniqId,
-    /// Relay only: Our identity keys needed for authentication.
-    #[cfg(feature = "relay")]
-    pub(crate) identities: Option<Arc<RelayIdentities>>,
 }
 
-/// A client channel on which versions have been negotiated,
-/// relay's handshake has been read, but the client has not yet
-/// finished the handshake.
+/// A base channel on which versions have been negotiated, relay's handshake has been read, but the
+/// client has not yet finished the handshake.
 ///
-/// This type is separate from UnverifiedChannel, since finishing the
-/// handshake requires a bunch of CPU, and you might want to do it as
-/// a separate task or after a yield.
-// TODO(relay): Split this into a Client and relay version.
-pub struct VerifiedChannel<
+/// This type is separate from UnverifiedChannel, since finishing the handshake requires a bunch of
+/// CPU, and you might want to do it as a separate task or after a yield.
+///
+/// Both relay and client have specialized objects for an unverified channel which include this one
+/// as the base in order to share functionnalities.
+pub(crate) struct VerifiedChannel<
     T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
     S: CoarseTimeProvider + SleepProvider,
 > {
-    /// Indicate what type of channel this is.
-    channel_type: ChannelType,
     /// Runtime handle (insofar as we need it)
-    sleep_prov: S,
+    pub(crate) sleep_prov: S,
     /// Memory quota account
-    memquota: ChannelAccount,
+    pub(crate) memquota: ChannelAccount,
     /// The negotiated link protocol.
-    link_protocol: u16,
+    pub(crate) link_protocol: u16,
     /// The Source+Sink on which we're reading and writing cells.
-    framed_tls: ChannelFrame<T>,
+    pub(crate) framed_tls: ChannelFrame<T>,
     /// Declared target method for this stream, if any.
-    target_method: Option<ChannelMethod>,
+    pub(crate) target_method: Option<ChannelMethod>,
     /// Logging identifier for this stream.  (Used for logging only.)
-    unique_id: UniqId,
+    pub(crate) unique_id: UniqId,
     /// Validated Ed25519 identity for this peer.
-    ed25519_id: Ed25519Identity,
-    /// Validated RSA identity for this peer.
-    rsa_id: RsaIdentity,
+    pub(crate) ed25519_id: Option<Ed25519Identity>,
+    /// Validated RSA identity and cert digets for this peer.
+    pub(crate) rsa_id_cert_digest: Option<(RsaIdentity, [u8; 32])>,
+    /// Peer TLS certificate digest
+    pub(crate) peer_cert_digest: [u8; 32],
     /// Authenticated clock skew for this peer.
-    clock_skew: ClockSkew,
-    /// Authentication data for the [msg::Authenticate] cell. It is sent during the finalization
-    /// process because the channel needs to be verified before it is sent.
-    #[cfg(feature = "relay")]
-    #[expect(unused)] // TODO(relay): Remove once used.
-    auth_data: Option<ChannelAuthenticationData>,
-}
-
-impl<
-    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
-    S: CoarseTimeProvider + SleepProvider,
-> ClientInitiatorHandshake<T, S>
-{
-    /// Construct a new OutboundClientHandshake.
-    pub(crate) fn new(
-        tls: T,
-        target_method: Option<ChannelMethod>,
-        sleep_prov: S,
-        memquota: ChannelAccount,
-    ) -> Self {
-        Self {
-            framed_tls: new_frame(tls, ChannelType::ClientInitiator),
-            target_method,
-            unique_id: UniqId::new(),
-            sleep_prov,
-            memquota,
-        }
-    }
-
-    /// Negotiate a link protocol version with the relay, and read
-    /// the relay's handshake information.
-    ///
-    /// Takes a function that reports the current time.  In theory, this can just be
-    /// `SystemTime::now()`.
-    #[instrument(skip_all, level = "trace")]
-    pub async fn connect<F>(mut self, now_fn: F) -> Result<UnverifiedChannel<T, S>>
-    where
-        F: FnOnce() -> SystemTime,
-    {
-        match &self.target_method {
-            Some(method) => debug!(
-                stream_id = %self.unique_id,
-                "starting Tor handshake with {:?}",
-                method
-            ),
-            None => debug!(stream_id = %self.unique_id, "starting Tor handshake"),
-        }
-        // Send versions cell.
-        let (versions_flushed_at, versions_flushed_wallclock) =
-            self.send_versions_cell(now_fn).await?;
-
-        // Receive versions cell.
-        let link_protocol = self.recv_versions_cell().await?;
-
-        // Receive the relay responder cells. Ignore the AUTH_CHALLENGE cell, we don't need it as
-        // we are not authenticating with our responder because we are a client.
-        let (_, certs_cell, (netinfo_cell, netinfo_rcvd_at)) =
-            self.recv_cells_from_responder().await?;
-
-        // Get the clock skew.
-        let clock_skew = unauthenticated_clock_skew(
-            &netinfo_cell,
-            netinfo_rcvd_at,
-            versions_flushed_at,
-            versions_flushed_wallclock,
-        );
-
-        trace!(stream_id = %self.unique_id, "received handshake, ready to verify.");
-
-        Ok(UnverifiedChannel {
-            channel_type: ChannelType::ClientInitiator,
-            link_protocol,
-            framed_tls: self.framed_tls,
-            certs_cell,
-            netinfo_cell,
-            auth_challenge_cell: None,
-            clock_skew,
-            target_method: self.target_method.take(),
-            unique_id: self.unique_id,
-            sleep_prov: self.sleep_prov.clone(),
-            memquota: self.memquota.clone(),
-            #[cfg(feature = "relay")]
-            identities: None,
-        })
-    }
+    pub(crate) clock_skew: ClockSkew,
 }
 
 impl<
@@ -417,18 +270,6 @@ impl<
     S: CoarseTimeProvider + SleepProvider,
 > UnverifiedChannel<T, S>
 {
-    /// Return the reported clock skew from this handshake.
-    ///
-    /// Note that the skew reported by this function might not be "true": the
-    /// relay might have its clock set wrong, or it might be lying to us.
-    ///
-    /// The clock skew reported here is not yet authenticated; if you need to
-    /// make sure that the skew is authenticated, use
-    /// [`Channel::clock_skew`](super::Channel::clock_skew) instead.
-    pub fn clock_skew(&self) -> ClockSkew {
-        self.clock_skew
-    }
-
     /// Validate the certificates and keys in the relay's handshake.
     ///
     /// 'peer' is the peer that we want to make sure we're connecting to.
@@ -443,22 +284,22 @@ impl<
     /// This is a separate function because it's likely to be somewhat
     /// CPU-intensive.
     #[instrument(skip_all, level = "trace")]
-    pub fn check<U: ChanTarget + ?Sized>(
+    pub(crate) fn check<U: ChanTarget + ?Sized>(
         self,
         peer: &U,
         peer_cert: &[u8],
         now: Option<std::time::SystemTime>,
     ) -> Result<VerifiedChannel<T, S>> {
-        let peer_cert_sha256 = ll::d::Sha256::digest(peer_cert);
-        self.check_internal(peer, &peer_cert_sha256[..], now)
+        let peer_cert_sha256 = ll::d::Sha256::digest(peer_cert).into();
+        self.check_internal(peer, peer_cert_sha256, now)
     }
 
     /// Same as `check`, but takes the SHA256 hash of the peer certificate,
     /// since that is all we use.
-    fn check_internal<U: ChanTarget + ?Sized>(
-        mut self,
+    pub(crate) fn check_internal<U: ChanTarget + ?Sized>(
+        self,
         peer: &U,
-        peer_cert_sha256: &[u8],
+        peer_cert_digest: [u8; 32],
         now: Option<SystemTime>,
     ) -> Result<VerifiedChannel<T, S>> {
         use tor_cert::CertType;
@@ -505,7 +346,11 @@ impl<
         //    the RSA->Ed25519 crosscert (type 7), which signs...
         //    peer.ed_identity().
 
-        let c = &self.certs_cell;
+        // Evidently, without a CERTS at this point we have a code flow issue.
+        let Some(c) = &self.certs_cell.as_ref() else {
+            return Err(Error::from(internal!("No CERTS cell found to verify")));
+        };
+
         /// Helper: get a cert from a Certs cell, and convert errors appropriately.
         fn get_cert(
             certs: &tor_cell::chancell::msg::Certs,
@@ -559,7 +404,7 @@ impl<
         sigs.push(&sk_tls_sig);
         let (sk_tls_timeliness, sk_tls) = check_timeliness(sk_tls, now, self.clock_skew);
 
-        if peer_cert_sha256 != sk_tls.subject_key().as_bytes() {
+        if peer_cert_digest != sk_tls.subject_key().as_bytes() {
             return Err(Error::HandshakeProto(
                 "Peer cert did not authenticate TLS cert".into(),
             ));
@@ -657,123 +502,49 @@ impl<
         sk_tls_timeliness?;
         rsa_cert_timeliness?;
 
-        // This part is relay specific as only relay will process an AUTH_CHALLENGE message.
-        //
-        // TODO(relay). We should somehow find a way to have this to be in the relay module. For
-        // this, I suspect we will need a relay specific UnverifiedChannel and VerifiedChannel
-        // which yields a Channel upon validation. Client and relay channels would share a lot of
-        // code so we would need to find an elegant way to do this. Until then, it lives here.
-        #[cfg(feature = "relay")]
-        let auth_data: Option<ChannelAuthenticationData>;
-        #[cfg(feature = "relay")]
-        {
-            auth_data = match (self.auth_challenge_cell, self.identities) {
-                (Some(auth_challenge_cell), Some(identities)) => {
-                    // Depending on if we are initiator or responder, we flip the identities in the
-                    // authentication challenge. See tor-spec.
-                    let (cid, sid, cid_ed, sid_ed) = if self.channel_type.is_initiator() {
-                        (
-                            ll::d::Sha256::digest(&identities.cert_id_x509_rsa).into(),
-                            (*rsa_cert.digest()),
-                            &identities.ed_id,
-                            identity_key,
-                        )
-                    } else {
-                        (
-                            (*rsa_cert.digest()),
-                            ll::d::Sha256::digest(&identities.cert_id_x509_rsa).into(),
-                            identity_key,
-                            &identities.ed_id,
-                        )
-                    };
-                    let link_auth = *crate::relay::channel::handshake::LINK_AUTH
-                        .iter()
-                        .filter(|m| auth_challenge_cell.methods().contains(m))
-                        .max()
-                        .ok_or(Error::BadCellAuth)?;
-
-                    let auth_data = ChannelAuthenticationData {
-                        link_auth,
-                        cid,
-                        sid,
-                        cid_ed: cid_ed
-                            .as_bytes()
-                            .try_into()
-                            .expect("ed25519 had an unexpected size"),
-                        sid_ed: sid_ed
-                            .as_bytes()
-                            .try_into()
-                            .expect("ed25519 had an unexpected size"),
-                        clog: self.framed_tls.codec_mut().get_clog_digest()?,
-                        slog: self.framed_tls.codec_mut().get_slog_digest()?,
-                        scert: peer_cert_sha256.try_into().expect("Peer cert not 32 bytes"),
-                    };
-                    Some(auth_data)
-                }
-                _ => None,
-            };
-        }
-
         Ok(VerifiedChannel {
-            channel_type: self.channel_type,
             link_protocol: self.link_protocol,
             framed_tls: self.framed_tls,
             unique_id: self.unique_id,
             target_method: self.target_method,
-            ed25519_id: *identity_key,
-            rsa_id,
+            ed25519_id: Some(*identity_key),
+            rsa_id_cert_digest: Some((rsa_id, *rsa_cert.digest())),
+            peer_cert_digest,
             clock_skew: self.clock_skew,
             sleep_prov: self.sleep_prov,
             memquota: self.memquota,
-            #[cfg(feature = "relay")]
-            auth_data,
         })
     }
-}
 
-impl<
-    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
-    S: CoarseTimeProvider + SleepProvider,
-> VerifiedChannel<T, S>
-{
-    /// Send a 'Netinfo' message to the relay to finish the handshake,
-    /// and create an open channel and reactor.
+    /// Finalize this channel into an actual channel and its reactor.
     ///
-    /// The channel is used to send cells, and to create outgoing circuits.
-    /// The reactor is used to route incoming messages to their appropriate
-    /// circuit.
+    /// An unverified channel can be finalized as it skipped the cert verification and
+    /// authentication because simply the other side is not authenticating.
+    ///
+    /// Two cases for this:
+    ///     - Client <-> Relay channel
+    ///     - Bridge <-> Relay channel
+    ///
+    // NOTE: Unfortunately, this function has duplicated code with the VerifiedChannel::finish()
+    // so make sure any changes here is reflected there. A proper refactoring is welcome!
     #[instrument(skip_all, level = "trace")]
-    pub async fn finish(mut self) -> Result<(Arc<super::Channel>, super::reactor::Reactor<S>)> {
-        // We treat a completed channel -- that is to say, one where the
-        // authentication is finished -- as incoming traffic.
+    pub(crate) fn finish(mut self) -> Result<(Arc<super::Channel>, super::reactor::Reactor<S>)> {
+        // We treat a completed channel as incoming traffic since all cells were exchanged.
         //
         // TODO: conceivably we should remember the time when we _got_ the
         // final cell on the handshake, and update the channel completion
         // time to be no earlier than _that_ timestamp.
+        //
+        // TODO: This shouldn't be here. This should be called in the trait functions that actually
+        // receives the data (recv_*). We'll move it at a later commit.
         crate::note_incoming_traffic();
-        trace!(stream_id = %self.unique_id, "Sending netinfo cell.");
-
-        // We do indeed want the real IP here, regardless of whether the
-        // ChannelMethod is Direct connection or not.  The role of the IP in a
-        // NETINFO cell is to tell our peer what address we believe they had, so
-        // that they can better notice MITM attacks and such.
-        let peer_ip = self
-            .target_method
-            .as_ref()
-            .and_then(ChannelMethod::socket_addrs)
-            .and_then(|addrs| addrs.first())
-            .map(SocketAddr::ip);
-        let netinfo = msg::Netinfo::from_client(peer_ip);
-        self.framed_tls.send(netinfo.into()).await?;
 
         // We have finalized the handshake, move our codec to Open.
         self.framed_tls.codec_mut().set_open()?;
 
-        debug!(
-            stream_id = %self.unique_id,
-            "Completed handshake with {} [{}]",
-            self.ed25519_id, self.rsa_id
-        );
+        // Grab the channel type from our underlying frame as we are about to consume the
+        // framed_tls and we need the channel type to be set into the resulting Channel.
+        let channel_type = self.framed_tls.codec().channel_type();
 
         // Grab a new handle on which we can apply StreamOps (needed for KIST).
         // On Unix platforms, this handle is a wrapper over the fd of the socket.
@@ -792,18 +563,101 @@ impl<
             peer_builder.method(target_method);
         }
         let peer_id = peer_builder
-            .ed_identity(self.ed25519_id)
-            .rsa_identity(self.rsa_id)
             .build()
             .expect("OwnedChanTarget builder failed");
 
-        // TODO(relay): This would be the time to set a "is_canonical" flag to Channel which is
-        // true if the Netinfo address matches the address we are connected to. Canonical
-        // definition is if the address we are connected to is what we expect it to be. This only
-        // makes sense for relay channels.
+        debug!(
+            stream_id = %self.unique_id,
+            "Completed handshake without authentication to {}", Redacted::new(&peer_id)
+        );
 
         super::Channel::new(
-            self.channel_type,
+            channel_type,
+            self.link_protocol,
+            Box::new(tls_sink),
+            Box::new(tls_stream),
+            stream_ops,
+            self.unique_id,
+            peer_id,
+            self.clock_skew,
+            self.sleep_prov,
+            self.memquota,
+        )
+    }
+}
+
+impl<
+    T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
+    S: CoarseTimeProvider + SleepProvider,
+> VerifiedChannel<T, S>
+{
+    /// Mark this channel as authenticated.
+    pub(crate) fn set_authenticated(&mut self) -> Result<()> {
+        self.framed_tls.codec_mut().set_authenticated()?;
+        Ok(())
+    }
+
+    /// The channel is used to send cells, and to create outgoing circuits.
+    /// The reactor is used to route incoming messages to their appropriate
+    /// circuit.
+    #[instrument(skip_all, level = "trace")]
+    pub(crate) async fn finish(
+        mut self,
+    ) -> Result<(Arc<super::Channel>, super::reactor::Reactor<S>)> {
+        // We treat a completed channel -- that is to say, one where the
+        // authentication is finished -- as incoming traffic.
+        //
+        // TODO: conceivably we should remember the time when we _got_ the
+        // final cell on the handshake, and update the channel completion
+        // time to be no earlier than _that_ timestamp.
+        //
+        // TODO: This shouldn't be here. This should be called in the trait functions that actually
+        // receives the data (recv_*). We'll move it at a later commit.
+        crate::note_incoming_traffic();
+
+        // We have finalized the handshake, move our codec to Open.
+        self.framed_tls.codec_mut().set_open()?;
+
+        // Grab the channel type from our underlying frame as we are about to consume the
+        // framed_tls and we need the channel type to be set into the resulting Channel.
+        let channel_type = self.framed_tls.codec().channel_type();
+
+        debug!(
+            stream_id = %self.unique_id,
+            "Completed handshake with {:?} [{:?}]",
+            self.ed25519_id, self.rsa_id_cert_digest
+        );
+
+        // Grab a new handle on which we can apply StreamOps (needed for KIST).
+        // On Unix platforms, this handle is a wrapper over the fd of the socket.
+        //
+        // Note: this is necessary because after `StreamExit::split()`,
+        // we no longer have access to the underlying stream
+        // or its StreamOps implementation.
+        let stream_ops = self.framed_tls.new_handle();
+        let (tls_sink, tls_stream) = self.framed_tls.split();
+
+        let mut peer_builder = OwnedChanTargetBuilder::default();
+        if let Some(target_method) = self.target_method {
+            if let Some(addrs) = target_method.socket_addrs() {
+                peer_builder.addrs(addrs.to_owned());
+            }
+            peer_builder.method(target_method);
+        }
+        // Non authenticating channels won't have these identities. This is possible as a relay
+        // responder handling an incoming channel from a client/bridge.
+        if let Some(ed25519_id) = self.ed25519_id {
+            peer_builder.ed_identity(ed25519_id);
+        }
+        if let Some((rsa_id, _)) = self.rsa_id_cert_digest {
+            peer_builder.rsa_identity(rsa_id);
+        }
+        let peer_id = peer_builder
+            .build()
+            .expect("OwnedChanTarget builder failed");
+
+        super::Channel::new(
+            channel_type,
             self.link_protocol,
             Box::new(tls_sink),
             Box::new(tls_stream),
@@ -850,9 +704,10 @@ pub(super) mod test {
     use std::time::{Duration, SystemTime};
 
     use super::*;
-    use crate::Result;
     use crate::channel::handler::test::MsgBuf;
+    use crate::channel::{ChannelType, new_frame};
     use crate::util::fake_mq;
+    use crate::{Result, channel::ClientInitiatorHandshake};
     use tor_cell::chancell::msg;
     use tor_linkspec::OwnedChanTarget;
     use tor_rtcompat::{PreferredRuntime, Runtime};
@@ -905,7 +760,7 @@ pub(super) mod test {
             let handshake = ClientInitiatorHandshake::new(mb, None, rt.clone(), fake_mq());
             let unverified = handshake.connect(|| now).await?;
 
-            assert_eq!(unverified.link_protocol, 5);
+            assert_eq!(unverified.link_protocol(), 5);
             // No timestamp in the NETINFO, so no skew.
             assert_eq!(unverified.clock_skew(), ClockSkew::None);
 
@@ -1057,26 +912,19 @@ pub(super) mod test {
     where
         R: Runtime,
     {
-        let localhost = std::net::IpAddr::V4(std::net::Ipv4Addr::LOCALHOST);
-        let netinfo_cell = msg::Netinfo::from_client(Some(localhost));
         let mut framed_tls = new_frame(MsgBuf::new(&b""[..]), ChannelType::ClientInitiator);
         let _ = framed_tls.codec_mut().set_link_version(4);
         let _ = framed_tls.codec_mut().set_open();
         let clock_skew = ClockSkew::None;
         UnverifiedChannel {
-            channel_type: ChannelType::ClientInitiator,
             link_protocol: 4,
             framed_tls,
-            certs_cell: certs,
-            netinfo_cell,
-            auth_challenge_cell: None,
+            certs_cell: Some(certs),
             clock_skew,
             target_method: None,
             unique_id: UniqId::new(),
             sleep_prov: runtime,
             memquota: fake_mq(),
-            #[cfg(feature = "relay")]
-            identities: None,
         }
     }
 
@@ -1091,7 +939,7 @@ pub(super) mod test {
         when: Option<SystemTime>,
         peer_ed: &[u8],
         peer_rsa: &[u8],
-        peer_cert_sha256: &[u8],
+        peer_cert_sha256: [u8; 32],
         runtime: &R,
     ) -> Result<VerifiedChannel<MsgBuf, R>>
     where
@@ -1117,7 +965,7 @@ pub(super) mod test {
             None,
             &[0_u8; 32],
             &[0_u8; 20],
-            &[0_u8; 128],
+            [0_u8; 32],
             &rt,
         )
         .err()
@@ -1142,7 +990,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             certs::PEER_ED,
             certs::PEER_RSA,
-            certs::PEER_CERT_DIGEST,
+            *certs::PEER_CERT_DIGEST,
             &rt,
         );
         let _ = res.unwrap();
@@ -1175,7 +1023,7 @@ pub(super) mod test {
                 Some(cert_timestamp()),
                 certs::PEER_ED,
                 certs::PEER_RSA,
-                certs::PEER_CERT_DIGEST,
+                *certs::PEER_CERT_DIGEST,
                 &rt,
             )
             .err()
@@ -1201,7 +1049,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             &[0x10; 32],
             certs::PEER_RSA,
-            certs::PEER_CERT_DIGEST,
+            *certs::PEER_CERT_DIGEST,
             &rt,
         )
         .err()
@@ -1219,7 +1067,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             certs::PEER_ED,
             &[0x99; 20],
-            certs::PEER_CERT_DIGEST,
+            *certs::PEER_CERT_DIGEST,
             &rt,
         )
         .err()
@@ -1237,7 +1085,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             certs::PEER_ED,
             certs::PEER_RSA,
-            &[0; 32],
+            [0; 32],
             &rt,
         )
         .err()
@@ -1267,7 +1115,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             certs::PEER_ED,
             certs::PEER_RSA,
-            certs::PEER_CERT_DIGEST,
+            *certs::PEER_CERT_DIGEST,
             &rt,
         )
         .err()
@@ -1288,7 +1136,7 @@ pub(super) mod test {
             Some(cert_timestamp()),
             certs::PEER_ED,
             certs::PEER_RSA,
-            certs::PEER_CERT_DIGEST,
+            *certs::PEER_CERT_DIGEST,
             &rt,
         )
         .err()
@@ -1323,7 +1171,7 @@ pub(super) mod test {
             "DCB604DB2034B00FD16986D4ADB9D16B21CB4E4457A33DEC0F538903683E96E90006DA3A805CF6006F9179066534DE6B45AD47A5C469063EE462762723396DC9F25452A0A52DA3F5087DD239F2A311F6B0D4DFEFF4ABD089DC3D0237A0ABAB19EB2045B91CDCAF04BE0A72D548A27BF2E77BD876ECFE5E1BE622350DA6BF31F6E306ED896488DD5B39409B23FC3EB7B2C9F7328EB18DA36D54D80575899EA6507CCBFCDF1F"
         );
 
-        pub(crate) const PEER_CERT_DIGEST: &[u8] =
+        pub(crate) const PEER_CERT_DIGEST: &[u8; 32] =
             &hex!("b4fd606b64e4cbd466b8d76cb131069bae6f3aa1878857c9f624e31d77a799b8");
 
         pub(crate) const PEER_ED: &[u8] =
@@ -1334,24 +1182,22 @@ pub(super) mod test {
     #[test]
     fn test_finish() {
         tor_rtcompat::test_with_one_runtime!(|rt| async move {
-            let ed25519_id = [3_u8; 32].into();
+            let ed25519_id = Some([3_u8; 32].into());
             let rsa_id = [4_u8; 20].into();
             let peer_addr = "127.1.1.2:443".parse().unwrap();
             let mut framed_tls = new_frame(MsgBuf::new(&b""[..]), ChannelType::ClientInitiator);
             let _ = framed_tls.codec_mut().set_link_version(4);
             let ver = VerifiedChannel {
-                channel_type: ChannelType::ClientInitiator,
                 link_protocol: 4,
                 framed_tls,
                 unique_id: UniqId::new(),
                 target_method: Some(ChannelMethod::Direct(vec![peer_addr])),
                 ed25519_id,
-                rsa_id,
+                rsa_id_cert_digest: Some((rsa_id, [0; 32])),
+                peer_cert_digest: [0; 32],
                 clock_skew: ClockSkew::None,
                 sleep_prov: rt,
                 memquota: fake_mq(),
-                #[cfg(feature = "relay")]
-                auth_data: None,
             };
 
             let (_chan, _reactor) = ver.finish().await.unwrap();
