@@ -38,7 +38,7 @@ use tor_protover::named;
 use std::num::NonZeroU32;
 use std::pin::Pin;
 use std::result::Result as StdResult;
-use std::sync::{Arc, Mutex, MutexGuard};
+use std::sync::{Arc, Mutex};
 use std::time::Instant;
 
 #[cfg(test)]
@@ -289,11 +289,6 @@ pub(crate) struct SendRelayCell {
 
 /// The inbound state of a hop.
 pub(crate) struct CircHopInbound {
-    /// Congestion control object.
-    ///
-    /// This object is also in charge of handling circuit level SENDME logic for this hop.
-    #[allow(dead_code)] // TODO(relay)
-    ccontrol: Arc<Mutex<CongestionControl>>,
     /// Decodes relay cells received from this hop.
     decoder: RelayCellDecoder,
     /// Remaining permitted incoming relay cells from this hop, plus 1.
@@ -344,24 +339,13 @@ pub(crate) struct CircHopOutbound {
 
 impl CircHopInbound {
     /// Create a new [`CircHopInbound`].
-    pub(crate) fn new(
-        ccontrol: Arc<Mutex<CongestionControl>>,
-        decoder: RelayCellDecoder,
-        settings: &HopSettings,
-    ) -> Self {
+    pub(crate) fn new(decoder: RelayCellDecoder, settings: &HopSettings) -> Self {
         Self {
-            ccontrol,
             decoder,
             n_incoming_cells_permitted: settings.n_incoming_cells_permitted.map(cvt),
         }
     }
 
-    /// Return a mutable reference to our CongestionControl object.
-    #[allow(dead_code)] // TODO(relay)
-    pub(crate) fn ccontrol(&self) -> MutexGuard<'_, CongestionControl> {
-        self.ccontrol.lock().expect("poisoned lock")
-    }
-
     /// Parse a RELAY or RELAY_EARLY cell body.
     ///
     /// Requires that the cryptographic checks on the message have already been
@@ -488,7 +472,12 @@ impl CircHopOutbound {
     ) -> Result<Option<Xon>> {
         // the call below will return an error if XON/XOFF aren't supported,
         // so we check for support here
-        if !self.ccontrol().uses_xon_xoff() {
+        if !self
+            .ccontrol()
+            .lock()
+            .expect("poisoned lock")
+            .uses_xon_xoff()
+        {
             return Ok(None);
         }
 
@@ -507,7 +496,12 @@ impl CircHopOutbound {
     pub(crate) fn maybe_send_xoff(&mut self, id: StreamId) -> Result<Option<Xoff>> {
         // the call below will return an error if XON/XOFF aren't supported,
         // so we check for support here
-        if !self.ccontrol().uses_xon_xoff() {
+        if !self
+            .ccontrol()
+            .lock()
+            .expect("poisoned lock")
+            .uses_xon_xoff()
+        {
             return Ok(None);
         }
 
@@ -531,7 +525,10 @@ impl CircHopOutbound {
     /// Delegate to CongestionControl, for testing purposes
     #[cfg(test)]
     pub(crate) fn send_window_and_expected_tags(&self) -> (u32, Vec<SendmeTag>) {
-        self.ccontrol().send_window_and_expected_tags()
+        self.ccontrol()
+            .lock()
+            .expect("poisoned lock")
+            .send_window_and_expected_tags()
     }
 
     /// Return the number of open streams on this hop.
@@ -542,9 +539,9 @@ impl CircHopOutbound {
         self.map.lock().expect("lock poisoned").n_open_streams()
     }
 
-    /// Return a mutable reference to our CongestionControl object.
-    pub(crate) fn ccontrol(&self) -> MutexGuard<'_, CongestionControl> {
-        self.ccontrol.lock().expect("poisoned lock")
+    /// Return a reference to our CongestionControl object.
+    pub(crate) fn ccontrol(&self) -> &Arc<Mutex<CongestionControl>> {
+        &self.ccontrol
     }
 
     /// We're about to send `msg`.
@@ -615,7 +612,12 @@ impl CircHopOutbound {
         rate_limit_updater: watch::Sender<StreamRateLimit>,
         drain_rate_requester: NotifySender<DrainRateRequest>,
     ) -> Result<StreamFlowCtrl> {
-        if self.ccontrol().uses_stream_sendme() {
+        if self
+            .ccontrol()
+            .lock()
+            .expect("poisoned lock")
+            .uses_stream_sendme()
+        {
             let window = sendme::StreamSendWindow::new(SEND_WINDOW_INIT);
             Ok(StreamFlowCtrl::new_window(window))
         } else {
