@@ -23,6 +23,7 @@ use crate::stream::queue::StreamQueueSender;
 use crate::streammap;
 use crate::util::notify::NotifySender;
 use crate::util::skew::ClockSkew;
+use crate::util::tunnel_activity::TunnelActivity;
 #[cfg(test)]
 use crate::{circuit::UniqId, client::circuit::CircParameters, crypto::cell::HopNum};
 use postage::watch;
@@ -34,8 +35,8 @@ use tor_error::{Bug, bad_api_usage, internal, into_bad_api_usage};
 use tracing::{debug, trace};
 #[cfg(feature = "hs-service")]
 use {
-    super::StreamReqSender, crate::client::reactor::IncomingStreamRequestHandler,
-    crate::client::stream::IncomingStreamRequestFilter,
+    crate::client::reactor::IncomingStreamRequestHandler,
+    crate::client::stream::IncomingStreamRequestFilter, crate::stream::incoming::StreamReqSender,
 };
 
 #[cfg(test)]
@@ -311,6 +312,12 @@ pub(crate) enum CtrlCmd {
         /// A sender to alert after we've changed the padding.
         sender: oneshot::Sender<Result<()>>,
     },
+
+    /// Yield the most active [`TunnelActivity`] for any hop on any leg of this tunnel.
+    GetTunnelActivity {
+        /// A sender to receive the reply.
+        sender: oneshot::Sender<TunnelActivity>,
+    },
 }
 
 /// A flow control update message.
@@ -562,7 +569,7 @@ impl<'a> ControlHandler<'a> {
                         let cell = AnyRelayMsgOuter::new(Some(stream_id), sendme.into());
 
                         let cell = SendRelayCell {
-                            hop: hop_num,
+                            hop: Some(hop_num),
                             early: false,
                             cell,
                         };
@@ -595,7 +602,7 @@ impl<'a> ControlHandler<'a> {
 
                 let cell = AnyRelayMsgOuter::new(None, msg);
                 let cell = SendRelayCell {
-                    hop: hop_num,
+                    hop: Some(hop_num),
                     early: false,
                     cell,
                 };
@@ -692,7 +699,7 @@ impl<'a> ControlHandler<'a> {
                 let handler = IncomingStreamRequestHandler {
                     incoming_sender,
                     cmd_checker,
-                    hop_num,
+                    hop_num: Some(hop_num),
                     filter,
                 };
 
@@ -788,6 +795,11 @@ impl<'a> ControlHandler<'a> {
                 let _ = sender.send(result);
                 Ok(())
             }
+            CtrlCmd::GetTunnelActivity { sender } => {
+                let count = self.reactor.circuits.tunnel_activity();
+                let _ = sender.send(count);
+                Ok(())
+            }
         }
     }
 }
