@@ -32,9 +32,10 @@ use std::sync::{Mutex, Weak};
 
 use educe::Educe;
 
+use crate::client::ClientTunnel;
 use crate::client::stream::StreamReceiver;
-use crate::client::{ClientTunnel, StreamTarget};
 use crate::memquota::StreamAccount;
+use crate::stream::StreamTarget;
 use crate::stream::cmdcheck::{AnyCmdChecker, CmdChecker, StreamStatus};
 use crate::stream::flow_ctrl::state::StreamRateLimit;
 use crate::stream::flow_ctrl::xon_xoff::reader::{BufferIsEmpty, XonXoffReader, XonXoffReaderCtrl};
@@ -551,7 +552,7 @@ impl DataStream {
     /// CONNECTED cell.
     ///
     /// This is used by hidden services, exit relays, and directory servers to accept streams.
-    #[cfg(feature = "hs-service")]
+    #[cfg(any(feature = "hs-service", feature = "relay"))]
     pub(crate) fn new_connected<P: SleepProvider + CoarseTimeProvider>(
         time_provider: P,
         receiver: StreamReceiver,
@@ -592,11 +593,19 @@ impl DataStream {
         };
 
         #[cfg(feature = "stream-ctrl")]
-        let ctrl = Arc::new(ClientDataStreamCtrl {
-            tunnel: Arc::downgrade(target.tunnel()),
-            status: status.clone(),
-            _memquota: memquota.clone(),
-        });
+        let ctrl = {
+            let tunnel = match target.tunnel() {
+                crate::stream::Tunnel::Client(t) => Arc::downgrade(t),
+                #[cfg(feature = "relay")]
+                crate::stream::Tunnel::Relay(_) => panic!("created a relay tunnel in the client?!"),
+            };
+
+            Arc::new(ClientDataStreamCtrl {
+                tunnel,
+                status: status.clone(),
+                _memquota: memquota.clone(),
+            })
+        };
         let r = DataReaderInner {
             state: Some(DataReaderState::Open(DataReaderImpl {
                 s: receiver,
