@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -41,14 +41,19 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 #![allow(non_upper_case_globals)]
 #![allow(clippy::upper_case_acronyms)]
 
+use std::sync::Arc;
+
 use caret::caret_int;
 
 use thiserror::Error;
+use tor_basic_utils::intern::InternCache;
 
 pub mod named;
 
@@ -234,12 +239,26 @@ struct SubprotocolEntry {
 /// use tor_protover::Protocols;
 /// let p: Result<Protocols,_> = "Link=1-3 LinkAuth=2-3 Relay=1-2".parse();
 /// ```
+///
+/// # Implementation notes
+///
+/// Because the number of distinct `Protocols` sets at any given time
+/// is much smaller than the number of relays, this type is interned in order to
+/// save memory and copying time.
+///
+/// This type is an Arc internally; it is cheap to clone.
 #[derive(Debug, Clone, Default, Eq, PartialEq, Hash)]
 #[cfg_attr(
     feature = "serde",
     derive(serde_with::DeserializeFromStr, serde_with::SerializeDisplay)
 )]
-pub struct Protocols {
+pub struct Protocols(Arc<ProtocolsInner>);
+
+/// Inner representation of Protocols.
+///
+/// We make this a separate type so that we can intern it inside an Arc.
+#[derive(Default, Clone, Debug, Eq, PartialEq, Hash)]
+struct ProtocolsInner {
     /// A mapping from protocols' integer encodings to bit-vectors.
     recognized: [u64; N_RECOGNIZED],
     /// A vector of unrecognized protocol versions,
@@ -249,21 +268,42 @@ pub struct Protocols {
     unrecognized: Vec<SubprotocolEntry>,
 }
 
+/// An InternCache of ProtocolsInner.
+///
+/// We intern ProtocolsInner objects because:
+///  - There are very few _distinct_ values in any given set of relays.
+///  - Every relay has one.
+///  - We often want to copy them when we're remembering information about circuits.
+static PROTOCOLS: InternCache<ProtocolsInner> = InternCache::new();
+
+impl From<ProtocolsInner> for Protocols {
+    fn from(value: ProtocolsInner) -> Self {
+        Protocols(PROTOCOLS.intern(value))
+    }
+}
+
 impl Protocols {
     /// Return a new empty set of protocol versions.
+    ///
+    /// # Warning
+    ///
+    /// To the extend possible, avoid using empty lists to represent the capabilities
+    /// of an unknown target.  Instead, if there is a consensus present, use the
+    /// `required-relay-protocols` field of the consensus.
     pub fn new() -> Self {
         Protocols::default()
     }
+
     /// Helper: return true iff this protocol set contains the
     /// version `ver` of the protocol represented by the integer `proto`.
     fn supports_recognized_ver(&self, proto: usize, ver: u8) -> bool {
         if usize::from(ver) > MAX_VER {
             return false;
         }
-        if proto >= self.recognized.len() {
+        if proto >= self.0.recognized.len() {
             return false;
         }
-        (self.recognized[proto] & (1 << ver)) != 0
+        (self.0.recognized[proto] & (1 << ver)) != 0
     }
     /// Helper: return true iff this protocol set contains version
     /// `ver` of the unrecognized protocol represented by the string
@@ -275,6 +315,7 @@ impl Protocols {
             return false;
         }
         let ent = self
+            .0
             .unrecognized
             .iter()
             .find(|ent| ent.proto.is_unrecognized(proto));
@@ -286,8 +327,8 @@ impl Protocols {
 
     /// Return true if this list of protocols is empty.
     pub fn is_empty(&self) -> bool {
-        self.recognized.iter().all(|v| *v == 0)
-            && self.unrecognized.iter().all(|p| p.supported == 0)
+        self.0.recognized.iter().all(|v| *v == 0)
+            && self.0.unrecognized.iter().all(|p| p.supported == 0)
     }
 
     // TODO: Combine these next two functions into one by using a trait.
@@ -359,22 +400,22 @@ impl Protocols {
     ///            "Desc=2,4 Microdesc=1-2,4-5".parse().unwrap());
     /// ```
     pub fn difference(&self, other: &Protocols) -> Protocols {
-        let mut r = Protocols::default();
+        let mut r = ProtocolsInner::default();
 
         for i in 0..N_RECOGNIZED {
-            r.recognized[i] = self.recognized[i] & !other.recognized[i];
+            r.recognized[i] = self.0.recognized[i] & !other.0.recognized[i];
         }
         // This is not super efficient, but we don't have to do it often.
-        for ent in self.unrecognized.iter() {
+        for ent in self.0.unrecognized.iter() {
             let mut ent = ent.clone();
-            if let Some(other_ent) = other.unrecognized.iter().find(|e| e.proto == ent.proto) {
+            if let Some(other_ent) = other.0.unrecognized.iter().find(|e| e.proto == ent.proto) {
                 ent.supported &= !other_ent.supported;
             }
             if ent.supported != 0 {
                 r.unrecognized.push(ent);
             }
         }
-        r
+        Protocols::from(r)
     }
 
     /// Return a Protocols holding every protocol flag that is present in `self`
@@ -388,11 +429,11 @@ impl Protocols {
     ///            "Desc=2-4 Microdesc=1-5,10".parse().unwrap());
     /// ```
     pub fn union(&self, other: &Protocols) -> Protocols {
-        let mut r = self.clone();
+        let mut r = (*self.0).clone();
         for i in 0..N_RECOGNIZED {
-            r.recognized[i] |= other.recognized[i];
+            r.recognized[i] |= other.0.recognized[i];
         }
-        for ent in other.unrecognized.iter() {
+        for ent in other.0.unrecognized.iter() {
             if let Some(my_ent) = r.unrecognized.iter_mut().find(|e| e.proto == ent.proto) {
                 my_ent.supported |= ent.supported;
             } else {
@@ -400,7 +441,7 @@ impl Protocols {
             }
         }
         r.unrecognized.sort();
-        r
+        Protocols::from(r)
     }
 
     /// Return a Protocols holding every protocol flag that is present in both `self`
@@ -414,12 +455,12 @@ impl Protocols {
     ///            "Desc=3".parse().unwrap());
     /// ```
     pub fn intersection(&self, other: &Protocols) -> Protocols {
-        let mut r = Protocols::default();
+        let mut r = ProtocolsInner::default();
         for i in 0..N_RECOGNIZED {
-            r.recognized[i] = self.recognized[i] & other.recognized[i];
+            r.recognized[i] = self.0.recognized[i] & other.0.recognized[i];
         }
-        for ent in self.unrecognized.iter() {
-            if let Some(other_ent) = other.unrecognized.iter().find(|e| e.proto == ent.proto) {
+        for ent in self.0.unrecognized.iter() {
+            if let Some(other_ent) = other.0.unrecognized.iter().find(|e| e.proto == ent.proto) {
                 let supported = ent.supported & other_ent.supported;
                 if supported != 0 {
                     r.unrecognized.push(SubprotocolEntry {
@@ -430,9 +471,11 @@ impl Protocols {
             }
         }
         r.unrecognized.sort();
-        r
+        Protocols::from(r)
     }
+}
 
+impl ProtocolsInner {
     /// Parsing helper: Try to add a new entry `ent` to this set of protocols.
     ///
     /// Uses `foundmask`, a bit mask saying which recognized protocols
@@ -593,7 +636,7 @@ impl std::str::FromStr for Protocols {
     type Err = ParseError;
 
     fn from_str(s: &str) -> Result<Self, ParseError> {
-        let mut result = Protocols::new();
+        let mut result = ProtocolsInner::default();
         let mut foundmask = 0_u64;
         for ent in s.split(' ') {
             if ent.is_empty() {
@@ -604,7 +647,7 @@ impl std::str::FromStr for Protocols {
             result.add(&mut foundmask, s)?;
         }
         result.unrecognized.sort();
-        Ok(result)
+        Ok(result.into())
     }
 }
 
@@ -669,13 +712,13 @@ fn dumpmask(mut mask: u64) -> String {
 impl std::fmt::Display for Protocols {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let mut entries = Vec::new();
-        for (idx, mask) in self.recognized.iter().enumerate() {
+        for (idx, mask) in self.0.recognized.iter().enumerate() {
             if *mask != 0 {
                 let pk: ProtoKind = (idx as u8).into();
                 entries.push(format!("{}={}", pk, dumpmask(*mask)));
             }
         }
-        for ent in &self.unrecognized {
+        for ent in &self.0.unrecognized {
             if ent.supported != 0 {
                 entries.push(format!(
                     "{}={}",
@@ -692,7 +735,7 @@ impl std::fmt::Display for Protocols {
 
 impl FromIterator<NamedSubver> for Protocols {
     fn from_iter<T: IntoIterator<Item = NamedSubver>>(iter: T) -> Self {
-        let mut r = Protocols::new();
+        let mut r = ProtocolsInner::default();
         for named_subver in iter {
             let proto_idx = usize::from(named_subver.kind.get());
             let proto_ver = named_subver.version;
@@ -702,7 +745,7 @@ impl FromIterator<NamedSubver> for Protocols {
             assert!(usize::from(proto_ver) <= MAX_VER);
             r.recognized[proto_idx] |= 1_u64 << proto_ver;
         }
-        r
+        Protocols::from(r)
     }
 }
 
@@ -754,7 +797,7 @@ pub mod doc_supported {}
 /// One concrete example of a very dangerous race that could occur:
 ///
 /// Suppose that the client supports protocols "HsDir=1-2" and the consensus
-/// requires protocols "HsDir=1-2.  If the client supported protocol list is
+/// requires protocols "HsDir=1-2".  If the client supported protocol list is
 /// then changed to "HSDir=2", while the consensus stills lists "HSDir=1-2",
 /// then these clients, even very recent ones, will shut down because they
 /// don't support "HSDir=1".
@@ -773,7 +816,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
