@@ -42,6 +42,7 @@
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
 #![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 pub mod config;
@@ -353,7 +354,7 @@ impl<R: Runtime> tor_chanmgr::factory::AbstractPtMgr for PtMgr<R> {
         };
 
         let proxy = ExternalProxyPlugin::new(self.runtime.clone(), cmethod.endpoint, cmethod.kind);
-        let factory = ChanBuilder::new(self.runtime.clone(), proxy, None);
+        let factory = ChanBuilder::new(self.runtime.clone(), proxy);
         // FIXME(eta): Should we cache constructed factories? If no: should this still be an Arc?
         // FIXME(eta): Should we track what transports are live somehow, so we can shut them down?
         Ok(Some(Arc::new(factory)))
