@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -42,6 +42,7 @@
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
 #![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 mod config;
@@ -78,7 +79,7 @@ pub(crate) mod testing {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -87,7 +88,7 @@ pub(crate) mod testing {
     use std::collections::HashSet;
     use std::sync::LazyLock;
     use tor_netdir::{NetDir, Relay, SubnetConfig};
-    use tor_netdoc::doc::netstatus::RelayFlags;
+    use tor_netdoc::types::relay_flags::RelayFlag;
 
     /// Use a predicate to divide a NetDir into the relays that do and do not
     /// conform (respectively).
@@ -121,10 +122,10 @@ pub(crate) mod testing {
     pub(crate) fn testnet() -> NetDir {
         tor_netdir::testnet::construct_custom_netdir(|idx, node, _| {
             if idx % 7 == 0 {
-                node.rs.clear_flags(RelayFlags::FAST);
+                node.rs.clear_flags(RelayFlag::Fast);
             }
             if idx % 5 == 0 {
-                node.rs.clear_flags(RelayFlags::STABLE);
+                node.rs.clear_flags(RelayFlag::Stable);
             };
         })
         .unwrap()
