@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -41,6 +41,8 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
+#![deny(clippy::unused_async)]
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 // TODO #1645 (either remove this, or decide to have it everywhere)
@@ -59,6 +61,7 @@ mod testing;
 use std::{io, sync::Arc};
 
 pub use connpt::{ParsedConnectPoint, ResolveError, ResolvedConnectPoint};
+use tor_general_addr::general;
 
 /// An action that an RPC client should take when a connect point fails.
 ///
@@ -85,8 +88,8 @@ impl HasClientErrorAction for tor_config_path::CfgPathError {
 }
 impl HasClientErrorAction for tor_config_path::addr::CfgAddrError {
     fn client_action(&self) -> ClientErrorAction {
-        use tor_config_path::addr::CfgAddrError as CAE;
         use ClientErrorAction as A;
+        use tor_config_path::addr::CfgAddrError as CAE;
         match self {
             CAE::NoAfUnixSocketSupport(_) => A::Decline,
             CAE::Path(cfg_path_error) => cfg_path_error.client_action(),
@@ -96,14 +99,31 @@ impl HasClientErrorAction for tor_config_path::addr::CfgAddrError {
         }
     }
 }
+impl HasClientErrorAction for tor_general_addr::general::AddrParseError {
+    fn client_action(&self) -> ClientErrorAction {
+        use ClientErrorAction as A;
+        use tor_general_addr::general::AddrParseError as E;
+        match self {
+            E::UnrecognizedSchema(_) => A::Decline,
+            E::NoSchema => A::Decline,
+            E::InvalidAfUnixAddress(_) => A::Abort,
+            // We might want to turn this into an Abort in the future, but I think that we might
+            // want to allow "auto" as a port format in CfgAddr.
+            E::InvalidInetAddress(_) => A::Decline,
+            // No variants are currently captured in this pattern, but they _could_ be in the future.
+            _ => A::Abort,
+        }
+    }
+}
+
 /// Return the ClientErrorAction for an IO error encountered
 /// while accessing the filesystem.
 ///
 /// Note that this is not an implementation of `HasClientErrorAction`:
 /// We want to decline on a different set of errors for network operation.
 fn fs_error_action(err: &std::io::Error) -> ClientErrorAction {
-    use std::io::ErrorKind as EK;
     use ClientErrorAction as A;
+    use std::io::ErrorKind as EK;
     match err.kind() {
         EK::NotFound => A::Decline,
         EK::PermissionDenied => A::Decline,
@@ -116,8 +136,8 @@ fn fs_error_action(err: &std::io::Error) -> ClientErrorAction {
 /// Note that this is not an implementation of `HasClientErrorAction`:
 /// We want to decline on a different set of errors for fs operation.
 fn net_error_action(err: &std::io::Error) -> ClientErrorAction {
-    use std::io::ErrorKind as EK;
     use ClientErrorAction as A;
+    use std::io::ErrorKind as EK;
     match err.kind() {
         EK::ConnectionRefused => A::Decline,
         EK::ConnectionReset => A::Decline,
@@ -128,8 +148,8 @@ fn net_error_action(err: &std::io::Error) -> ClientErrorAction {
 }
 impl HasClientErrorAction for fs_mistrust::Error {
     fn client_action(&self) -> ClientErrorAction {
-        use fs_mistrust::Error as E;
         use ClientErrorAction as A;
+        use fs_mistrust::Error as E;
         match self {
             E::Multiple(errs) => {
                 if errs.iter().any(|e| e.client_action() == A::Abort) {
@@ -182,10 +202,27 @@ pub enum ConnectError {
     /// Unable to access the location of an AF\_UNIX socket.
     #[error("Unix domain socket path access")]
     AfUnixSocketPathAccess(#[from] fs_mistrust::Error),
+    /// Unable to access the location of `socket_address_file`.
+    #[error("Problem accessing socket address file")]
+    SocketAddressFileAccess(#[source] fs_mistrust::Error),
+    /// We couldn't parse the JSON contents of a socket address file.
+    #[error("Invalid JSON contents in socket address file")]
+    SocketAddressFileJson(#[source] Arc<serde_json::Error>),
+    /// We couldn't parse the address in a socket address file.
+    #[error("Invalid address in socket address file")]
+    SocketAddressFileContent(#[source] general::AddrParseError),
+    /// We found an address in the socket address file that didn't match the connect point.
+    #[error("Socket address file contents didn't match connect point")]
+    SocketAddressFileMismatch,
     /// Another process was holding a lock for this connect point,
     /// so we couldn't bind to it.
     #[error("Could not acquire lock: Another process is listening on this connect point")]
     AlreadyLocked,
+    /// We encountered an internal logic error.
+    //
+    // (We're not using tor_error::Bug here because we want this code to work properly in rpc-client-core.)
+    #[error("Internal error: {0}")]
+    Internal(String),
 }
 
 impl From<io::Error> for ConnectError {
@@ -204,7 +241,12 @@ impl crate::HasClientErrorAction for ConnectError {
             E::UnsupportedSocketType => A::Decline,
             E::UnsupportedAuthType => A::Decline,
             E::AfUnixSocketPathAccess(err) => err.client_action(),
+            E::SocketAddressFileAccess(err) => err.client_action(),
+            E::SocketAddressFileJson(_) => A::Decline,
+            E::SocketAddressFileContent(_) => A::Decline,
+            E::SocketAddressFileMismatch => A::Decline,
             E::AlreadyLocked => A::Abort, // (This one can't actually occur for clients.)
+            E::Internal(_) => A::Abort,
         }
     }
 }
@@ -263,7 +305,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
