@@ -1,10 +1,11 @@
 //! Server operations for working with connect points.
 
-use std::{io, path::PathBuf};
+use std::{io, path::PathBuf, sync::Arc};
 
 use crate::{
-    auth::{Cookie, RpcAuth},
     ConnectError, ResolvedConnectPoint,
+    auth::{RpcAuth, RpcCookieSource, cookie::Cookie},
+    connpt::{AddressFile, ConnectAddress},
 };
 use fs_mistrust::Mistrust;
 use tor_general_addr::general;
@@ -76,18 +77,26 @@ impl crate::connpt::Connect<crate::connpt::Resolved> {
     where
         R: NetStreamProvider<general::SocketAddr, Listener = tor_rtcompat::general::Listener>,
     {
-        if let Some(sock_parent_dir) = crate::socket_parent_path(self.socket.as_ref()) {
-            // TODO RPC: Revisit this and other uses of make_directory; do we really want to do so?
-            mistrust.make_directory(sock_parent_dir)?;
+        // Create parent directory for socket if needed.
+        if let ConnectAddress::Socket(bind_to_socket) = &self.socket {
+            if let Some(sock_parent_dir) = crate::socket_parent_path(bind_to_socket.as_ref()) {
+                mistrust.make_directory(sock_parent_dir)?;
+            }
         }
 
-        let guard = if let Some(socket_path) = self.socket.as_ref().as_pathname() {
+        let af_unix_pathname = if let ConnectAddress::Socket(bind_to_socket) = &self.socket {
+            bind_to_socket.as_ref().as_pathname()
+        } else {
+            None
+        };
+
+        let guard = if let Some(socket_path) = af_unix_pathname {
             // This socket has a representation in the filesystem.
             // We need an associated lock to make sure that we don't delete the socket
             // while it is in use.
             //
             // (We can't just rely on getting an EADDRINUSE when we bind the socket,
-            // since unix sockets give that error unconditionally if the file exists,
+            // since AF_UNIX sockets give that error unconditionally if the file exists,
             // whether anybody has bound to it or not.
             // We can't just check whether the socket file exists,
             // since it might be a stale socket left over from a process that has crashed.
@@ -122,17 +131,35 @@ impl crate::connpt::Connect<crate::connpt::Resolved> {
             }
         };
 
-        let listener = runtime.listen(self.socket.as_ref()).await?;
+        let (listener, chosen_address) = self.socket.bind(runtime).await?;
+
+        if let Some(addr_file) = &self.socket_address_file {
+            let file_contents = serde_json::to_string(&AddressFile {
+                address: chosen_address.clone(),
+            })
+            .expect("Unable to serialize address!");
+
+            mistrust
+                .verifier()
+                .permit_readable()
+                .file_access()
+                .write_and_replace(addr_file, file_contents)
+                .map_err(ConnectError::SocketAddressFileAccess)?;
+        };
 
         // We try to bind to the listener before we (maybe) create the cookie file,
         // so that if we encounter an `EADDRINUSE` we won't overwrite the old cookie file.
         let auth = match &self.auth {
-            crate::connpt::Auth::None => RpcAuth::None,
+            crate::connpt::Auth::None => RpcAuth::Inherent,
             crate::connpt::Auth::Cookie { path } => RpcAuth::Cookie {
-                secret: Cookie::create(path.as_path(), &mut rand::thread_rng(), mistrust)?,
-                server_address: self.socket.as_str().to_owned(),
+                secret: RpcCookieSource::Loaded(Arc::new(Cookie::create(
+                    path.as_path(),
+                    &mut rand::rng(),
+                    mistrust,
+                )?)),
+                server_address: chosen_address,
             },
-            crate::connpt::Auth::Unrecognized {} => return Err(ConnectError::UnsupportedAuthType),
+            crate::connpt::Auth::Unrecognized(_) => return Err(ConnectError::UnsupportedAuthType),
         };
 
         Ok(Listener {
