@@ -10,7 +10,9 @@ macro_rules! implement_opaque_runtime {
 } => {
 
     impl futures::task::Spawn for $t {
+        // `track_caller` required for tokio-console
         #[inline]
+        #[track_caller]
         fn spawn_obj(&self, future: futures::future::FutureObj<'static, ()>) -> Result<(), futures::task::SpawnError> {
             self.$member.spawn_obj(future)
         }
@@ -19,7 +21,9 @@ macro_rules! implement_opaque_runtime {
     impl $crate::traits::Blocking for $t {
         type ThreadHandle<T: Send + 'static> = <$mty as $crate::traits::Blocking>::ThreadHandle<T>;
 
+        // `track_caller` required for tokio-console
         #[inline]
+        #[track_caller]
         fn spawn_blocking<F, T>(&self, f: F) -> <$mty as $crate::traits::Blocking>::ThreadHandle<T>
         where
             F: FnOnce() -> T + Send + 'static,
@@ -28,7 +32,9 @@ macro_rules! implement_opaque_runtime {
             self.$member.spawn_blocking(f)
         }
 
+        // `track_caller` required for tokio-console
         #[inline]
+        #[track_caller]
         fn reenter_block_on<F>(&self, future: F) -> F::Output
         where
             F: futures::Future,
@@ -39,7 +45,9 @@ macro_rules! implement_opaque_runtime {
     }
 
     impl $crate::traits::ToplevelBlockOn for $t {
+        // `track_caller` required for tokio-console
         #[inline]
+        #[track_caller]
         fn block_on<F: futures::Future>(&self, future: F) -> F::Output {
             self.$member.block_on(future)
         }
@@ -66,6 +74,7 @@ macro_rules! implement_opaque_runtime {
         type Stream = <$mty as $crate::traits::NetStreamProvider>::Stream;
         type Listener = <$mty as $crate::traits::NetStreamProvider>::Listener;
         #[inline]
+        #[tracing::instrument(skip_all, level = "trace")]
         async fn connect(&self, addr: &std::net::SocketAddr) -> std::io::Result<Self::Stream> {
             self.$member.connect(addr).await
         }
@@ -79,6 +88,7 @@ macro_rules! implement_opaque_runtime {
         type Stream = <$mty as $crate::traits::NetStreamProvider<tor_general_addr::unix::SocketAddr>>::Stream;
         type Listener = <$mty as $crate::traits::NetStreamProvider<tor_general_addr::unix::SocketAddr>>::Listener;
         #[inline]
+        #[tracing::instrument(skip_all, level = "trace")]
         async fn connect(&self, addr: &tor_general_addr::unix::SocketAddr) -> std::io::Result<Self::Stream> {
             self.$member.connect(addr).await
         }
@@ -93,11 +103,18 @@ macro_rules! implement_opaque_runtime {
     {
         type Connector = <$mty as $crate::traits::TlsProvider<S>>::Connector;
         type TlsStream = <$mty as $crate::traits::TlsProvider<S>>::TlsStream;
+        type Acceptor = <$mty as $crate::traits::TlsProvider<S>>::Acceptor;
+        type TlsServerStream = <$mty as $crate::traits::TlsProvider<S>>::TlsServerStream;
+
         #[inline]
         fn tls_connector(&self) -> Self::Connector {
             self.$member.tls_connector()
         }
         #[inline]
+        fn tls_acceptor(&self, settings: $crate::traits::TlsAcceptorSettings) -> std::io::Result<Self::Acceptor> {
+           <$mty as $crate::traits::TlsProvider<S>>::tls_acceptor(&self.$member, settings)
+        }
+        #[inline]
         fn supports_keying_material_export(&self) -> bool {
             <$mty as $crate::traits::TlsProvider<S>>::supports_keying_material_export(&self.$member)
         }
@@ -120,6 +137,7 @@ macro_rules! implement_opaque_runtime {
     }
 
     // This boilerplate will fail unless $t implements Runtime.
+    #[allow(unused)]
     const _ : () = {
         fn assert_runtime<R: $crate::Runtime>() {}
         fn check() {
