@@ -1,8 +1,9 @@
 //! Declarations for traits that we need our runtimes to implement.
 use async_trait::async_trait;
 use asynchronous_codec::Framed;
+use futures::future::{FutureExt, RemoteHandle};
 use futures::stream;
-use futures::task::Spawn;
+use futures::task::{Spawn, SpawnError};
 use futures::{AsyncRead, AsyncWrite, Future};
 use std::fmt::Debug;
 use std::io::{self, Result as IoResult};
@@ -10,12 +11,15 @@ use std::net;
 use std::time::{Duration, Instant, SystemTime};
 use tor_general_addr::unix;
 
+#[cfg(feature = "tls-server")]
+use tor_cert_x509::TlsKeyAndCert;
+
 /// A runtime for use by Tor client library code.
 ///
 /// This trait comprises several other traits that we require all of our
 /// runtimes to provide:
 ///
-/// * [`futures::task::Spawn`] to launch new background tasks.
+/// * [`futures::task::Spawn`] or [`SpawnExt`] to launch new background tasks.
 /// * [`SleepProvider`] to pause a task for a given amount of time.
 /// * [`CoarseTimeProvider`] for a cheaper but less accurate notion of time.
 /// * [`NetStreamProvider`] to launch and accept network connections.
@@ -392,6 +396,47 @@ pub trait Blocking: Clone + Send + Sync + 'static {
     }
 }
 
+/// Extension trait for [`Spawn`].
+///
+/// This is very similar to, and preferred over, [`futures::task::SpawnExt`].
+/// Unlike `futures::task::SpawnExt`, it is compatible with tokio-console,
+/// and preserves span information for `tracing`.
+// If https://github.com/rust-lang/futures-rs/issues/2977 is ever addressed,
+// we can consider transitioning back to `futures::task::SpawnExt`.
+pub trait SpawnExt: Spawn {
+    /// Spawns a task that polls the given future with output `()` to completion.
+    ///
+    /// See [`futures::task::SpawnExt::spawn`].
+    #[track_caller]
+    fn spawn<Fut>(&self, future: Fut) -> Result<(), SpawnError>
+    where
+        Fut: Future<Output = ()> + Send + 'static,
+    {
+        use tracing::Instrument as _;
+        self.spawn_obj(Box::new(future.in_current_span()).into())
+    }
+
+    /// Spawns a task that polls the given future to completion and returns a future that resolves
+    /// to the spawned future’s output.
+    ///
+    /// See [`futures::task::SpawnExt::spawn_with_handle`].
+    #[track_caller]
+    fn spawn_with_handle<Fut>(
+        &self,
+        future: Fut,
+    ) -> Result<RemoteHandle<<Fut as Future>::Output>, SpawnError>
+    where
+        Fut: Future + Send + 'static,
+        <Fut as Future>::Output: Send,
+    {
+        let (future, handle) = future.remote_handle();
+        self.spawn(future)?;
+        Ok(handle)
+    }
+}
+
+impl<T: Spawn> SpawnExt for T {}
+
 /// Trait providing additional operations on network sockets.
 pub trait StreamOps {
     /// Set the [`TCP_NOTSENT_LOWAT`] socket option, if this `Stream` is a TCP stream.
@@ -560,6 +605,18 @@ pub trait CertifiedConn {
     /// Try to return the (DER-encoded) peer certificate for this
     /// connection, if any.
     fn peer_certificate(&self) -> IoResult<Option<Vec<u8>>>;
+
+    /// Try to return the (DER-encoded) link certificate (if any) containing
+    /// the key we used to authenticate this connection.
+    ///
+    /// Ordinarily, this will return a certificate for server connections,
+    /// and None for client connections.
+    //
+    // NOTE: (The correct return value in the _absence_ of a certificate is None.
+    // Later, if we support optional certificates for clients,
+    // the place to return an Unsupported error would be
+    // from whatever function tries to set such a certificate.)
+    fn own_certificate(&self) -> IoResult<Option<Vec<u8>>>;
 }
 
 /// An object that knows how to wrap a TCP connection (where the type of said TCP
@@ -595,11 +652,14 @@ pub trait TlsConnector<S> {
 
     /// Start a TLS session over the provided TCP stream `stream`.
     ///
-    /// Declare `sni_hostname` as the desired hostname, but don't actually check
+    /// For a client connection,
+    /// declare `sni_hostname` as the desired hostname, but don't actually check
     /// whether the hostname in the certificate matches it.  The connector may
     /// send `sni_hostname` as part of its handshake, if it supports
     /// [SNI](https://en.wikipedia.org/wiki/Server_Name_Indication) or one of
     /// the TLS 1.3 equivalents.
+    ///
+    /// (For a server connection, `sni_hostname` is ignored.)
     async fn negotiate_unvalidated(&self, stream: S, sni_hostname: &str) -> IoResult<Self::Conn>;
 }
 
@@ -620,9 +680,82 @@ pub trait TlsProvider<S: StreamOps>: Clone + Send + Sync + 'static {
     /// The type of the stream returned by that connector.
     type TlsStream: AsyncRead + AsyncWrite + StreamOps + CertifiedConn + Unpin + Send + 'static;
 
+    /// The Acceptor object that this provider can return, for handling incoming connections.
+    type Acceptor: TlsConnector<S, Conn = Self::TlsServerStream> + Send + Sync + Unpin;
+
+    /// The type of stream returned by that Acceptor.
+    type TlsServerStream: AsyncRead
+        + AsyncWrite
+        + StreamOps
+        + CertifiedConn
+        + Unpin
+        + Send
+        + 'static;
+
     /// Return a TLS connector for use with this runtime.
     fn tls_connector(&self) -> Self::Connector;
 
+    /// Return a TLS acceptor for use with this runtime.
+    ///
+    /// Not every [`TlsProvider`] supports this method.
+    /// For those that do, this method is only supported
+    /// when `tor-rtcompat` is built with the `tls-server` feature.
+    /// When this method is unsupported, it returns an error.
+    fn tls_acceptor(&self, settings: TlsAcceptorSettings) -> IoResult<Self::Acceptor>;
+
     /// Return true iff the keying material exporters (RFC 5705) is supported.
     fn supports_keying_material_export(&self) -> bool;
 }
+
+/// Settings used for constructing a TlsAcceptor.
+#[derive(Debug, Clone)]
+#[non_exhaustive]
+pub struct TlsAcceptorSettings {
+    /// The certificates and keys for this acceptor.
+    #[cfg(feature = "tls-server")]
+    pub(crate) identity: TlsKeyAndCert,
+
+    #[cfg(not(feature = "tls-server"))]
+    unconstructable: void::Void,
+    //
+    // TODO: Add support for additional certificates in a chain.
+    // TODO: Possibly, add support for PEM.
+}
+
+impl TlsAcceptorSettings {
+    /// Create a new TlsAcceptorSettings from a certificate and its associated private key,
+    /// both in DER format.
+    ///
+    /// Does not perform full (or even, necessarily, any) validation.
+    //
+    // TODO: It would be great to take a tor_cert::x509::TlsKeyAndCert instead,
+    // but that would (apparently) introduce a dependency cycle.  It would be cool to figure out how
+    // to invert that.
+    #[allow(clippy::unnecessary_wraps)]
+    #[cfg(feature = "tls-server")]
+    pub fn new(identity: TlsKeyAndCert) -> std::io::Result<Self> {
+        Ok(Self { identity })
+    }
+
+    /// Return the primary certificate for this [`TlsAcceptorSettings`], in DER format.
+    pub fn cert_der(&self) -> &[u8] {
+        #[cfg(not(feature = "tls-server"))]
+        {
+            void::unreachable(self.unconstructable);
+        }
+        #[cfg(feature = "tls-server")]
+        self.identity.certificates_der()[0]
+    }
+}
+
+/// An error returned by TlsProvider::tls_acceptor when the TlsProvider does not have TLS server support.
+#[derive(Clone, Debug, thiserror::Error)]
+#[non_exhaustive]
+#[error("This TlsProvider does not support running as a server")]
+pub struct TlsServerUnsupported {}
+
+impl From<TlsServerUnsupported> for io::Error {
+    fn from(value: TlsServerUnsupported) -> Self {
+        io::Error::new(io::ErrorKind::Unsupported, value)
+    }
+}
