@@ -28,7 +28,7 @@ fn main() -> anyhow::Result<()> {
         .context("Failed to read body.")?;
 
     // Will output if request was made using Tor.
-    println!("Response: {}", response);
+    println!("Response: {response}");
 
     Ok(())
 }
