@@ -4,9 +4,12 @@ Type-based wrappers around our FFI functions.
 These types are responsible for providing a python-like API
 to the Arti RPC library.
 
-TODO RPC: NOTE that these APIs are still in flux;
-we will break them a lot before we declare them stable.
-Don't use them in production.
+Note: As of Jan 2025, this library, and the Arti RPC system,
+are still under active development.
+We believe that they are ready to try out, but it is likely
+that they still have bugs and design flaws that we'll need to fix.
+Please be ready to report issues at
+<https://gitlab.torproject.org/tpo/core/arti>.
 """
 
 # mypy: allow-redefinition
@@ -27,7 +30,7 @@ import logging
 import os
 import socket
 import sys
-from ctypes import POINTER, byref, c_int, _Pointer as Ptr
+from ctypes import POINTER, byref, _Pointer as Ptr
 from enum import Enum
 import arti_rpc.ffi
 from arti_rpc.ffi import (
@@ -37,6 +40,7 @@ from arti_rpc.ffi import (
     ArtiRpcConn as FfiConn,
     ArtiRpcConnBuilder as FfiBuilder,
     _ArtiRpcStatus as FfiStatus,
+    _ArtiRpcRawSocket as FfiSocket,
 )
 from typing import (
     Optional,
@@ -207,6 +211,7 @@ class ArtiRpcConn(_RpcBase):
 
     _conn: Optional[Ptr[FfiConn]]
     _session: ArtiRpcObject
+    _conn_object: ArtiRpcObject
 
     def __init__(self, rpc_lib=None, _conn: Optional[Ptr[FfiConn]] = None):
         """
@@ -229,6 +234,7 @@ class ArtiRpcConn(_RpcBase):
         self._conn = _conn
         s = self._rpc.arti_rpc_conn_get_session_id(self._conn).decode("utf-8")
         self._session = self.make_object(s)
+        self._conn_object = self.make_object("connection")
 
     def __del__(self):
         if self._conn is not None:
@@ -245,6 +251,15 @@ class ArtiRpcConn(_RpcBase):
         """
         return ArtiRpcObject(object_id, self)
 
+    def connection(self) -> ArtiRpcObject:
+        """
+        Return an ArtiRpcObject for this connection itself.
+
+        The connection object is used to cancel and otherwise manipulate
+        RPC requests.
+        """
+        return self._conn_object
+
     def session(self) -> ArtiRpcObject:
         """
         Return an ArtiRpcObject for this connection's Session object.
@@ -294,7 +309,7 @@ class ArtiRpcConn(_RpcBase):
             self._conn, msg.encode("utf-8"), byref(handle), byref(error)
         )
         self._handle_error(rv, error)
-        return ArtiRequestHandle(handle, self._rpc)
+        return ArtiRequestHandle(handle, self, self._rpc)
 
     def open_stream(
         self,
@@ -319,8 +334,20 @@ class ArtiRpcConn(_RpcBase):
         If `want_stream_id` is true, then we register the resulting data stream
         as an RPC object, and return it along with the resulting socket.
 
-        Caveats: TODO RPC.  Copy-paste the caveats from arti-rpc-client-core,
-        once they have stabilized.
+        # Important caveats
+
+        When possible, use a hostname rather than an IP address.
+        If you *must* use an IP address, make sure that you have not gotten it
+        by a non-anonymous DNS lookup.
+        (Calling `gethostname()` or `getaddrinfo()` directly
+        would lose anonymity: they inform the user's DNS server,
+        and possibly many other parties, about the target address
+        you are trying to visit.)
+
+        The resulting socket will actually be a TCP connection to Arti,
+        not directly to your destination.
+        Therefore, passing it to functions like `getpeername()`
+        may give unexpected results.
         """
         hostname: bytes = hostname.encode("utf-8")
         isolation: bytes = isolation.encode("utf-8")
@@ -330,7 +357,7 @@ class ArtiRpcConn(_RpcBase):
             stream_id_ptr = byref(stream_id)
         else:
             stream_id_ptr = None
-        sock_cint = c_int(arti_rpc.ffi.INVALID_SOCKET)
+        sock_cint = FfiSocket(arti_rpc.ffi.INVALID_SOCKET)
         error = POINTER(arti_rpc.ffi.ArtiRpcError)()
 
         rv = self._rpc.arti_rpc_conn_open_stream(
@@ -468,7 +495,7 @@ class ArtiRpcError(Exception):
 
 
 def _opt_object_id_to_bytes(
-    object_id: Union[ArtiRpcObject, str, None]
+    object_id: Union[ArtiRpcObject, str, None],
 ) -> Optional[bytes]:
     """
     Convert `object_id` (if it is present) to a `bytes`.
@@ -531,6 +558,9 @@ class ArtiRpcObject(_RpcBase):
         Return a helper that can be used to set meta-parameters
         on a request made with this object.
 
+        Currently recognized meta-parameters are "updates"
+        and "require": See rpc-meta-draft.md for more information.
+
         The wrapper will support `invoke` and `invoke_with_handle`,
         and will pass them any provided `params` given as an argument
         to this function as meta-request parameters.
@@ -651,12 +681,19 @@ class ArtiRpcResponse:
 class ArtiRequestHandle(_RpcBase):
     """
     Handle to a pending RPC request.
+
+    NOTE: Dropping this handle does not cancel the request.
+    If you want to cancel the request on the server side,
+    use the cancel method.
     """
 
     _handle: Ptr[FfiHandle]
+    _conn: ArtiRpcConn
+    _id: str
 
-    def __init__(self, handle: Ptr[FfiHandle], rpc):
+    def __init__(self, handle: Ptr[FfiHandle], conn: ArtiRpcConn, rpc):
         _RpcBase.__init__(self, rpc)
+        self._conn = conn
         self._handle = handle
 
     def __del__(self):
@@ -682,3 +719,17 @@ class ArtiRequestHandle(_RpcBase):
         expected_kind = ArtiRpcResponseKind(responsetype.value)
         assert response_obj.kind() == expected_kind
         return response_obj
+
+    def cancel(self):
+        """
+        Attempt to cancel this request.
+
+        This can fail if the request has already stopped,
+        or if it stops before we have a chance to cancel it.
+        """
+        error = POINTER(arti_rpc.ffi.ArtiRpcError)()
+        rv = self._rpc.arti_rpc_conn_cancel_handle(
+            self._conn._conn, self._handle, byref(error)
+        )
+
+        self._handle_error(rv, error)
